\ start by loading the constants
load ../../dataDictionary/common/const.4th

\ strings and arrays
: place 2dup 2>r char+ swap move 2r> c! ; 
: erased here over erase allot ;
: string create erased ;
: array create cells erased ;
: no-op ;
: strlen c@ ;
: c+! dup >r c@ + r> c! ;

: +place ( str len add2 -- )
 2dup 2>r count + swap move 2r> c+! ;

: c+place ( char addr -- )
 dup >r count dup 1+ r> c! + c! ;

\ copyToHeap is usually done more specifically with the ObjectAsCell primitive
: copyToHeap ( buf count -- heapstr )
	 dup 4 + BufferAlloc    \ include extra space for length field
         dup >r place r> ;      \ copy in str and leave heap pointer on stack

: 3dup dup 2over rot ;

\ initialize constants - most of these may not be needed any more...

256 string CONFIGDIR       s" ../../private" CONFIGDIR place
256 string GLOBALCONFIGDIR s" ../../etc/config" GLOBALCONFIGDIR place
256 string PROTOCOLGROUPS  s" protocolGroups.txt" PROTOCOLGROUPS place
256 string GEOIP           s" GeoIP.csv" GEOIP place
256 string CONFIGFILE      s" config.ini" CONFIGFILE place
256 string STATEDIR        s" ../../state" STATEDIR place
256 string AGENTSFILE      s" agents.txt" AGENTSFILE place
256 string INTERFACESFILE  s" interfaces.txt" INTERFACESFILE place
256 string ACTIVEDIR       s" ../../data/activeData" ACTIVEDIR place
256 string HISTORYDIR      s" ../../data/historyData" HISTORYDIR place
256 string NFPDIR          s" ../../data/nfpData" NFPDIR place
256 string ACTIVEFLOWS     s" activeFlows" ACTIVEFLOWS place
256 string ACTIVESTREAMS   s" activeStreams" ACTIVESTREAMS place
256 string ACTIVECOUNTERS  s" activeCounters" ACTIVECOUNTERS place
256 string HISTORYFLOWS    s" historyFlows" HISTORYFLOWS place
256 string HISTORYSTREAMS  s" historyStreams" HISTORYSTREAMS place
256 string HISTORYCOUNTERS s" historyCounters" HISTORYCOUNTERS place
256 string HISTORYCOUNTERS2 s" historyCounters2" HISTORYCOUNTERS2 place
256 string HISTORYTRMX     s" historyTrmx" HISTORYTRMX place
256 string NFPFLOWS        s" nfpFlows" NFPFLOWS place
256 string EXTENSIONFILE   s" ../../dataDictionary/common/common2.4th" EXTENSIONFILE place

\ shadow fields - should really be read from fields.h
ft_file_seconds ft_intervalStart FieldTypeSetShadow

\ =======================================================================

: testParser
	s" 10.0.0.251" ot_IP ObjectParse ObjectPrint type cr ;


: parseField ObjectParse ObjectPrint type cr ;

\ =======================================================================

60 constant minute
3600 constant hour
86400 constant day
604800 constant week

: asSeconds ot_UnixSeconds ObjectParse 2drop dup 0= IF ELSE q@ THEN ;
: now s" NOW" asSeconds ;
: start over swap mod - ;
: startOfHour now hour start ;
: startOfDay now day start ;
: startOfLastHour startOfHour hour - ;

\ =======================================================================

: dumpHistoryTrmx >r HISTORYDIR count HISTORYTRMX count r> FileNew FileDump ;
: dumpHistoryFlows >r HISTORYDIR count HISTORYFLOWS count r> FileNew FileDump ;
: dumpActiveFlows >r ACTIVEDIR count ACTIVEFLOWS count r> FileNew FileDump ;
: dumpActiveCounters >r ACTIVEDIR count ACTIVECOUNTERS count r> FileNew FileDump ;
: dumpHistoryCounters >r HISTORYDIR count HISTORYCOUNTERS count r> FileNew FileDump ;
: dumpHistoryCounters2 >r HISTORYDIR count HISTORYCOUNTERS2 count r> FileNew FileDump ;

: dumpHistoryTrmxStats >r HISTORYDIR count HISTORYTRMX count r> FileNew FileDumpStats ;
: dumpHistoryFlowsStats >r HISTORYDIR count HISTORYFLOWS count r> FileNew FileDumpStats ;
: dumpActiveFlowsStats >r ACTIVEDIR count ACTIVEFLOWS count r> FileNew FileDumpStats ;
: dumpActiveCountersStats >r ACTIVEDIR count ACTIVECOUNTERS count r> FileNew FileDumpStats ;
: dumpHistoryCountersStats >r HISTORYDIR count HISTORYCOUNTERS count r> FileNew FileDumpStats ;
: dumpHistoryCounters2Stats >r HISTORYDIR count HISTORYCOUNTERS2 count r> FileNew FileDumpStats ;

\ =======================================================================

: nullString 0 0 ; \ length and pointer
: prefix s" data>" ;
: dumpPrefix s" dump>" ;
: delim s" ," ;
: nullTime 0 ;

0 constant nullCommentChar
59 constant semicolonCommentChar

: printTable { table tstamp flags | -- } table tstamp flags prefix delim nullString TablePrint ;
: dumpTable { table tstamp flags | -- } table tstamp flags dumpPrefix delim nullString TablePrint ;
: .table { table | -- } table nullTime 0 printTable ;
: .table_with_hdr { table | -- } table nullTime 1 printTable ;
: .table_with_time { table tstamp | -- } table tstamp 6 printTable ; \ seconds=2, time=4
: .table_dump { table | -- } table nullTime 9 dumpTable ; \ includeHidden=8, includeHeader=1
: .table_or_emptyRow { table | -- } table nullTime 16 printTable ; \ includeEmptyRow=16


\ ============= testing address fields ====================================

: MACBroadcast  [ s" FFFFFFFFFFFF" ot_MAC ObjectParse 2drop ] LITERAL ;  \ objectParse returns ot,len,data, so drop ot and len

\ filter expressions - pointer to MAC address is on stack
: isMACBroadcast DUP IF 6 MACBroadcast 6 compare 0= THEN ;   \ compare is (ptr1 len ptr2 len -- bool)
	
 \ MAC multicast if lsb of first byte is set and it is not a broadcast
: isMACMulticast DUP IF DUP c@ 1 AND IF isMACBroadcast 0= ELSE DROP 0 THEN THEN ;

 \ MAC unicast if lsb of first byte is not set
: isMACUnicast DUP IF c@ 1 AND 0= THEN ;

\ IP multicast if first byte AND 224 == 244
: isIPMulticast DUP IF c@ 224 AND 224 = THEN ;

\ IP6 multicast if first byte is 0xFF
: isIP6Multicast DUP IF c@ 255 = THEN ;

\ ================== testing flow flags =======================================

\ testing decodeFlags - no need to test for NULL input, but make sure output is true or false
: isUnicast  q@ [ flags_L2BROADCAST flags_L2MULTICAST flags_L3MULTICAST OR OR ] LITERAL AND 0= ;
: isL2Multicast q@ flags_L2MULTICAST AND 0> ;
: isL2Broadcast q@ flags_L2BROADCAST AND 0> ;
: isL3Multicast q@ flags_L3MULTICAST AND 0> ;
: hasL3         q@ flags_HASL3       AND 0> ;
: wasDiscarded  q@ flags_DISCARDED   AND 0> ;
: isRTP         q@ flags_RTP         AND 0> ;
: isL2Tunnel    q@ flags_L2TUNNEL    AND 0> ;
 
\ now add extended colums for these so they can be used as simple filters, and also so that
\ fractions of traffic can be ascentained just by taking the mean

: valueField st_UNKNOWN pt_UNKNOWN kl_VALUE fc_VALUE ;
: int32ValueField ot_INT32 valueField ;

: ft_isUnicast     ( -- ft ) [ int32ValueField  s" ft_isUnicast" ft_flags 1 ' isUnicast calc_BOOLEAN CalcNew FieldTypeDefine ] LITERAL ;
: ft_isL2Broadcast ( -- ft ) [ int32ValueField s" ft_isL2Broadcast" ft_flags 1 ' isL2Broadcast calc_BOOLEAN CalcNew FieldTypeDefine ] LITERAL ;
: ft_isL2Multicast ( -- ft ) [ int32ValueField s" ft_isL2Multicast" ft_flags 1 ' isL2Multicast calc_BOOLEAN CalcNew FieldTypeDefine ] LITERAL ;
: ft_isL3Multicast ( -- ft ) [ int32ValueField s" ft_isL3Multicast" ft_flags 1 ' isL3Multicast calc_BOOLEAN CalcNew FieldTypeDefine ] LITERAL ;
: ft_hasL3         ( -- ft ) [ int32ValueField s" ft_hasL3" ft_flags 1 ' hasL3 calc_BOOLEAN CalcNew FieldTypeDefine ] LITERAL ;
: ft_wasDiscarded  ( -- ft ) [ int32ValueField s" ft_wasDiscarded" ft_flags 1 ' wasDiscarded calc_BOOLEAN CalcNew FieldTypeDefine ] LITERAL ;
: ft_isRTP         ( -- ft ) [ int32ValueField s" ft_isRTP" ft_flags 1 ' isRTP calc_BOOLEAN CalcNew FieldTypeDefine ] LITERAL ;
: ft_isL2Tunnel    ( -- ft ) [ int32ValueField s" ft_isRTP" ft_flags 1 ' isL2Tunnel calc_BOOLEAN CalcNew FieldTypeDefine ] LITERAL ;

\ ======= similar pattern for the hostFlags ============

: isHostMacSrc  q@ hostFlags_MACSRC AND 0> ;
: isHostMacDst  q@ hostFlags_MACDST AND 0> ;

: ft_isHostMacSrc ( -- ft ) [ int32ValueField  s" ft_isHostMacSrc" ft_hostFlags 1 ' isHostMacSrc calc_BOOLEAN CalcNew FieldTypeDefine ] LITERAL ;
: ft_isHostMacDst ( -- ft ) [ int32ValueField  s" ft_isHostMacDst" ft_hostFlags 1 ' isHostMacDst calc_BOOLEAN CalcNew FieldTypeDefine ] LITERAL ;

\ ======= and for host-counter-set flags (except that they are 64-bit fields) ============

: hasCounters_IF        @ 0x0000001 AND 0> ; \ SA_DSFLAGS_C_GENERIC
: hasCounters_SWITCH    @ 0x0000200 AND 0> ; \ SA_DSFLAGS_C_PROCESSOR
: hasCounters_HOST      @ 0x00005C0 AND 0> ; \ SA_DSFLAGS_C_HOST*
: hasCounters_VM        @ 0x001E000 AND 0> ; \ SA_DSFLAGS_C_HOST_VRT*
: hasCounters_HTTP      @ 0x0080000 AND 0> ; \ SA_DSFLAGS_C_HTTP
: hasCounters_MEMCACHE  @ 0x0040000 AND 0> ; \ SA_DSFLAGS_C_MEMCACHE
: hasCounters_JAVA      @ 0x0200000 AND 0> ; \ SA_DSFLAGS_C_JMX
: hasCounters_APP       @ 0x0400000 AND 0> ; \ SA_DSFLAGS_C_APP
: hasCounters_VDI       @ 0x2000000 AND 0> ; \ SA_DSFLAGS_C_VDI


: ft_hasCounters_IF ( -- ft )       [ int32ValueField  s" ft_hasCounters_IF"       ft_DSFlags_Ctr 1 ' hasCounters_IF       calc_BOOLEAN CalcNew FieldTypeDefine ] LITERAL ;
: ft_hasCounters_SWITCH ( -- ft )   [ int32ValueField  s" ft_hasCounters_SWITCH"   ft_DSFlags_Ctr 1 ' hasCounters_SWITCH   calc_BOOLEAN CalcNew FieldTypeDefine ] LITERAL ;
: ft_hasCounters_HOST ( -- ft )     [ int32ValueField  s" ft_hasCounters_HOST"     ft_DSFlags_Ctr 1 ' hasCounters_HOST     calc_BOOLEAN CalcNew FieldTypeDefine ] LITERAL ;
: ft_hasCounters_VM ( -- ft )       [ int32ValueField  s" ft_hasCounters_VM"       ft_DSFlags_Ctr 1 ' hasCounters_VM       calc_BOOLEAN CalcNew FieldTypeDefine ] LITERAL ;
: ft_hasCounters_HTTP ( -- ft )     [ int32ValueField  s" ft_hasCounters_HTTP"     ft_DSFlags_Ctr 1 ' hasCounters_HTTP     calc_BOOLEAN CalcNew FieldTypeDefine ] LITERAL ;
: ft_hasCounters_MEMCACHE ( -- ft ) [ int32ValueField  s" ft_hasCounters_MEMCACHE" ft_DSFlags_Ctr 1 ' hasCounters_MEMCACHE calc_BOOLEAN CalcNew FieldTypeDefine ] LITERAL ;
: ft_hasCounters_JAVA ( -- ft )     [ int32ValueField  s" ft_hasCounters_JAVA"     ft_DSFlags_Ctr 1 ' hasCounters_JAVA     calc_BOOLEAN CalcNew FieldTypeDefine ] LITERAL ;
: ft_hasCounters_APP ( -- ft )      [ int32ValueField  s" ft_hasCounters_APP"      ft_DSFlags_Ctr 1 ' hasCounters_APP      calc_BOOLEAN CalcNew FieldTypeDefine ] LITERAL ;
: ft_hasCounters_VDI ( -- ft )      [ int32ValueField  s" ft_hasCounters_VDI"      ft_DSFlags_Ctr 1 ' hasCounters_VDI      calc_BOOLEAN CalcNew FieldTypeDefine ] LITERAL ;

\ ======= we keep datasource flags for flow data too  ============

: hasFlows_SWITCH     @ 0x0000008 AND 0> ; \ SA_DSFLAGS_SWITCH
: hasFlows_ROUTER     @ 0x0000010 AND 0> ; \ SA_DSFLAGS_ROUTER

: ft_hasFlows_SWITCH ( -- ft )    [ int32ValueField  s" ft_hasFlows_SWITCH"    ft_DSFlags64 1 ' hasFlows_SWITCH   calc_BOOLEAN CalcNew FieldTypeDefine ] LITERAL ;
: ft_hasFlows_ROUTER ( -- ft )    [ int32ValueField  s" ft_hasFlows_ROUTER"    ft_DSFlags64 1 ' hasFlows_ROUTER   calc_BOOLEAN CalcNew FieldTypeDefine ] LITERAL ;

\ ====================== MAC vendor codes ===========================================

: vendorField ot_VENDOR st_UNKNOWN pt_MAC kl_LAYER2 ;
: srcVendorField vendorField fc_SRC ;
: dstVendorField vendorField fc_DST ;

: vendorCode ( mac -- code ) ; \ no-op - just point to the bytes at the start of the mac address
: ft_MAC_src_vendor ( -- ft ) [ srcVendorField s" ft_MAC_src_vendor" ft_MAC_src 1 ' vendorCode calc_STATIC CalcNew FieldTypeDefine ] LITERAL ;
: ft_MAC_dst_vendor ( -- ft ) [ dstVendorField s" ft_MAC_dst_vendor" ft_MAC_dst 1 ' vendorCode calc_STATIC CalcNew FieldTypeDefine ] LITERAL ;

\ ======================= adding columns to table spec ======================================

\ the fully expanded word would be:
\ addColumnToSpec ( spec field aggCode sortMode hidden -- ) AggColumnNew TableSpecAddColumn
\ but for convenience we define various words that all amount to the same thing...

256 array COLUMNFTS
variable NextColumnIndex

: resetColumnIndex  0 NextColumnIndex ! ;
: getNextColumnIndex ( -- i ) NextColumnIndex @ dup 1+ NextColumnIndex ! ;   \ post-increment
: storeColumnFT ( ft -- ) COLUMNFTS getNextColumnIndex cells + ! ;           \ store into the next slot
: getColumnFieldTypeAtIndex ( index -- ft ) COLUMNFTS swap cells + @ ;       \ array access

\ store the column field types into the array as they are added to the spec (but not for hidden cols)

: addColumn ( spec field aggCode sortMode -- ) 0 AggColumnNew dup AggColumnFieldType storeColumnFT TableSpecAddColumn ;
: addHidden ( spec field aggCode sortMode -- ) 1 AggColumnNew TableSpecAddColumn ;

: addValue  ( spec field -- ) agg_SUM sort_NONE addColumn ;
: addSortValue ( spec field -- ) agg_SUM sort_DESCENDING addColumn ;
: addKey ( spec field -- ) agg_KEY sort_NONE addColumn ;
: addDerived ( spec field -- ) agg_UNDEFINED sort_NONE addColumn ;
: addMeanValue  ( spec field -- ) agg_MEAN sort_NONE addColumn ;
: ignoreColumn ( spec field -- ) agg_UNDEFINED sort_NONE addHidden ;

\ ================== unpack ifStatus into adminup (b0) and operup (b1) ==============================

: statusAdminUp q@ 1 AND 0> ;  \ no need to test for NULL input, but make sure output is true or false
: statusOperUp  q@ 2 AND 0> ;
: statusAdminDown  q@ 1 AND 0= ;
: statusOperDown  q@ 2 AND 0= ;

: ft_admin_up   [ int32ValueField s" ft_admin_up" ft_ifStatus 1 ' statusAdminUp calc_BOOLEAN CalcNew FieldTypeDefine ] LITERAL ;
: ft_oper_up    [ int32ValueField s" ft_oper_up" ft_ifStatus 1 ' statusOperUp calc_BOOLEAN CalcNew FieldTypeDefine ] LITERAL ;
: ft_admin_down [ int32ValueField s" ft_admin_down" ft_ifStatus 1 ' statusAdminDown calc_BOOLEAN CalcNew FieldTypeDefine ] LITERAL ;
: ft_oper_down  [ int32ValueField s" ft_oper_down" ft_ifStatus 1 ' statusOperDown calc_BOOLEAN CalcNew FieldTypeDefine ] LITERAL ;

\ ================== addressPair and destinationSap use the implicit concatenate fn ===================

: implicitCalc ( input1 input2 n -- calc )
  0 \ null calc word
  calc_STATIC CalcNew ;

: listValueField ot_LIST valueField ;

: ft_addressPair ( -- ft )    [ listValueField s" ft_addressPair" ft_src ft_dst 2 implicitCalc FieldTypeDefine ] LITERAL ;
: ft_destinationSap ( -- ft ) [ listValueField s" ft_destinationSap" ft_dst ft_dst_port 2 implicitCalc FieldTypeDefine ] LITERAL ;

\ ======================== address-to-path lookup table  ==============================

: stringField  ot_STRING st_UNKNOWN pt_UNKNOWN kl_UNDEFINED ;
: srcStringField stringField fc_SRC ;
: dstStringField stringField fc_DST ;

\ group paths
: ft_sourcePath      [ srcStringField s" ft_sourcePath" ft_src 1 depth_GROUP 1 1 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_destinationPath [ dstStringField s" ft_destinationPath" ft_dst 1 depth_GROUP 1 1 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;

\ config node lookups
\ note: calc with mode RDCALC_NODE will confer cd_CONFIG cacheDep automatically, but we need to add it anyway for placementCache
: ft_sourceEnterprise      [ srcStringField s" ft_sourceEnterprise" ft_src 1 depth_ENTERPRISE 0 1 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_destinationEnterprise [ dstStringField s" ft_destinationEnterprise" ft_dst 1 depth_ENTERPRISE 0 1 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_sourceSite            [ srcStringField s" ft_sourceSite" ft_src 1 depth_SITE 0 1 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_destinationSite       [ dstStringField s" ft_destinationSite" ft_dst 1 depth_SITE 0 1 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_sourceZone            [ srcStringField s" ft_sourceZone" ft_src 1 depth_ZONE 0 1 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_destinationZone       [ dstStringField s" ft_destinationZone" ft_dst 1 depth_ZONE 0 1 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_sourceGroup           [ srcStringField s" ft_sourceGroup" ft_src 1 depth_GROUP 0 1 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_destinationGroup      [ dstStringField s" ft_destinationGroup" ft_dst 1 depth_GROUP 0 1 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_sourceSubnet          [ srcStringField s" ft_sourceSubnet" ft_src 1 depth_SUBNET 0 1 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_destinationSubnet     [ dstStringField s" ft_destinationSubnet" ft_dst 1 depth_SUBNET 0 1 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;

\ sourceaddress and destinationaddress as strings (to shadow ft_DNS_SRC and ft_DNS_DST)
\ : objToString ( -- cell )
\   count ot_OBJECT ObjectPrint ( -- str len )
\   over >r  \ save intermediate str so we can free it after making the cell
\   ot_STRING ObjectAsCell ( -- cell )
\   r> BufferFree ;
\
\ : ft_src_str ( -- ft ) [ srcStringField s" ft_src_str" ft_src 1 ' objToString calc_ONHEAP CalcNew FieldTypeDefine dup ft_DNS_SRC FieldTypeSetShadow ] LITERAL ;
\ : ft_dst_str ( -- ft ) [ dstStringField s" ft_dst_str" ft_dst 1 ' objToString calc_ONHEAP CalcNew FieldTypeDefine dup ft_DNS_DST FieldTypeSetShadow ] LITERAL ;

: commaDelimited s" ," ; \ comma,space,tab is the default in the engine (bad for strings with spaces)

\ ======================== interzone/intergroup flags  =============================

\ zone and group names are symbols, so we can just compare the pointers directly
: interZone ( srcz dstz -- neq ) = 0= ;
: interGroup ( srcz dstz -- neq ) = 0= ;

: ft_interZone  [ int32ValueField s" ft_interZone" ft_sourceZone ft_destinationZone 2 ' interZone calc_BOOLEAN CalcNew FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_interGroup [ int32ValueField s" ft_interGroup" ft_sourcePath ft_destinationPath 2 ' interGroup calc_BOOLEAN CalcNew FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;

\ ======================== agent-to-path lookup table  =============================

\ interface and agent path may require allocating strings that are not in the config symbol table, so they are handled specially

: agentPathStringField stringField fc_UNDEFINED ;

: getPathForInterface { agentIP ifIndex | -- path }
	agentIP count ot_OBJECT ifIndex PathForInterface ;

: getPathForAgentIP { agentIP | -- path }
	agentIP count ot_OBJECT PathForAgentIP ;

: getPathForStream { streamid | -- path }
	streamid PathForStream ;

: ft_agentPath     [ agentPathStringField s" ft_agentPath" ft_agent_IP 1 ' getPathForAgentIP calc_ONHEAP CalcNew FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;

: ft_interfacePath [ agentPathStringField s" ft_interfacePath" ft_agent_IP ft_ifIndex 2 ' getPathForInterface calc_ONHEAP CalcNew FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;

: ft_streamPath [ agentPathStringField s" ft_streamPath" ft_stream_id 1 ' getPathForStream calc_ONHEAP CalcNew FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;

\ but group,zone site and enterprise can be optimized
\ note: calc with mode RDCALC_NODE will confer cd_CONFIG cacheDep automatically, but we need to add it anyway for placementCache
: ft_enterprise [ agentPathStringField s" ft_enterprise" ft_agent_IP 1 depth_ENTERPRISE 0 0 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_site       [ agentPathStringField s" ft_site" ft_agent_IP 1 depth_SITE 0 0 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_zone       [ agentPathStringField s" ft_zone" ft_agent_IP 1 depth_ZONE 0 0 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_group      [ agentPathStringField s" ft_group" ft_agent_IP 1 depth_GROUP 0 0 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_zonePath   [ agentPathStringField s" ft_zonePath" ft_agent_IP 1 depth_ZONE 1 0 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_groupPath  [ agentPathStringField s" ft_groupPath" ft_agent_IP 1 depth_GROUP 1 0 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;


\  repeat for sourceAgent and destinationAgent
: ft_sourceAgentEnterprise [ agentPathStringField s" ft_sourceAgentEnterprise" ft_hostDS_src 1 depth_ENTERPRISE 0 0 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_sourceAgentSite [ agentPathStringField s" ft_sourceAgentSite" ft_hostDS_src 1 depth_SITE 0 0 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_sourceAgentZone [ agentPathStringField s" ft_sourceAgentZone" ft_hostDS_src 1 depth_ZONE 0 0 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_sourceAgentGroup [ agentPathStringField s" ft_sourceAgentGroup" ft_hostDS_src 1 depth_GROUP 0 0 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_sourceAgentZonePath [ agentPathStringField s" ft_sourceAgentZonePath" ft_hostDS_src 1 depth_ZONE 1 0 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_sourceAgentGroupPath [ agentPathStringField s" ft_sourceAgentGroupPath" ft_hostDS_src 1 depth_GROUP 1 0 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;

: ft_destinationAgentEnterprise [ agentPathStringField s" ft_destinationAgentEnterprise" ft_hostDS_dst 1 depth_ENTERPRISE 0 0 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_destinationAgentSite [ agentPathStringField s" ft_destinationAgentSite" ft_hostDS_dst 1 depth_SITE 0 0 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_destinationAgentZone [ agentPathStringField s" ft_destinationAgentZone" ft_hostDS_dst 1 depth_ZONE 0 0 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_destinationAgentGroup [ agentPathStringField s" ft_destinationAgentGroup" ft_hostDS_dst 1 depth_GROUP 0 0 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_destinationAgentZonePath [ agentPathStringField s" ft_destinationAgentZonePath" ft_hostDS_dst 1 depth_ZONE 1 0 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_destinationAgentGroupPath [ agentPathStringField s" ft_destinationAgentGroupPath" ft_hostDS_dst 1 depth_GROUP 1 0 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;

\ ================ also need MAC,IP,IP6 address versions so we can apply correct mask or range filtering =============

: macField ot_MAC st_UNKNOWN pt_UNKNOWN kl_UNDEFINED ;
: ipField ot_IP st_UNKNOWN pt_UNKNOWN kl_UNDEFINED ;
: ip6Field ot_IP6 st_UNKNOWN pt_UNKNOWN kl_UNDEFINED ;
: fbcnField ot_FBCN st_UNKNOWN pt_UNKNOWN kl_UNDEFINED ;
: subnetField ot_SUBNET st_UNKNOWN pt_UNKNOWN kl_UNDEFINED ;
: subnet6Field ot_SUBNET6 st_UNKNOWN pt_UNKNOWN kl_UNDEFINED ;
: objectField ot_OBJECT st_UNKNOWN pt_UNKNOWN kl_UNDEFINED ;

: ft_MAC_server ( -- ft )  [ macField  fc_SERVER s" serverMAC" ft_MAC_dst ft_MAC_src ft_serviceDirection 3 CalcNewFlip FieldTypeDefine ] LITERAL ;
: ft_MAC_client ( -- ft )  [ macField  fc_CLIENT s" clientMAC" ft_MAC_src ft_MAC_dst ft_serviceDirection 3 CalcNewFlip FieldTypeDefine ] LITERAL ;
: ft_IP_server ( -- ft )   [ ipField   fc_SERVER s" serverIP" ft_IP_dst ft_IP_src ft_serviceDirection 3 CalcNewFlip FieldTypeDefine ] LITERAL ;
: ft_IP_client ( -- ft )   [ ipField   fc_CLIENT s" clientIP" ft_IP_src ft_IP_dst ft_serviceDirection 3 CalcNewFlip FieldTypeDefine ] LITERAL ;
: ft_IP6_server ( -- ft )  [ ip6Field  fc_SERVER s" serverIP6" ft_IP6_dst ft_IP6_src ft_serviceDirection 3 CalcNewFlip FieldTypeDefine ] LITERAL ;
: ft_IP6_client ( -- ft )  [ ip6Field  fc_CLIENT s" clientIP6" ft_IP6_src ft_IP6_dst ft_serviceDirection 3 CalcNewFlip FieldTypeDefine ] LITERAL ;
: ft_FBCN_server ( -- ft ) [ fbcnField fc_SERVER s" serverFBCN" ft_FBCN_dst ft_FBCN_src ft_serviceDirection 3 CalcNewFlip FieldTypeDefine ] LITERAL ;
: ft_FBCN_client ( -- ft ) [ fbcnField fc_CLIENT s" clientFBCN" ft_FBCN_src ft_FBCN_dst ft_serviceDirection 3 CalcNewFlip FieldTypeDefine ] LITERAL ;
: ft_SUBNET_server ( -- ft ) [ subnetField fc_SERVER s" serverSUBNET" ft_SUBNET_dst ft_SUBNET_src ft_serviceDirection 3 CalcNewFlip FieldTypeDefine ] LITERAL ;
: ft_SUBNET_client ( -- ft ) [ subnetField fc_SERVER s" clientSUBNET" ft_SUBNET_src ft_SUBNET_dst ft_serviceDirection 3 CalcNewFlip FieldTypeDefine ] LITERAL ;
: ft_SUBNET6_server ( -- ft ) [ subnet6Field fc_SERVER s" server6SUBNET" ft_SUBNET6_dst ft_SUBNET6_src ft_serviceDirection 3 CalcNewFlip FieldTypeDefine ] LITERAL ;
: ft_SUBNET6_client ( -- ft ) [ subnet6Field fc_SERVER s" client6SUBNET" ft_SUBNET6_src ft_SUBNET6_dst ft_serviceDirection 3 CalcNewFlip FieldTypeDefine ] LITERAL ;

\ ================== service hostpaths =================================

: ft_serverPath ( -- ft )       [ stringField fc_SERVER s" serverPath" ft_serveraddress 1 depth_GROUP 1 1 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_clientPath ( -- ft )       [ stringField fc_CLIENT s" clientPath" ft_clientaddress 1 depth_GROUP 1 1 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_serverEnterprise ( -- ft ) [ stringField fc_SERVER s" serverEnterprise" ft_serveraddress 1 depth_ENTERPRISE 0 1 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_clientEnterprise ( -- ft ) [ stringField fc_CLIENT s" clientEnterprise" ft_clientaddress 1 depth_ENTERPRISE 0 1 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_serverSite ( -- ft )       [ stringField fc_SERVER s" serverSite" ft_serveraddress 1 depth_SITE 0 1 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_clientSite ( -- ft )       [ stringField fc_CLIENT s" clientSite" ft_clientaddress 1 depth_SITE 0 1 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_serverZone ( -- ft )       [ stringField fc_SERVER s" serverZone" ft_serveraddress 1 depth_ZONE 0 1 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_clientZone ( -- ft )       [ stringField fc_CLIENT s" clientZone" ft_clientaddress 1 depth_ZONE 0 1 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_serverGroup ( -- ft )      [ stringField fc_SERVER s" serverGroup" ft_serveraddress 1 depth_GROUP 0 1 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_clientGroup ( -- ft )      [ stringField fc_CLIENT s" clientGroup" ft_clientaddress 1 depth_GROUP 0 1 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_serverSubnet ( -- ft )     [ stringField fc_SERVER s" serverSubnet" ft_serveraddress 1 depth_SUBNET 0 1 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;
: ft_clientSubnet ( -- ft )     [ stringField fc_CLIENT s" clientSubnet" ft_clientaddress 1 depth_SUBNET 0 1 CalcNewLookup FieldTypeDefine dup cd_CONFIG FieldTypeAddCacheDep ] LITERAL ;

: dnsField ot_DNS st_UNKNOWN pt_UNKNOWN kl_UNDEFINED ;

: ft_DNS_server ( -- ft ) [ dnsField fc_SERVER s" serverDNS" ft_DNS_dst ft_DNS_src ft_serviceDirection 3 CalcNewFlip FieldTypeDefine ] LITERAL ;
: ft_DNS_client ( -- ft ) [ dnsField fc_CLIENT s" clientDNS" ft_DNS_src ft_DNS_dst ft_serviceDirection 3 CalcNewFlip FieldTypeDefine ] LITERAL ;

: unmField ot_USER_ID st_UNKNOWN pt_UNKNOWN kl_UNDEFINED ;

: ft_UNM_server ( -- ft ) [ unmField fc_SERVER s" serverUNM" ft_UNM_dst ft_UNM_src ft_serviceDirection 3 CalcNewFlip FieldTypeDefine ] LITERAL ;
: ft_UNM_client ( -- ft ) [ unmField fc_CLIENT s" clientUNM" ft_UNM_src ft_UNM_dst ft_serviceDirection 3 CalcNewFlip FieldTypeDefine ] LITERAL ;

: uuidField ot_UUID st_UNKNOWN pt_UNKNOWN kl_UNDEFINED ;

: ft_uuid_server ( -- ft ) [ uuidField fc_SERVER s" serverUUID" ft_uuid_dst ft_uuid_src ft_serviceDirection 3 CalcNewFlip FieldTypeDefine ] LITERAL ;
: ft_uuid_client ( -- ft ) [ uuidField fc_CLIENT s" clientUUID" ft_uuid_src ft_uuid_dst ft_serviceDirection 3 CalcNewFlip FieldTypeDefine ] LITERAL ;

: ft_ifLocalAddress ( -- ft ) [ objectField fc_SERVER s" ifLocalAddress" ft_dst ft_src ft_interfaceDirection 3 CalcNewFlip FieldTypeDefine ] LITERAL ;

\ ======================= protocol grouping ======================================
\ may want to pass in serveraddress as well as serverport, in case the same port is used for
\ different things on different servers,  but for now we'll keep it simple.


: buildProtocolGroupLookup
	TableSpecNew
		dup ft_PROCESS_src addKey
		dup ft_src_port agg_FIRST sort_NONE addColumn
		dup ft_src_port agg_LAST sort_NONE addColumn
	RangeMapNew
		dup GLOBALCONFIGDIR count PROTOCOLGROUPS count nullString semicolonCommentChar RangeMapLoadProtocolGroups ;


: getOrBuildProtocolGroupLookup
	s" protocolGroupLookup"
	[ 0 ' buildProtocolGroupLookup calc_STATIC CalcNew ] LITERAL
	GlobalVar ;

variable PGROUPLOOKUP 0 PGROUPLOOKUP !

: protocolGroup ( portstring -- protocolGroup )
   PGROUPLOOKUP @ ?dup 0= IF getOrBuildProtocolGroupLookup dup PGROUPLOOKUP ! THEN
   RangeMapLookup ;

: pgroupStringField stringField fc_FLOW ;

: ft_protocolGroup_dynamic ( -- ft ) [ pgroupStringField s" protocolGroup_dynamic" ft_serverport 1 ' protocolGroup calc_STATIC CalcNew FieldTypeDefine dup ft_protocolGroup FieldTypeSetShadow dup cd_PGROUP FieldTypeAddCacheDep ] LITERAL ;

\ ======================= country lookup ======================================


: countrySpec ( -- spec )
	\ the relevant fields will be picked out by the agg_code.
	\ The range will be from agg_FIRST to agg_LAST
	\ The value will be taken as the first agg_KEY column.
	TableSpecNew
		dup ft_IP_src agg_FIRST sort_NONE addColumn   \ lower
		dup ft_IP_src agg_LAST sort_NONE addColumn    \ upper
		dup ft_PROCESS_src ignoreColumn               \ coordinate1
		dup ft_PROCESS_src ignoreColumn               \ coordinate2
		dup ft_PROCESS_src addKey                     \ ISO abbreviation
		\ dup ft_PROCESS_src addKey                   \ country name
;

: countryTable ( -- table )
        countrySpec GLOBALCONFIGDIR count GEOIP count 0 nullString semicolonCommentChar nullString TableSpecParseFile ;

: buildCountryLookup ( -- rangeMap )
	countryTable
        dup TableObjectSpec RangeMapNew dup RangeMapAllowSort
	\ now have ( tab rangeMap -- )
	dup -rot TableAddTo ;

: getOrBuildCountryLookup
	s" countryLookup"
	[ 0 ' buildCountryLookup calc_STATIC CalcNew ] LITERAL
	GlobalVar ;

variable COUNTRYLOOKUP 0 COUNTRYLOOKUP !

: country ( ipaddress -- country )
   COUNTRYLOOKUP @ ?dup 0= IF getOrBuildCountryLookup dup COUNTRYLOOKUP ! THEN
   RangeMapLookup ;

: countryStringField stringField fc_FLOW ;

: ft_sourceCountry ( -- ft )      [ countryStringField s" sourceCountry" ft_IP_src 1 ' country calc_STATIC CalcNew FieldTypeDefine dup cd_GEOIP FieldTypeAddCacheDep ] LITERAL ;
: ft_destinationCountry ( -- ft ) [ countryStringField s" destinationCountry" ft_IP_dst 1 ' country calc_STATIC CalcNew FieldTypeDefine dup cd_GEOIP FieldTypeAddCacheDep ] LITERAL ;
: ft_clientCountry ( -- ft )      [ countryStringField s" clientCountry" ft_IP_client 1 ' country calc_STATIC CalcNew FieldTypeDefine dup cd_GEOIP FieldTypeAddCacheDep ] LITERAL ;
: ft_serverCountry ( -- ft )      [ countryStringField s" serverCountry" ft_IP_server 1 ' country calc_STATIC CalcNew FieldTypeDefine dup cd_GEOIP FieldTypeAddCacheDep ] LITERAL ;
: ft_xffCountry1 ( -- ft )      [ countryStringField s" xffCountry1" ft_HTTP_XFF_IP1 1 ' country calc_STATIC CalcNew FieldTypeDefine dup cd_GEOIP FieldTypeAddCacheDep ] LITERAL ;

\ =====================  extracting ipdscp from iptos ======================

: tos2dscp ( tosPtr -- dscpPtr ) TOS_to_DSCP ;
: ipDSCPField ot_INT32 st_UNKNOWN pt_IP kl_LAYER3HOP fc_FLOW ;
: ip6DSCPField ot_INT32 st_UNKNOWN pt_IP6 kl_LAYER3HOP fc_FLOW ;
: ft_IP_DSCP_dynamic ( -- ft )  [ ipDSCPField s" ipdscp_dynamic" ft_IP_TOS 1 ' tos2dscp calc_TABLE CalcNew FieldTypeDefine dup ft_IP_DSCP FieldTypeSetShadow ] LITERAL ;
: ft_IP6_DSCP_dynamic ( -- ft ) [ ip6DSCPField s" ip6dscp_dynamic" ft_IP6_TOS 1 ' tos2dscp calc_TABLE CalcNew FieldTypeDefine dup ft_IP6_DSCP FieldTypeSetShadow ] LITERAL ;


\ ====================== scaling layer2 counts ============================

: doubleField ot_DOUBLE st_UNKNOWN pt_UNKNOWN kl_VALUE fc_VALUE ;

: ft_frames_esr ( -- ft )         [ doubleField s" frames_esr" ft_samples ft_MEANSKIP ft_ESR 3 ' scaleByESR calc_TABLE CalcNew FieldTypeDefine dup ft_frames FieldTypeSetShadow ] LITERAL ;
: ft_scaledL2Bytes_esr ( -- ft )  [ doubleField s" scaledL2Bytes_esr" ft_L2Bytes ft_MEANSKIP ft_ESR 3 ' scaleByESR calc_TABLE CalcNew FieldTypeDefine dup ft_scaledL2Bytes FieldTypeSetShadow ] LITERAL ;
: ft_bytes_esr ( -- ft )          [ doubleField s" bytes_esr" ft_L3Bytes ft_MEANSKIP ft_ESR 3 ' scaleByESR calc_TABLE CalcNew FieldTypeDefine dup ft_bytes FieldTypeSetShadow ] LITERAL ;
: ft_transportbytes_esr ( -- ft ) [ doubleField s" transportbytes_esr" ft_TRBytes ft_MEANSKIP ft_ESR 3 ' scaleByESR calc_TABLE CalcNew FieldTypeDefine dup ft_transportbytes FieldTypeSetShadow ] LITERAL ;
  
: ft_SYNACKcount_esr ( -- ft )  [ doubleField s" synackcount_esr" ft_samples_SYNACK ft_MEANSKIP ft_ESR 3 ' scaleByESR calc_TABLE CalcNew FieldTypeDefine dup ft_SYNACKCount FieldTypeSetShadow ] LITERAL ;
: ft_SYNOnlycount_esr ( -- ft ) [ doubleField s" SYNOnlycount_esr" ft_samples_SYNOnly ft_MEANSKIP ft_ESR 3 ' scaleByESR calc_TABLE CalcNew FieldTypeDefine dup ft_SYNOnlyCount FieldTypeSetShadow ] LITERAL ;

: ft_sizeTo63_esr ( -- ft )   [ doubleField s" sizeTo63_esr" ft_samples_sizeTo63 ft_MEANSKIP ft_ESR 3 ' scaleByESR calc_TABLE CalcNew FieldTypeDefine dup ft_sizeTo63 FieldTypeSetShadow ] LITERAL ;
: ft_size64_esr ( -- ft )     [ doubleField s" size64_esr" ft_samples_size64 ft_MEANSKIP ft_ESR 3 ' scaleByESR calc_TABLE CalcNew FieldTypeDefine dup ft_size64 FieldTypeSetShadow ] LITERAL ;
: ft_sizeTo127_esr ( -- ft )  [ doubleField s" sizeTo127_esr" ft_samples_sizeTo127 ft_MEANSKIP ft_ESR 3 ' scaleByESR calc_TABLE CalcNew FieldTypeDefine dup ft_sizeTo127 FieldTypeSetShadow ] LITERAL ;
: ft_sizeTo255_esr ( -- ft )  [ doubleField s" sizeTo255_esr" ft_samples_sizeTo255 ft_MEANSKIP ft_ESR 3 ' scaleByESR calc_TABLE CalcNew FieldTypeDefine dup ft_sizeTo255 FieldTypeSetShadow ] LITERAL ;
: ft_sizeTo511_esr ( -- ft )  [ doubleField s" sizeTo511_esr" ft_samples_sizeTo511 ft_MEANSKIP ft_ESR 3 ' scaleByESR calc_TABLE CalcNew FieldTypeDefine dup ft_sizeTo511 FieldTypeSetShadow ] LITERAL ;
: ft_sizeTo1023_esr ( -- ft ) [ doubleField s" sizeTo1023_esr" ft_samples_sizeTo1023 ft_MEANSKIP ft_ESR 3 ' scaleByESR calc_TABLE CalcNew FieldTypeDefine dup ft_sizeTo1023 FieldTypeSetShadow ] LITERAL ;
: ft_sizeTo1518_esr ( -- ft ) [ doubleField s" sizeTo1518_esr" ft_samples_sizeTo1518 ft_MEANSKIP ft_ESR 3 ' scaleByESR calc_TABLE CalcNew FieldTypeDefine dup ft_sizeTo1518 FieldTypeSetShadow ] LITERAL ;
: ft_sizeJumbo_esr ( -- ft )  [ doubleField s" sizeJumbo_esr" ft_samples_sizeJumbo ft_MEANSKIP ft_ESR 3 ' scaleByESR calc_TABLE CalcNew FieldTypeDefine dup ft_sizeJumbo FieldTypeSetShadow ] LITERAL ;

\ ==================== time conversion fields ===================
: secondsField ot_UNIXSECONDS st_UNKNOWN pt_UNKNOWN kl_UNDEFINED fc_FLOW ;
: int32Field ot_INT32 st_UNKNOWN pt_UNKNOWN kl_UNDEFINED fc_FLOW ;
: localtime_offset ( gmtsecs -- tzsecs ) RDTU_zone unixsecondsToLocal ;
: localtime_hmin ( gmtsecs -- hmin ) q@ RDTU_hmin unixsecondsToLocal ;
: localtime_dhour ( gmtsecs -- dhour ) q@ RDTU_dhour unixsecondsToLocal ;
: localtime_wday ( gmtsecs -- wday ) q@ RDTU_wday unixsecondsToLocal ;
: localtime_mday ( gmtsecs -- mday ) q@ RDTU_mday unixsecondsToLocal ;
: localtime_yweek ( gmtsecs -- yweek ) q@ RDTU_yweek unixsecondsToLocal ;
: localtime_ymonth ( gmtsecs -- ymonth ) q@ RDTU_ymonth unixsecondsToLocal ;
: localtime_smin ( gmtsecs -- gmtsecs2 ) q@ RDTU_smin unixsecondsToLocal ;
: localtime_shour ( gmtsecs -- gmtsecs2 ) q@ RDTU_shour unixsecondsToLocal ;
: localtime_sday ( gmtsecs -- gmtsecs2 ) q@ RDTU_sday unixsecondsToLocal ;
: localtime_sweek ( gmtsecs -- gmtsecs2 ) q@ RDTU_sweek unixsecondsToLocal ;

: ft_tzoffset ( -- ft ) [ int32Field s" tzoffset" ft_INTERVALSTART 1 ' localtime_offset calc_TABLE CalcNew FieldTypeDefine ] LITERAL ;
: ft_hourminute ( -- ft ) [ int32Field s" hourminute" ft_INTERVALSTART 1 ' localtime_hmin calc_TABLE CalcNew FieldTypeDefine ] LITERAL ;
: ft_dayhour ( -- ft ) [ int32Field s" dayhour" ft_INTERVALSTART 1 ' localtime_dhour calc_TABLE CalcNew FieldTypeDefine ] LITERAL ;
: ft_weekday ( -- ft ) [ int32Field s" weekday" ft_INTERVALSTART 1 ' localtime_wday calc_TABLE CalcNew FieldTypeDefine ] LITERAL ;
: ft_monthday ( -- ft ) [ int32Field s" monthday" ft_INTERVALSTART 1 ' localtime_mday calc_TABLE CalcNew FieldTypeDefine ] LITERAL ;
: ft_yearweek ( -- ft ) [ int32Field s" yearweek" ft_INTERVALSTART 1 ' localtime_yweek calc_TABLE CalcNew FieldTypeDefine ] LITERAL ;
: ft_yearmonth ( -- ft ) [ int32Field s" yearmonth" ft_INTERVALSTART 1 ' localtime_ymonth calc_TABLE CalcNew FieldTypeDefine ] LITERAL ;
: ft_minutestart ( -- ft ) [ secondsField s" minutestart" ft_INTERVALSTART 1 ' localtime_smin calc_TABLE CalcNew FieldTypeDefine ] LITERAL ;
: ft_hourstart ( -- ft ) [ secondsField s" hourstart" ft_INTERVALSTART 1 ' localtime_shour calc_TABLE CalcNew FieldTypeDefine ] LITERAL ;
: ft_daystart ( -- ft ) [ secondsField s" daystart" ft_INTERVALSTART 1 ' localtime_sday calc_TABLE CalcNew FieldTypeDefine ] LITERAL ;
: ft_weekstart ( -- ft ) [ secondsField s" weekstart" ft_INTERVALSTART 1 ' localtime_sweek calc_TABLE CalcNew FieldTypeDefine ] LITERAL ;

\ ========= variables used to control view options ===========

variable Suppress_Other false Suppress_Other !

\ ===================== linkedPorts ===============================

: streamIdField ot_OBJECT st_UNKNOWN pt_UNKNOWN kl_STREAMID fc_FLOW ;

64 string LINKEDPORTS  s" <./linkedPorts" LINKEDPORTS place

: linkedPortsSpec ( -- spec )
  	resetColumnIndex
	TableSpecNew
		dup ft_stream_id addKey
;

: linkedPortsTable ( spec -- table )
        linkedPortsSpec nullstring LINKEDPORTS count 0 nullString semicolonCommentChar nullString TableSpecParseFile ;

: buildLinkedPortsMap ( -- map )
	linkedPortsTable
        dup TableObjectSpec MapNew	\  ( tab1 map -- )
	dup				\  ( tab1 map map -- )
        -rot				\  ( map tab1 map -- )
	TableAddTo ;

: getOrBuildLinkedPortsMap
	s" linkedPortsMap"
	[ 0 ' buildLinkedPortsMap calc_STATIC CalcNew ] LITERAL
	GlobalVar ;

variable LINKEDPORTSMAP 0 LINKEDPORTSMAP !

: isLinkedPort ( interface -- bool )
   LINKEDPORTSMAP @ ?dup 0= IF getOrBuildLinkedPortsMap dup LINKEDPORTSMAP ! THEN
   MapIn ;

: ft_interface_in ( -- ft ) [ streamIdField s" ft_interface_in" ft_agent_IP ft_IF_in 2 implicitCalc FieldTypeDefine dup cd_NOCACHE FieldTypeAddCacheDep ] LITERAL ;
: ft_interface_out ( -- ft ) [ streamIdField s" ft_interface_out" ft_agent_IP ft_IF_out 2 implicitCalc FieldTypeDefine dup cd_NOCACHE FieldTypeAddCacheDep ] LITERAL ;
: ft_isLinkedPort ( -- ft )  [ int32ValueField s" ft_isLinkedPort" ft_stream_id 1 ' isLinkedPort calc_BOOLEAN CalcNew FieldTypeDefine dup cd_NOCACHE FieldTypeAddCacheDep ] LITERAL ;
: ft_isLinkedPort_in ( -- ft )  [ int32ValueField s" ft_isLinkedPort_in" ft_interface_in 1 ' isLinkedPort calc_BOOLEAN CalcNew FieldTypeDefine dup cd_NOCACHE FieldTypeAddCacheDep ] LITERAL ;
: ft_isLinkedPort_out ( -- ft )  [ int32ValueField s" ft_isLinkedPort_out" ft_interface_out 1 ' isLinkedPort calc_BOOLEAN CalcNew FieldTypeDefine dup cd_NOCACHE FieldTypeAddCacheDep ] LITERAL ;

\ ===================== address location ===============================

64 string L3LOCATIONS  s" <./l3locations" L3LOCATIONS place
: ft_location ( -- ft ) [ streamIdField s" ft_location" ft_agent_IP ft_ifIndex 2 implicitCalc FieldTypeDefine dup cd_NOCACHE FieldTypeAddCacheDep  ] LITERAL ;

: l3LocationsSpec ( -- spec )
  	resetColumnIndex
	TableSpecNew
		dup ft_src ignoreColumn
		dup ft_MAC_src ignoreColumn
		dup ft_agent_IP ignoreColumn
		dup ft_ifIndex ignoreColumn

		dup ft_src addKey
		dup ft_agent_IP agg_FIRST sort_NONE addColumn
		dup ft_location agg_FIRST sort_NONE addColumn
;

: l3LocationsTable ( spec -- table )
        \ add '>' to the list of separators because we want to separate the ip>ifindex field into two fields
        l3LocationsSpec nullstring L3LOCATIONS count 0 s" ,>" semicolonCommentChar nullString TableSpecParseFile ;

: buildL3LocationsMap ( -- map )
	l3LocationsTable
	\ dup .table_dump cr
        dup TableObjectSpec MapNew	\  ( tab1 map -- )
	    \ tell the map loader to use the input cells, e.g. ft_location, rather than first(ft_location)
	    dup 1 MapSetPreAggregation
	dup				\  ( tab1 map map -- )
        -rot				\  ( map tab1 map -- )
	TableAddTo ;

: getOrBuildL3LocationsMap
	s" l3LocationsMap"
	[ 0 ' buildL3LocationsMap calc_STATIC CalcNew ] LITERAL
	GlobalVar ;

variable L3LOCATIONSMAP 0 L3LOCATIONSMAP !

: addrLocationAgent ( addr -- agent )
   L3LOCATIONSMAP @ ?dup 0= IF getOrBuildL3LocationsMap dup L3LOCATIONSMAP ! THEN
   0 MapLookupAt ;

: addrLocation ( addr -- interface )
   L3LOCATIONSMAP @ ?dup 0= IF getOrBuildL3LocationsMap dup L3LOCATIONSMAP ! THEN
   1 MapLookupAt ;

: locationField ot_OBJECT st_UNKNOWN pt_UNKNOWN kl_UNDEFINED fc_VALUE ;

\ these are just shadowing the inxsazd-generated fields, to smooth the transition
\ so don't bother marking them with cache dependencies
: ft_location_src_dyn ( -- ft )  [ locationField s" ft_location_src_dyn" ft_src 1 ' addrLocation calc_STATIC CalcNew FieldTypeDefine dup ft_location_src FieldTypeSetShadow ] LITERAL ;
: ft_location_dst_dyn ( -- ft )  [ locationField s" ft_location_dst_dyn" ft_dst 1 ' addrLocation calc_STATIC CalcNew FieldTypeDefine dup ft_location_dst FieldTypeSetShadow ] LITERAL ;
: ft_locationagent_src_dyn ( -- ft )  [ locationField s" ft_locationagent_src_dyn" ft_src 1 ' addrLocationAgent calc_STATIC CalcNew FieldTypeDefine dup ft_locationagent_src FieldTypeSetShadow ] LITERAL ;
: ft_locationagent_dst_dyn ( -- ft )  [ locationField s" ft_locationagent_dst_dyn" ft_dst 1 ' addrLocationAgent calc_STATIC CalcNew FieldTypeDefine dup ft_locationagent_dst FieldTypeSetShadow ] LITERAL ;

\ synthesize the rolled up ones here,  whether they were written out by sample analyzer or conjured dynamically

: ft_locationgroup_src [ agentPathStringField s" ft_locationgroup_src" ft_locationagent_src 1 depth_GROUP 0 0 CalcNewLookup FieldTypeDefine ] LITERAL ;
: ft_locationgroup_dst [ agentPathStringField s" ft_locationgroup_dst" ft_locationagent_dst 1 depth_GROUP 0 0 CalcNewLookup FieldTypeDefine ] LITERAL ;
: ft_locationzone_src [ agentPathStringField s" ft_locationzone_src" ft_locationagent_src 1 depth_ZONE 0 0 CalcNewLookup FieldTypeDefine ] LITERAL ;
: ft_locationzone_dst [ agentPathStringField s" ft_locationzone_dst" ft_locationagent_dst 1 depth_ZONE 0 0 CalcNewLookup FieldTypeDefine ] LITERAL ;

\ ================== MAC locations ==========================================

64 string MACLOCATIONS  s" macLocations.txt" MACLOCATIONS place

\ : streamIdField ot_OBJECT st_UNKNOWN pt_UNKNOWN kl_STREAMID fc_FLOW ;
\ : ft_location ( -- ft ) [ streamIdField s" ft_location" ft_agent_IP ft_ifIndex 2 implicitCalc FieldTypeDefine ] LITERAL ;

: macLocationsSpec ( -- spec )
  	resetColumnIndex
	TableSpecNew
		dup ft_MAC_src addKey
		dup ft_agent_IP ignoreColumn
		dup ft_ifIndex ignoreColumn

		dup ft_src addKey
		dup ft_agent_IP agg_FIRST sort_NONE addColumn
		dup ft_location agg_FIRST sort_NONE addColumn
;

: macLocationsTable ( spec -- table )
        macLocationsSpec STATEDIR count MACLOCATIONS count 0 s" ," semicolonCommentChar nullString TableSpecParseFile ;

: buildMacLocationsMap ( -- map )
	macLocationsTable
	\ dup .table_dump cr
        dup TableObjectSpec MapNew	\  ( tab1 map -- )
	    \ tell the map loader to use the input cells, e.g. ft_location, rather than first(ft_location)
	    dup 1 MapSetPreAggregation
	dup				\  ( tab1 map map -- )
        -rot				\  ( map tab1 map -- )
	TableAddTo ;

: getOrBuildMacLocationsMap
	s" macLocationsMap"
	[ 0 ' buildMacLocationsMap calc_STATIC CalcNew ] LITERAL
	GlobalVar ;

variable MACLOCATIONSMAP 0 MACLOCATIONSMAP !

: macLocationAgent ( mac -- agent )
   MACLOCATIONSMAP @ ?dup 0= IF getOrBuildMacLocationsMap dup MACLOCATIONSMAP ! THEN
   0 MapLookupAt ;

: macLocation ( mac -- interface )
   MACLOCATIONSMAP @ ?dup 0= IF getOrBuildMacLocationsMap dup MACLOCATIONSMAP ! THEN
   1 MapLookupAt ;

\ these are just shadowing the inxsazd-generated fields, to smooth the transition
\ so don't bother marking them with cache dependencies
: ft_maclocation_src_dyn ( -- ft )  [ locationField s" ft_maclocation_src_dyn" ft_MAC_src 1 ' macLocation calc_STATIC CalcNew FieldTypeDefine dup ft_maclocation_src FieldTypeSetShadow ] LITERAL ;
: ft_maclocation_dst_dyn ( -- ft )  [ locationField s" ft_location_dst_dyn" ft_MAC_dst 1 ' macLocation calc_STATIC CalcNew FieldTypeDefine dup ft_maclocation_dst FieldTypeSetShadow ] LITERAL ;
: ft_maclocationagent_src_dyn ( -- ft )  [ locationField s" ft_maclocationagent_src_dyn" ft_MAC_src 1 ' macLocationAgent calc_STATIC CalcNew FieldTypeDefine dup ft_maclocationagent_src FieldTypeSetShadow ] LITERAL ;
: ft_maclocationagent_dst_dyn ( -- ft )  [ locationField s" ft_maclocationagent_dst_dyn" ft_MAC_dst 1 ' macLocationAgent calc_STATIC CalcNew FieldTypeDefine dup ft_maclocationagent_dst FieldTypeSetShadow ] LITERAL ;

\ ================== VLAN names ======================================

64 string VLANNAMES  s" vlans.txt" VLANNAMES place

: vlanNamesSpec ( -- spec )
  	resetColumnIndex
	TableSpecNew
		dup ft_VLAN_in addKey
		dup ft_g_str agg_FIRST sort_NONE addColumn
;

: vlanNamesTable ( spec -- table )
        vlanNamesSpec GLOBALCONFIGDIR count VLANNAMES count 0 s" ," semicolonCommentChar nullString TableSpecParseFile ;

: buildVlanNamesMap ( -- map )
	vlanNamesTable
	\ dup .table_dump cr
        dup TableObjectSpec MapNew	\  ( tab1 map -- )
	    \ tell the map loader to use the input cells, e.g. ft_location, rather than first(ft_location)
	    dup 1 MapSetPreAggregation
	dup				\  ( tab1 map map -- )
        -rot				\  ( map tab1 map -- )
	TableAddTo ;

: getOrBuildVlanNamesMap
	s" vlanNamesMap"
	[ 0 ' buildVlanNamesMap calc_STATIC CalcNew ] LITERAL
	GlobalVar ;

variable VLANNAMESMAP 0 VLANNAMESMAP !

: vlanName ( vlan -- name )
   VLANNAMESMAP @ ?dup 0= IF getOrBuildVlanNamesMap dup VLANNAMESMAP ! THEN
   0 MapLookupAt ;

: ft_vlanname_in      [ srcStringField s" ft_vlanname_in" ft_VLAN_in 1 ' vlanName calc_STATIC CalcNew FieldTypeDefine dup cd_VLAN FieldTypeAddCacheDep ] LITERAL ;
: ft_vlanname_out      [ srcStringField s" ft_vlanname_out" ft_VLAN_out 1 ' vlanName calc_STATIC CalcNew FieldTypeDefine dup cd_VLAN FieldTypeAddCacheDep ] LITERAL ;

\ ================ domain lookups =====================
\ Making these avaiable so that they can be used more easily
\ in the UI

: getDomain2 ( str -- dom ) 2 GetDomain ;
: ft_domain2_src ( -- ft ) [ srcStringField s" ft_domain2_src" ft_DNS_src 1 ' getDomain2 calc_ONHEAP CalcNew FieldTypeDefine ] LITERAL ;
: ft_domain2_dst ( -- ft ) [ dstStringField s" ft_domain2_dst" ft_DNS_dst 1 ' getDomain2 calc_ONHEAP CalcNew FieldTypeDefine ] LITERAL ;


\ =============== friendly name lookups for name() function ===================

: agentsFileSpec ( -- spec )
    resetColumnIndex
    TableSpecNew
		dup ft_agent_IP addKey
		dup ft_g_str ignoreColumn \ contact address
		dup ft_g_str ignoreColumn \ community
		dup ft_g_str ignoreColumn \ write-community
		dup ft_g_str ignoreColumn \ enterprise
		dup ft_g_str ignoreColumn \ site
		dup ft_g_str ignoreColumn \ zone
		dup ft_g_str ignoreColumn \ subnetname
		dup ft_hostname agg_FIRST sort_NONE addColumn \ agent-name
		\ dnsname
		\ sysname
;

: agentsTable ( spec -- table )
     agentsFileSpec STATEDIR count AGENTSFILE count 0 s" ," semicolonCommentChar nullString TableSpecParseFile ;

: buildAgentNameMap ( -- map )
      agentsTable
       \ dup .table_dump cr
       dup TableObjectSpec MapNew \ ( tab map -- )
       dup 1 MapSetPreAggregation
       dup    \ (tab map map -- )
       -rot   \ ( map tab map -- )
       TableAddTo ;

: getOrBuildAgentNameMap
	s" agentNameMap"
	[ 0 ' buildAgentNameMap calc_STATIC CalcNew ] LITERAL
	GlobalVar ;

variable AGENTNAMEMAP 0 AGENTNAMEMAP !

: getNameForAgentIP ( agentIP | -- name )
   AGENTNAMEMAP @ ?dup 0= IF getOrBuildAgentNameMap dup AGENTNAMEMAP ! THEN
   0 MapLookupAt ;

: ft_interfacesFile_streamId ( -- ft ) [ streamIdField s" ft_interfacesFile_streamId" ft_agent_IP ft_DS 2 implicitCalc FieldTypeDefine dup cd_NOCACHE FieldTypeAddCacheDep ] LITERAL ;

: interfacesFileSpec ( -- spec )
    resetColumnIndex
    TableSpecNew
		dup ft_agent_IP ignoreColumn
		dup ft_DS ignoreColumn
		dup ft_g_str ignoreColumn \ mac
		dup ft_g_str ignoreColumn \ ipAddr
		dup ft_g_str ignoreColumn \ subnet
		dup ft_g_str ignoreColumn \ lastSeen
		dup ft_g_str ignoreColumn \ bridgePort
		dup ft_ifName agg_FIRST sort_NONE addColumn
		dup ft_ifDescr agg_FIRST sort_NONE addColumn
		dup ft_ifType agg_FIRST sort_NONE addColumn
		dup ft_ifSpeed agg_FIRST sort_NONE addColumn
		dup ft_g_str ignoreColumn \  physicalIfIndex
		dup ft_VLAN_in agg_FIRST sort_NONE addColumn
		dup ft_g_str ignoreColumn \ vlan lastSeen
		dup ft_g_str ignoreColumn \ vlan flags
		dup ft_g_str ignoreColumn \ LAG
		dup ft_ifAlias agg_FIRST sort_NONE addColumn
		dup ft_g_str ignoreColumn \ sa_ifSpeed
		dup ft_g_str ignoreColumn \ sa_ifName
		dup ft_g_str ignoreColumn \ ifStatus
		\ add derived field here
		dup ft_interfacesFile_streamId addKey
;

: interfacesTable ( spec -- table )
     interfacesFileSpec STATEDIR count INTERFACESFILE count 0 s" ," semicolonCommentChar nullString TableSpecParseFile ;

: buildInterfaceNameMap ( -- map )
      interfacesTable
       \  dup .table_dump cr
       dup TableObjectSpec MapNew \ ( tab map -- )
       dup 1 MapSetPreAggregation
       dup    \ (tab map map -- )
       -rot   \ ( map tab map -- )
       TableAddTo ;

: getOrBuildInterfaceNameMap
	s" interfaceNameMap"
	[ 0 ' buildInterfaceNameMap calc_STATIC CalcNew ] LITERAL
	GlobalVar ;

variable INTERFACENAMEMAP 0 INTERFACENAMEMAP !

: getNameForInterface ( interface | -- name )
   INTERFACENAMEMAP @ ?dup 0= IF getOrBuildInterfaceNameMap dup INTERFACENAMEMAP ! THEN
   0 MapLookupAt ;


\ : getNameForPort ( port | -- str )
\	agentIP count ot_OBJECT PathForAgentIP ;
\
\ : getNameForAS ( aspath | -- str )
\	aspath ;



\ ===========================================================================

\ set some defaults that may be overridden below
\ pref_* constants are set by fcl.c at startup
variable READERTHREADS pref_READERTHREADS READERTHREADS !
: numReaderThreads READERTHREADS @ ;
: jobFifoDepth pref_JOBFIFODEPTH ;
: resultFifoDepth 100 ;

\ allow custom code to be added here
: loadExtensions EXTENSIONFILE count 1 open-file 0= IF
   include-file
  THEN ;

loadExtensions

\ important to have a cr after the last ';' to make sure the last word gets compiled
