// Copyright (c) 2014 InMon Corp. ALL RIGHTS RESERVED

(function($) {
    var bottomInset = 20;
    var leftInset = 50;
    var topInset = 5;
    var rightInset = 20;
    var timeAxisLabelSeparation = 50
    var click_xwidth_tweak = -1; // not sure why.

    var stepsizes = [1,2,5,10,20,50,100,200,500];
    var suffixes = ["μ", "\uD835\uDC5A", "","K","M","G","T","P","E"];

    function maxValue(values,sumFlag) {
	var maxVal = 0;
	var maxSum = 0;
	for(var i = 0; i < values[0].length; i++) {
	    var sum = 0;
	    for(var s = 0; s < values.length; s++) {
		var val = values[s][i];
		sum += val;
		if(val > maxVal) maxVal = val;
	    }
	    if(sum > maxSum) maxSum = sum;
	}
	return sumFlag ? maxSum : maxVal;
    }

    function drawLine(ctx, startx, starty, endx, endy, style) {
	ctx.beginPath();
	ctx.moveTo(Math.floor(startx) + 0.5, Math.floor(starty) + 0.5);
	ctx.lineTo(Math.floor(endx) + 0.5, Math.floor(endy) + 0.5); 
	// TODO: should it not be like this?:
	//ctx.moveTo(Math.floor(startx + 0.5), Math.floor(starty + 0.5));
	//ctx.lineTo(Math.floor(endx + 0.5), Math.floor(endy + 0.5)); 
	ctx.strokeStyle = style;
	ctx.lineWidth = 1;
	ctx.stroke();
    }

    function calculateStepSize(steps, target) {
	for (var ss in stepsizes) {
	    var stepsize = stepsizes[ss];
	    if(Math.floor(steps / stepsize) <= target) return stepsize;
	}
	// degenerate case for unexpected large steps
	return Math.floor(steps / target);
    }

    function drawValueAxis(ctx,h,w,maxVal,option,valueAxisLabel) {
	// find range
	var i = 0;
	var divisor = 1e-6;

	for(i = 0; i < suffixes.length; i++) {
	    if((maxVal / divisor) < 1000) break;
	    divisor *= 1000;
	}
	var steps = Math.floor(maxVal/divisor);
	if(steps < 5 && i > 0) {
	    divisor /= 1000;
	    i--;
	    steps = Math.floor(maxVal / divisor);
	}

	var rOffset = 5;
	ctx.textAlign = 'right';
	ctx.textBaseline = 'middle';
	
	var stepsize = calculateStepSize(steps, 8);

	ctx.fillStyle = "#000";

	yZero = Math.floor(h - bottomInset - topInset) + topInset;
	ctx.fillText("0",leftInset - rOffset, yZero);

	vscale = (h - bottomInset - topInset) / maxVal;
	var vstep = vscale * stepsize * divisor;

	var meas = ctx.measureText("0");
	var maxLabel = meas.width;

	var yval = yZero - vstep;
	var s = 1;
	while(yval > topInset) {
            var lval = s * stepsize;
            var suffix = suffixes[i];
            if(lval >= 1000 && i < (suffixes.length - 1)) {
		lval /= 1000;
		suffix = suffixes[i + 1]; 
            }
            var label = lval + (s && suffix ? suffix : "");
            meas = ctx.measureText(label);
            maxLabel = Math.max(maxLabel,meas.width);
            ctx.fillText(label, leftInset - rOffset, yval);
            drawLine(ctx, leftInset, yval, w - rightInset, yval, "#ccc");
            s++;
            yval -= vstep;
	}

	// add units
	if(valueAxisLabel != null) {
            var font = ctx.font;
            var fontArgs = font.split(' ');
            ctx.font = '12px ' + fontArgs[fontArgs.length - 1];
            ctx.rotate(-Math.PI/2);
            ctx.textAlign = "center";
            ctx.textBaseline = "bottom";
            ctx.fillText(valueAxisLabel, -topInset - ((h - topInset - bottomInset) / 2), Math.max(10,leftInset - maxLabel - rOffset - 5));
            ctx.rotate(Math.PI/2);
            ctx.font = font;
	}
    }

    function timeStr(ms) {
        var date = new Date(Math.round(ms / 1000) * 1000);
        //var sec = date.getSeconds().toString();
        //if(sec.length == 1) sec = "0"+sec;
        var min = date.getMinutes().toString();
        if(min.length == 1) min = "0"+min;
        var hrs = date.getHours().toString();
        if(hrs.length == 1) hrs = "0"+hrs;
        return hrs+":"+min; // +":"+sec;
    }

    function drawTimeAxis(ctx,h,w,maxVal,option,times,selectedIdx) {
	var step = (w - leftInset - rightInset) / times.length;
	var stepsize = calculateStepSize(times.length, (w / timeAxisLabelSeparation));
	var bottom = h - bottomInset;
	
	var tOffset = 5;
	ctx.textAlign = 'center';
	ctx.textBaseline = 'top';

	var markerPos;
	var tstep = 60;
	for(var t = 0; t < times.length; t++) {
	    var xPos = leftInset + (t * step);

	    // time labels and tick marks
	    var time = times[t];
	    if(selectedIdx == t) {
		markerPos = xPos;
	    } else if(Math.round(time / 1000) % (stepsize * tstep) == 0) {
		ctx.fillStyle = "#000";
		drawLine(ctx, xPos, bottom, xPos, bottom + 4, "#000");
		drawLine(ctx, xPos, bottom, xPos, topInset, "#ccc");
		ctx.fillText(timeStr(times[t]), xPos, bottom + tOffset);
	    }
	}
        drawLine(ctx, leftInset, topInset, leftInset, h - bottomInset, "#000");
	drawLine(ctx,leftInset,yZero,w - rightInset,yZero,"#000");
	if(markerPos) {
	    ctx.fillStyle = "#000";
	    drawLine(ctx, markerPos, bottom, markerPos, bottom + 4, "#000");
	    drawLine(ctx, markerPos, bottom, markerPos, topInset, "#ccc");
	    var label = timeStr(times[selectedIdx]);
	    var meas = ctx.measureText(label);
	    ctx.strokeStyle = "#000";
	    ctx.fillStyle = "#fff";
	    var radius = 2;
	    var width = meas.width + 4;
	    var height = h - bottom - tOffset - 2;
	    var x = markerPos - width / 2;
	    var y = bottom + tOffset;
	    ctx.beginPath();
	    ctx.moveTo(x + radius, y);
	    ctx.lineTo(x + width - radius, y);
	    ctx.quadraticCurveTo(x + width, y, x + width, y + radius);
	    ctx.lineTo(x + width, y + height - radius);
	    ctx.quadraticCurveTo(x + width, y + height, x + width - radius, y + height);
	    ctx.lineTo(x + radius, y + height);
	    ctx.quadraticCurveTo(x, y + height, x, y + height - radius);
	    ctx.lineTo(x, y + radius);
	    ctx.quadraticCurveTo(x, y, x + radius, y);
	    ctx.closePath();
	    ctx.stroke();
	    ctx.fill();
	    ctx.fillStyle = "#000";
	    ctx.fillText(label,markerPos,bottom + tOffset);
	} 
    }
    function drawThreshold(ctx,h,w,maxVal,threshVal) {
	if(maxVal < threshVal) return;
        var bottom = h - bottomInset;
        vscale = (bottom - topInset) / maxVal;
	thresh = Math.floor(topInset + (vscale * (maxVal - threshVal)));
	ctx.lineWidth=2;
	drawLine(ctx,leftInset,thresh,w - rightInset,thresh,"#f00");
    }

    function drawStackedSeries(ctx,h,w,maxVal,option,values,colors) {
        var bottom = h - bottomInset;
        yZero = Math.floor(h - bottomInset - topInset) + topInset;
        vscale = (h - bottomInset - topInset) / maxVal;
	for(var s = values.length - 1; s >=0; s--) {
	    var series = values[s];
	    var step = (w - leftInset - rightInset) / series.length;
	    ctx.fillStyle=colors[s % colors.length];
	    ctx.beginPath();
	    var yPos, xPos;
	    for(var i = 0; i < series.length; i++) {
		var val = series[i];
		if(s > 0) {
		    for(var c = 0; c < s; c++) val += values[c][i];
		}
		yPos = yZero - (vscale * val);
		xPos = leftInset + (i * step);
		if(i == 0) {
		    ctx.moveTo(xPos+0.5,yZero);
		    ctx.lineTo(xPos+0.5,yPos);
		}
		else ctx.lineTo(xPos,yPos);
	    }
	    ctx.lineTo(xPos,yZero);
	    ctx.closePath();
	    ctx.fill();
	}
    }
    function drawSeries(ctx,h,w,maxVal,option,values,colors) {
        var bottom = h - bottomInset;
        yZero = Math.floor(h - bottomInset - topInset) + topInset;
        vscale = (h - bottomInset - topInset) / maxVal;
	// reverse the order so that "other" is drawn first
	for(var s = values.length; --s >= 0; ) {
	    var series = values[s];
	    var step = (w - leftInset - rightInset) / series.length;
	    ctx.strokeStyle=colors[s % colors.length];
	    ctx.lineWidth=2;
	    ctx.beginPath();
	    var yPos, xPos;
	    for(var i = 0; i < series.length; i++) {
		var val = series[i];
		yPos = yZero - (vscale * val);
		xPos = leftInset + (i * step);
		if(i == 0) ctx.moveTo(xPos+0.5,yPos);
		else ctx.lineTo(xPos,yPos);
	    }
	    ctx.stroke(); 
	}
    }
    function addLegend(chart,h,w,option,headings,names,colors) {
	var legend = '<table>';
	if(headings) {
	    legend += '<head><tr><td></td>';
	    for(var i = 0; i < headings.length; i++) legend += '<th>'+headings[i] + '</th>';
	    legend += '</tr></thead>';
	}
	if(names) {
	    legend += '<tbody>';
	    for(var r = 0; r < names.length; r++) {
		legend += '<tr>';
		legend += '<td><div class="swatch" style="background:' + colors[r % colors.length] + '"></div></td>';
		if(names[r] instanceof Array) {
		    var row = names[r];
		    for(var c = 0; c < row.length; c++) legend += '<td>' + row[c] + '</td>';
		}
		else legend += '<td>' + names[r] + '</td>';
		legend += '</tr>';
	    }
	    legend += '</tbody>';
	}
	legend += '</table>';
	if(chart._legend) chart._legend.remove();
	var pos = chart._canvas.position();
	chart._legend = $(legend).css('position','absolute').css('top',pos.top).css('left',pos.left+leftInset).appendTo(chart.element);
    }
    $.widget('sfa.stripchart', {
	options: { clickable: false },
	_create: function() {
	    this.element.addClass('stripchart');
	    this._canvas = $('<canvas/>').appendTo(this.element);
	    if(this.options.clickable) {
		this._canvas.addClass("clickable").click(function(e) {
		    var $canvas = $(this);
		    var off = $canvas.offset(); // e.g. off.left=23.68, off.top=86.77
		    var clickWidth = $canvas.width() - leftInset - rightInset; // e.g. 430
		    var clickHeight = $canvas.height() - topInset - bottomInset; // e.g. 125
		    var x = (e.pageX - off.left - leftInset) / clickWidth;
		    var y = 1 - (e.pageY - off.top - topInset) / clickHeight;
		    // console.log("click (" + x + ", " + y + ") pageX=" + e.pageX + " pageY=" + e.pageY + " canvas=" + $canvas.width() + ", " + $canvas.height() + ", off.left=" + off.left + " off.top=" + off.top);
		    $canvas.trigger('stripchartclick',{x:x,y:y});
		}); 
	    }
	},
	_destory: function() {
	    if(this.options.clickable) this._canvas.removeClass('clickable').unbind('click');
	    this.element.removeClass('stripchart');
	    this.element.empty();
	    delete this._canvas;
	    delete this._legend;
	},
	draw: function(cdata) {
	    var canvas = this._canvas[0];
	    if(!canvas || !canvas.getContext) return;
	    
	    var ctx = canvas.getContext('2d');
            var h = this._canvas.height();
            var w = this._canvas.width();
            var ratio = window.devicePixelRatio;
            if(ratio && ratio > 1) {
		canvas.height = h * ratio;
		canvas.width = w * ratio;
                ctx.scale(ratio,ratio);
            }
	    else {
                canvas.height = h;
                canvas.width = w;
            }	
	    var maxVal = maxValue(cdata.values,cdata.stack);
	    if(maxVal == 0) maxVal = 1;
	    maxVal *= 1.2;
	    drawValueAxis(ctx,h,w,maxVal,this.options,cdata.units);
	    drawTimeAxis(ctx,h,w,maxVal,this.options,cdata.times,cdata.selectedIdx);
	    if(cdata.stack) drawStackedSeries(ctx,h,w,maxVal,this.options,cdata.values,cdata.colors);
            else drawSeries(ctx,h,w,maxVal,this.options,cdata.values,cdata.colors);
	    if(cdata.threshold) drawThreshold(ctx,h,w,maxVal,cdata.threshold);
            if(cdata.legend) addLegend(this,h,w,this.options,cdata.legend.headings,cdata.legend.labels,cdata.colors);
	}
    });
})(jQuery);
