$(function() { 
    var dataURL = null;
    var dataUpdate = false;
    var refresh = 60000;
    var backgroundColor = '#ffffff';
    var default_colors = [
	'#3366cc',
	'#dc3912',
	'#ff9900',
	'#109618',
	'#990099',
	'#0099c6',
	'#dd4477',
	'#66aa00',
	'#b82e2e',
	'#316395',
	'#994499',
	'#22aa99',
	'#aaaa11',
	'#6633cc',
	'#e67300',
	'#8b0707',
	'#651067',
	'#329262',
	'#5574a6',
	'#3b3eac',
	'#b77322',
	'#16d620',
	'#b91383',
	'#f4359e',
	'#9c5935',
	'#a9c413',
	'#2a778d',
	'#668d1c',
	'#bea413',
	'#0c5922',
	'#743411' ];
    var chartData = {};
    var tableData = {};

    var suffixes = ["μ","m",,"K","M","G","T","P","E"];
    function formatValue(value,includeMillis) {
	if(value == 0) return "0.000";
	
	var i = 2;
	var divisor = 1;
	
	if(includeMillis) {
            i = 0;
            divisor = 0.000001;
	}
	
	var absval = Math.abs(value);
	while(i < suffixes.length) {
            if((absval /divisor) < 1000) break;
            divisor *= 1000;
            i++;
	}
	var scaled = Math.round(absval * 1000 / divisor) / 1000;
	return scaled.toFixed(3) + (suffixes[i] ? suffixes[i] : "");
    }
    
    function createChart(chid, newData) {
	// We assume that $(chid) has been set up as a div in the page.
	// It's also the name of the 'metric' that the data is under.

	// copy the colors
	var colors = [];
	colors = colors.concat(default_colors);
	// and allow them to be selectively overridden
	if(newData.colors) {
	    for(var cc = 0; cc < newData.colors.length; cc++) {
		var override = newData.colors[cc];
		if(override) colors[cc] = override;
	    }
	}

	$('#'+chid).chart({
	    type: 'topn',
	    stack: newData.stack,
	    sep: newData.separator,
	    metric: chid,
	    legendHeadings: newData.keynames,
	    colors: colors,
	    backgroundColor: backgroundColor,
	    units: newData.units,
	    threshold: newData.threshold
	}, newData); 
    }
    
    function updateChartData(chid, oldData, newData) {
	// update the data object in situ
	if(dataUpdate) {
	    // work backwards from the end to find the first time-bin
	    // that the new data is going to replace.
	    var newDataStart = newData.trend.times[0];
	    var cur = oldData.trend.times.length;
	    var chop = 0;
	    for(; cur >= 0; cur--) {
		// use <= so that if there was a gap then we just append
		if(oldData.trend.times[cur] <= newDataStart) break;
		chop++;
	    }
	    // how many to age out from the left - i.e. how much is time advancing on the right
	    var age = newData.trend.times.length - chop;
	    // age on the left and chop the overwritten data on the right
	    oldData.trend.times = oldData.trend.times.slice(age, cur);
	    oldData.trend.trends[chid] = oldData.trend.trends[chid].slice(age, cur);
	    // now append the new data (overwritten + new)
	    oldData.trend.times = oldData.trend.times.concat(newData.trend.times);
	    oldData.trend.trends[chid] = oldData.trend.trends[chid].concat(newData.trend.trends[chid]);
	}
	else {
	    oldData.trend.times = newData.trend.times;
	    oldData.trend.trends[chid] = newData.trend.trends[chid];
	}
    }

    function createTable(chid, newData, tailnew) {
	var colNames = newData.keynames;

	var panel = $('<div id="topn"></div>').addClass('topn');
	var table = $('<table></table>').addClass('topntable');
	if(colNames) {
	    var headers = $('<thead></thead>').addClass('topnhead');
	    for(var ll in colNames) {
		var hdr = $('<th></th>').addClass('topnheadcell').text(colNames[ll]);
		headers.append(hdr);
	    }
	    table.append(headers);
	}
	
	var rows = newData.rows;
	if(rows) {
	    var odd = true;
	    for(var rr=0; rr < rows.length; rr++) {
		var row = $('<tr></tr>').addClass(odd ? 'odd' : 'even');
		// if(rr < tailnew) row.addClass('tailnew'); // this seems to affect all rows?
		var fields = rows[rr];
		var nvalues = newData.nvalues;
		var nkeys = fields.length - nvalues;
		// key fields
		for(var col=0; col < nkeys; col++) {
                    var cellvalue = fields[col];
                    if(!cellvalue) cellvalue = "null";
		    var cell = $('<td></td>').addClass('topnkeycell').html(cellvalue);
		    row.append(cell);
		}
		// value fields
		for(var col=nkeys; col < fields.length; col++) {
		    var valstr;
		    if(newData.format && newData.format[col - nkeys]) {
			// pre-formatted
			valstr = fields[col];
		    }
		    else {
			// apply default formatting
			valstr = formatValue(fields[col], false);
		    }
		    var vcell =  $('<td></td>').addClass('topnvalcell').text(valstr);
		    row.append(vcell);

		    if(newData.threshold) {
			var num = parseFloat(fields[col]);
			if(num && num > newData.threshold) {
			    row.addClass("threshold");
			}
		    }

		}
		table.append(row);
		odd = !odd;
	    }
	}
	panel.append(table);
	$('#'+chid).empty().append(panel);
    }
    
    function updateTableData(chid, oldData, newData) {
	var tailnew = 0;
	if(newData.tail) {
	    // may need to keep some rows from before
	    // so we have the most recent N rows.  When
	    // .tail is set the rows are sent to us in
	    // newest-first order.
	    tailnew = newData.rows.length;
	    var keep = tailnew - newData.n;
	    if(keep > 0) {
		newData.rows = newData.rows.concat(oldData.rows.slice(0, keep));
	    }
	}
	// now build the table in the same way as before,
	// except that now we may choose to highlight the newly
	// added rows.
	createTable(chid, newData, tailnew);
    }

    function updateData(data) {
	if(typeof(data) == 'string') {
	    // get here if the dataURL content was html or txt
	    // rather than JSON
	    data = JSON.parse(data);
	}
        if(!data.ok) {
	    console.log(data.error);
	    return;
	}
	if(data.refresh != null) {
	    refresh = data.refresh * 1000;
	}
	// create or update each chart/table
	for(var chid in data.charts) {
	    var newData = data.charts[chid];
	    if(newData.divclass == "trend") {
		var oldData = chartData[chid];
		if(oldData) updateChartData(chid, oldData, newData);
		else {
		    createChart(chid, newData);
		    chartData[chid] = newData;
		}
	    }
	    else if(newData.divclass == "topntable") {
		var oldData = tableData[chid];
		if(oldData) updateTableData(chid, oldData, newData);
		else {
		    createTable(chid, newData, 0);
		    tableData[chid] = newData;
		}
	    }
	}
	// kick them all to redraw
	$.event.trigger({type:'updateChart'});
    }
    
    function pollTrends() {
	$.ajax({
	    url: dataURL,
            data: null,
	    success: function(data) {
                updateData(data);
                if(refresh) {
		    // all future queries are updates
		    dataUpdate = true;
		    dataURL + "&update=yes";
		    setTimeout(pollTrends, refresh);
		}
            },
            error: function(result,status,errorThrown) {
		setTimeout(pollTrends,5000);
            },
            timeout: 120000
        });
    };
    
    $(window).resize(function() {
        $.event.trigger({type:'updateChart'});
    });
    
    $(document).ready(function() {
	var query = window.location.search;
	var sep = query ? "&" : "?";
	dataURL = window.location.href + sep + "action=data";
        pollTrends();
    });
});
