//Copyright (c) InMon Corp. ALL RIGHTS RESERVED

(function($) {
    /**
     * class: $.jqplot.ScaledYAxisRenderer
     * Renders y2axis using the same tick marks as y axis but applying a 
     * scale factor to the yaxis tick values to obtain the tick values for
     * this axis. Can be used, for example, to express yaxis bits/s in terms
     * of utilisation
     * 
     * Example usage:
     * plot2 = $.jqplot("chart2", [s1, s2], {
     *      axes: {
     *          y2axis = {
     *              label: "Utilization (%)",
     *              show: true,
     *              renderer: $.jqplot.ScaledYAxisRenderer,
     *              rendererOptions: {
     *                  scaleFactor: settings.y2axis.scaleFactor// eg 1/10M
     *              },
     *              tickOptions: {
     *                  formatString: '%g', 
     *                  formatter: util.getValueFormatter(false)
     *              }
     *          }
     *      }
     * });
     * 
     */
    $.jqplot.ScaledYAxisRenderer = function() {
        $.jqplot.LinearAxisRenderer.call(this);
     };
    
    $.jqplot.ScaledYAxisRenderer.prototype = new $.jqplot.LinearAxisRenderer();
    $.jqplot.ScaledYAxisRenderer.prototype.constructor = 
        $.jqplot.ScaledYAxisRenderer;
    
    $.jqplot.ScaledYAxisRenderer.prototype.init = function(options) {
        //prop: factor
        //factor to scale yaxis tick values by to obtain this y2axis tick values
        this.scaleFactor = 1.0;
        $.extend(true, this, {}, options);
    };
    
    /**
     * Creates the ticks using the ticks from yaxis and applying 
     * this.scaleFactor
     * to the values.
     * Assumes that yaxis ticks have already been generated!
     * Called with scope of axis.
     */
    $.jqplot.ScaledYAxisRenderer.prototype.createTicks = function(plot) {
        var ticks = this._ticks,
            yaxis = plot.axes.yaxis,
            factor = this.scaleFactor,
            yticks, yticksLength, i, ytick, tick, value;
        if (yaxis) {
            yticks = yaxis._ticks;
            yticksLength = yticks.length;
            if (yticksLength) {
                for (i = 0; i < yticksLength; i++) {
                    ytick = yticks[i];
                    value = ytick.value * factor;
                    tick = new this.tickRenderer(this.tickOptions);
                    tick.value = value;
                    tick.setTick(value, this.name);
                    ticks.push(tick);
                }
                this.min = yaxis.min * factor;
                this.max = yaxis.max * factor;
                this.numberTicks = yaxis.numberTicks;
            }
        }
    };
    
})(jQuery);