// Copyright (c) 2012 InMon Corp. ALL RIGHTS RESERVED

(function($) {
  $.fn.charts = function(settings) {
    
    var defaults = {
        height         : 180,
        width          : 500,
        largeHeight    : 300,
        columns        : 2,
        legendRows     : 1,
        refresh        : false,
        rebuild        : true,
        large          : false,
        dataTimeout    : 600000
    };
    var option = $.extend(defaults,settings);

    var myID = $(this).attr('id');
    var myDiv = $(this);

    if(option.large) {
       $(window).resize(function() {
          var plot = $('#chart0').data('jqplot');
          if(plot) plot.replot();
       });
    }

    if(option.dataURL) {
         getData();
    }

   function removeLoading() {
     $("#loading").empty().hide();
     $("#content").css("min-height","0");
     return true;
   }

   var loadingRemoved = false;
   function getData() {
     $.ajax({
        url: option.dataURL, 
        success: function(data) {
          if(!loadingRemoved) { loadingRemoved = removeLoading(); }
          drawCharts(data.data);
          if(option.refresh) setTimeout(getData, 60000);    
        },
        error: function(result,status,errorThrown) {
             myDiv.empty();
             $("#loading").empty().append('<span class="warn">' + status + (errorThrown ? '&nbsp;' + errorThrown : '') + '</span>').show();
        },
        timeout: parseInt(option.dataTimeout)
     });
   };

   function processResponse(response,data) {
     if(!response) return;

     switch(response.action) {
       case "loadpage":
         if(response.url) window.location = response.url;
         break;
       default:
     }
   }

   function chartEventHandler(event, gridpos, datapos, neighbor, plot) {
     var nodeClick = $(this).data("nodeClick");
     if(nodeClick) {
       $.getJSON(option.dataURL + "&_evtName=nodeClick&_evtId=" + escape(nodeClick.id) + "&_evtValue=" + escape(nodeClick.value ? nodeClick.value : ""), function(response) {
         processResponse(response,nodeClick);
       });
     }
   }

   var initialized = false;

   function drawCharts(data) {
      if(!initialized || option.rebuild) {
         initCharts(data);
         initialized = true;
      }
      updateCharts(data);
   }

   var suffixes = ["μ","m",,"K","M","G","T","P","E"];
   function formatValue(format,value,includeMillis) {
      if(value == 0) return $.jqplot.sprintf(format,value);

      var i = 2;
      var divisor = 1;

      if(includeMillis) {
         i = 0;
         divisor = 0.000001;
      }

      var absval = Math.abs(value);
      while(i < suffixes.length) {
         if((absval /divisor) < 1000) break;
         divisor *= 1000;
         i++;
      }
      var scaled = Math.round(absval * 1000 / divisor) / 1000;
      return $.jqplot.sprintf(format,scaled) + (suffixes[i] ? suffixes[i] : "");
   }

   function getValueFormatter(suppressMillis) {
     return function(format,value) {
       return formatValue(format,value,suppressMillis); 
     }
   }

   function initCharts(data) {
      var ncols = option.columns;
      var charts;
      if(ncols > 0) {
        charts = '<table><tbody><tr>';
        for(var i = 0; i < data.length; i++) {
          if(i > 0 && i % ncols == 0) charts += '</tr><tr>';
          charts += '<td class="padChart"><div id="chart' + i + '" style="height:' + option.height + 'px;width:' + option.width + 'px;"></div></td>';
        };
        charts += '</tr></tbody></table>';
      } else {
        charts = ''
        if(option.large && data.length == 1) {
          charts += '<div id="chart0" style="height:' + option.largeHeight + 'px;width:100%;"></div>';
        } else {
          for(var i = 0; i < data.length; i++) charts += '<div id="chart' + i + '" class="floatChart" style="height:' + option.height + 'px;width:' + option.width + 'px;"></div>';
          charts += '<div class="clearChart"></div>';
        }
      }
      myDiv.empty();
      myDiv.append(charts);
      for(var i = 0; i < data.length; i++) {
        var chart = data[i];
        if(chart.nodeClick) {
          $('#chart' + i).data("nodeClick", chart.nodeClick)
                         .bind("jqplotClick",chartEventHandler);
        }
      }
   }

   function updateCharts(data) {
      for(var i = 0; i < data.length; i++) {
        var options = {
          axesDefaults: {},
          seriesDefaults: {
            showMarker: false,
            renderer: $.jqplot.MirrorSeriesRenderer,
            rendererOptions: {
              showAsLine: true,
              highlightMouseOver: false,
              highlightMouseDown: false,
              hightlistColor: null
            }
          },
          axes:{xaxis:{}, yaxis:{}}
        };
	var chart = data[i];
        var type = chart.type;
        if(!option.large || data.length > 1) options.seriesDefaults.lineWidth = 1;
        if(chart.title) options.title = chart.title;
	if(chart.xaxis.label && 'trend' != type) options.axes.xaxis.label = chart.xaxis.label;
 	if(chart.yaxis.label) options.axes.yaxis.label = chart.yaxis.label;
        options.axes.yaxis.tickOptions = { formatString: '%g', formatter: getValueFormatter(chart.includeMillis) };
        if(chart.stack || chart.mirror) {
           options.stackSeries = true;
           options.seriesDefaults.fill = true;
           options.seriesDefaults.fillAndStroke = true;
           options.seriesDefaults.fillToZero = true;
        };
	var lines = [];

        switch(type) {
          case 'trend': 
            options.axesDefaults.labelRenderer = $.jqplot.CanvasAxisLabelRenderer;
	    options.axesDefaults.pad = 0;
	    options.axes.yaxis.labelOptions = { fontSize: '10pt', fontFamily: 'Arial, Helvetica, sans-serif' };
            options.axes.xaxis.renderer = $.jqplot.TimeAxisRenderer;
            options.axes.xaxis.rendererOptions = {};
	    options.axes.xaxis.rendererOptions.timestamps = $.map(chart.xaxis.data,function (val) { return (new Date(val)).getTime(); });
            options.axes.xaxis.ticks = chart.timeLabels;
           
            var tvals = chart.xaxis.data;
            var labels = [];
	    if(chart.mirror) {
		//options.axes.yaxis.renderer = $.jqplot.MultiLabelAxisRenderer;
                options.series = [];
		// positive values
		options.showMirrorLabels = true;
		for(var s = 0; s < chart.yaxis.series.length; s+=2) {
		    var yvals = chart.yaxis.series[s].values;
                    labels.push(chart.yaxis.series[s].name);
		    lines.push(yvals);
                    options.series.push({label:chart.yaxis.series[s].name});
                }
		// negative values
                var mirrorIdx = 0;
                for(var s = 1; s < chart.yaxis.series.length; s+=2) {
		    var yvals = $.map(chart.yaxis.series[s].values, function(y) { return -y;});
		    labels.push(chart.yaxis.series[s].name);
		    lines.push(yvals);
                    options.series.push({label:chart.yaxis.series[s].name, mirrorIndex: mirrorIdx++});
		}
            } else {
		options.axes.yaxis.min = 0;
		for (var s = 0; s < chart.yaxis.series.length; s++) {
		    var yvals = chart.yaxis.series[s].values;
		    labels.push(chart.yaxis.series[s].name);
		    lines.push(yvals);
		}
	    }
            if(chart.colors) options.seriesColors = chart.colors.split(",");
            options.legend = {
               renderer: $.jqplot.EnhancedLegendRenderer,
               rendererOptions: {
                  numberRows: option.legendRows,
                  seriesToggle: false },
               show: true,
               placement: 'outsideGrid',
               location: 's',
               labels: labels };
            break;
          
        }
        var plot = $('#chart' + i).data('jqplot');
	if(plot) plot.destroy();
        $('#chart' + i).empty().jqplot(lines,options);
      }
   }

   return this;
  }
})(jQuery);
