(function() {

    var id;
    var option;
    var graphControl;
    var overviewControl;
    var myData;

    // styles
    var nodeStyles = {};
    function getNodeStyle(shape,color) {
	var shape = shape ? shape : "rounded";
        var color = color ? color : "LIGHT_GREEN";
	var key = shape + "-" + color;
	var style = nodeStyles[key];
	if(!style) {
	    var fillColor = yfiles.system.Brushes[color.toUpperCase()];
	    if(!fillColor) fillColor = yfiles.system.Brushes.LIGHT_BLUE;
	    var penColor = yfiles.system.Brushes.DARK_BLUE;
	    var penWidth = 2;
	    switch(shape) {
	    case "ellipse":
		style = new yfiles.drawing.ShapeNodeStyle.WithShapePenAndBrush(yfiles.drawing.ShapeNodeShape.ELLIPSE, new yfiles.system.Pen.FromBrushAndThickness(penColor, penWidth), fillColor);
		break;
	    case "rectangle":
		style = new yfiles.drawing.ShapeNodeStyle.WithShapePenAndBrush(yfiles.drawing.ShapeNodeShape.RECTANGLE, new yfiles.system.Pen.FromBrushAndThickness(penColor, penWidth), fillColor);
		break;
	    case "rounded":
		style = new yfiles.drawing.ShapeNodeStyle.WithShapePenAndBrush(yfiles.drawing.ShapeNodeShape.ROUND_RECTANGLE, new yfiles.system.Pen.FromBrushAndThickness(penColor, penWidth), fillColor);
		break;
	    case "hexagon":
		style = new yfiles.drawing.ShapeNodeStyle.WithShapePenAndBrush(yfiles.drawing.ShapeNodeShape.HEXAGON, new yfiles.system.Pen.FromBrushAndThickness(penColor, penWidth), fillColor);
		break;
	    case "octogon":
		style = new yfiles.drawing.ShapeNodeStyle.WithShapePenAndBrush(yfiles.drawing.ShapeNodeShape.OCTAGON, new yfiles.system.Pen.FromBrushAndThickness(penColor, penWidth), fillColor);
		break;
	    case "diamond":
		style = new yfiles.drawing.ShapeNodeStyle.WithShapePenAndBrush(yfiles.drawing.ShapeNodeShape.DIAMOND, new yfiles.system.Pen.FromBrushAndThickness(penColor, penWidth), fillColor);
		break;
	    case "shiny":
	    default: 
		style = new yfiles.drawing.ShinyPlateNodeStyle.WithColor(fillColor);
		style.drawShadow = true;
	    }
	    nodeStyles[key] = style;
	}
	return style;
    }

    var edgeStyles = {};
    function getEdgeStyle(color,width) {
       var color = color ? color : 'BLACK';
       var width = width ? width : 1;
       var key = color + "-" + width;
       var style = edgeStyles[key];
       if(!style) {
         var penColor = yfiles.system.Brushes[color.toUpperCase()];
         if(!penColor) penColor = yfiles.system.Brushes.BLACK;
         style = graphControl.graph.edgeDefaults.style.clone();
         style.pen = new yfiles.system.Pen.FromBrushAndThickness(penColor, width);
       }
       return style;
    }

    function click(evtName,evtId,evtValue) {
	$.getJSON(option.dataURL + "&_evtName=" + escape(evtName) + "&_evtId=" + escape(evtId) + "&_evtValue=" + escape(evtValue), function(response) {
		processResponse(response);
	});
    }
	
    function itemClicked(sender, e) {
	$('#details').empty();
        var tag = e.item.tag;
        if(!tag) return;

	switch(tag.type) {
	    case "group":
		break;
	    case "node":
		click("nodeClick",tag.node.evtId,tag.node.evtValue ? tag.node.evtValue : tag.id);
		break;
	    case "edge":
		click("edgeClick",tag.edge.evtId, tag.edge.evtValue ? tag.edge.evtValue : tag.id);
		break;
	}
    }      

    var maxLabelWidth = 0;
    var labelWidthPad = 10;
    var defaultNodeWidth = 100;
    var defaultNodeHeight = 30;

    function displayGraph() {	
	
	if(myData && myData.graph) {
	    
	    var nodeByTag = {};
	    if(myData.graph.nodes) {
		
		var nodeDefaults = graphControl.graph.nodeDefaults;
                if(myData.graph.exteriorNodeLabels) {
                   nodeDefaults.size = new yfiles.geometry.SizeD(100, 25);
                   nodeDefaults.labels.labelModelParameter = yfiles.drawing.ExteriorLabelModel.SOUTH;
                } else {
                   nodeDefaults.size = new yfiles.geometry.SizeD(100, 25);
		   nodeDefaults.labels.labelModelParameter = yfiles.drawing.InteriorLabelModel.CENTER;
                }
		
		nodeDefaults.style = getNodeStyle();
		
		for (var nodeKey in myData.graph.nodes) {
		    var node = myData.graph.nodes[nodeKey];
		    var nd = graphControl.graph.createNodeWithBoundsAndStyle(new yfiles.geometry.RectD(0,0,defaultNodeWidth,defaultNodeHeight), getNodeStyle(node.shape,node.color));
		    if(nd) {
			nodeByTag[nodeKey] = nd;
			nd.tag = { type: "node", id: nodeKey, node:node };
			if(node.label) {
			    var label = graphControl.graph.addLabel(nd, node.label);
			    var labelWidth = label.preferredSize.width + labelWidthPad;
			    maxLabelWidth = Math.max(maxLabelWidth,labelWidth);
			    if(labelWidth > defaultNodeWidth) graphControl.graph.setBounds(nd,new yfiles.geometry.RectD(0,0,labelWidth,defaultNodeHeight));
			}
		    }
		}
	    }
	    if(myData.graph.edges) {
		if(myData.graph.digraph) graphControl.graph.edgeDefaults.style.targetArrow = yfiles.drawing.DefaultArrow.DEFAULT;
		else graphControl.graph.edgeDefaults.style.targetArrow = yfiles.drawing.DefaultArrow.NONE;
		var labelModel1 = yfiles.drawing.NinePositionsEdgeLabelModel.SOURCE_CENTERED;
		var labelModel2 = yfiles.drawing.NinePositionsEdgeLabelModel.TARGET_CENTERED;
		for(var edgeKey in myData.graph.edges) {
		    var edge = myData.graph.edges[edgeKey];
		    var nd1 = nodeByTag[edge.node1];
		    var nd2 = nodeByTag[edge.node2];
		    if(nd1 && nd2) {
			var ed = graphControl.graph.createEdge(nd1, nd2);
			if(ed) {
                            ed.tag = { type: "edge", edge: edge,  id: edgeKey };
			    if(edge.name1) graphControl.graph.addLabelWithParameter(ed, labelModel1, edge.name1);
			    if(edge.name2) graphControl.graph.addLabelWithParameter(ed, labelModel2, edge.name2);
			    if(edge.label) graphControl.graph.addLabel(ed,edge.label);
                       
			    graphControl.graph.setEdgeStyle(ed,getEdgeStyle(edge.color,edge.weight));
                        }
		    }
		}
	    }
	}

	setLayout($("#layout").val());
    }

    function updateMap(newData) {
	if(!myData || !myData.graph) return;

	if(!newData || !newData.graph) return;

	var nodeByTag = {};

	var nodesToAdd = {};
	var edgesToAdd = {};
	
	var rmNodes = [];
	var e = graphControl.graph.nodes.getEnumerator();
	while(e.moveNext()) {
	    var node = e.current;
	    var id = node.tag.id;
            var newNode = newData.graph.nodes[id];
	    if(newNode) {
                graphControl.graph.setNodeStyle(node,getNodeStyle(node.tag.node.shape,newNode.color));
		nodeByTag[id] = node;
	    } else rmNodes.push(node);
	}
	for (var i = 0; i < rmNodes.length; i++)  graphControl.graph.removeNode(rmNodes[i]);
	
	var rmEdges = [];
	e = graphControl.graph.edges.getEnumerator();
	while(e.moveNext()) {
	    var edge = e.current;
	    var id = edge.tag.id;
            var newEdge = newData.graph.edges[id];
	    if(newEdge) {
                graphControl.graph.setEdgeStyle(edge,getEdgeStyle(newEdge.color,newEdge.weight));
	    } else rmEdges.push(edge);
	}
	for (var i = 0; i < rmEdges.length; i++)  graphControl.graph.removeEdge(rmEdges[i]);

	// add new nodes / edges
	if(newData.graph.nodes) {
	    for(var nodeKey in newData.graph.nodes) {
		var node = newData.graph.nodes[nodeKey];
		if(!myData.graph.nodes || !myData.graph.nodes[nodeKey]) {
		    var nd = graphControl.graph.createNodeWithBoundsAndStyle(new yfiles.geometry.RectD(0,0,defaultNodeWidth,defaultNodeHeight), getNodeStyle(node.shape,node.color));
		    if(nd) {
			nodeByTag[nodeKey] = nd;
			nd.tag = { type: "node", id: nodeKey, node:node };
			if(node.label) {
			    var label = graphControl.graph.addLabel(nd, node.label);
			    var labelWidth = label.preferredSize.width + labelWidthPad;
			    maxLabelWidth = Math.max(maxLabelWidth,labelWidth);
			    if(labelWidth > defaultNodeWidth) graphControl.graph.setBounds(nd,new yfiles.geometry.RectD(0,0,labelWidth,defaultNodeHeight));
			}
		    }
		}
	    }
	}
	if(newData.graph.edges) {
	    for(var edgeKey in newData.graph.edges) {
		var edge = newData.graph.edges[edgeKey];
		if(!myData.graph.edges || !myData.graph.edges[edgeKey]) {
		    // add edge		    
		    var nd1 = nodeByTag[edge.node1];
		    var nd2 = nodeByTag[edge.node2];
		    if(nd1 && nd2) {
			var ed = graphControl.graph.createEdge(nd1, nd2);
			if(ed) {
                            ed.tag = { type: "edge", edge: edge,  id: edgeKey };
			    if(edge.name1) graphControl.graph.addLabelWithParameter(ed, labelModel1, edge.name1);
			    if(edge.name2) graphControl.graph.addLabelWithParameter(ed, labelModel2, edge.name2);
			    if(edge.label) graphControl.graph.addLabel(ed,edge.label);
                       
			    graphControl.graph.setEdgeStyle(ed,getEdgeStyle(edge.color,edge.weight));
                        }
		    }
		}
	    }
	}

	myData = newData;

	// maybe this should be conditional - suppose we are just polling for link status / width
	if(myData.refreshLayout) setLayout($("#layout").val());
    }

    function detailsClickAction(event) {
	var data = $(this).data("evt");
	if(data) {
	    $.getJSON(option.dataURL + "&_evtName=dialogClick&_evtId=" + escape(data.evtId) + "&_evtValue=" + escape(data.evtValue ? data.evtValue : data.value), function(response) {
		    processResponse(response,null);
	    });
	}
    }

    function updateDetails(response) {
	if(!response.values) return;

	var cdata = [];

	var html = '<table>';
	if(response.title) html += '<thead><tr><th colspan="2">' + response.title + '</th></tr></thead>';
	html += '<tbody>';
	for(var r = 0; r < response.values.length; r++) {
	    var label = response.values[r].label;
	    var value = response.values[r].value;
	    var evtId = response.values[r].evtId;
	    var evtValue = response.values[r].evtValue;
	    if(!(label && value)) continue;

	    html += '<tr class="' + (r % 2 == 0 ? "even" : "odd") + '">';
	    html += '<td class="alignr">' + label + '</td>';
	    html += '<td class="alignl">' ;
	    if(evtId) html += '<a href="javascript:void(0)">';
	    html += value;
	    if(evtId) html += '</a>';
	    if(evtId) cdata.push(response.values[r]);
	    html += '</td>';
	    html += '</tr>';
	}
	html += '</tbody>';
	html += '</table>';
	$('#details').html(html).find('a').each(function(i) { $(this).data("evt",cdata[i]).click(detailsClickAction) });
    }
    
    function processResponse(response) {
	if(!response) return;
	
	switch(response.action) {
        case "loadgraph":
	    updateMap(response);
            break;
	case "loadpage":
	    if(response.url) window.location = response.url;
	    break;
	case "dialog":
	    updateDetails(response);
	    break;
	default:
	}
    }

    function setLayout(layout) {	
	var layouter;
	switch(layout) {
        case "organic" :
            layouter = new yfiles.organic.SmartOrganicLayouter();
            layouter.nodeOverlapsAllowed=false;
            layouter.minimalNodeDistance=10;
            layouter.compactess=0.5;
            layouter.deterministic=true;
            break;
	case "othogonal":
	    layouter = new yfiles.orthorgonal.OrthogonalLayouter();
	    break;
	case "tree":
	    layouter = new yfiles.tree.TreeLayouter();
	    break;
	case "circular":
	    layouter = new yfiles.circular.CircularLayouter();
	    break;
	case "hierarchic":
	    layouter = new yfiles.hierarchic.IncrementalHierarchicLayouter();
	    break;
	default:
	    layouter = new yfiles.hierarchic.OrganicLayouter();
	    break;
	}
	
	// disable controls?
	yfiles.graph.LayoutExtensions.applyLayoutWithControlAndCallback(graphControl.graph, layouter, yfiles.system.TimeSpan.fromSeconds(2), graphControl, function layoutDoneHandler() {
	    // actions when done
	    // enable controls?
	    // send event back to server to remember the setting as default
	    click("layout", "layout", layout);
	    
	});  
    }

    function controlPanel() {
	var html = '<form action="" class="subfilterbar" id="subfilterForm">';
	html += '<fieldset>';
       
	html += '<label>' + option.panLabel+ '</label>&nbsp;';
	html += '<button type="button" id="panLeft">' + option.leftLabel + '</button>';
	html += '<button type="button" id="panRight">' + option.rightLabel + '</button>';
	html += '<button type="button" id="panUp">' +  option.upLabel + '</button>';
	html += '<button type="button" id="panDown">' + option.downLabel + '</button>';
       	
	html += '<label>' + option.zoomLabel + '</label>&nbsp;';
	html += '<button type="button" id="zoomIn">' + option.inLabel + '</button>';
	html += '<button type="button" id="zoomOut">' + option.outLabel + '</button>';

	html += '<label>' + option.layoutLabel + '</label>&nbsp;';
	html += '<select id="layout" name="layout">';
	html += '<option value="organic">' + option.organicLabel + '</option>';
	html += '<option value="circular">' + option.circularLabel +  '</option>';
	html += '<option value="hierarchic">' + option.hierarchicLabel + '</option>';
	html += '</select>';
        html += '<button type="button" id="zoomReset" >' + option.resetZoomLabel + '</button>';
       
	html += '</fieldset>';
	html += '</form>';
	return html;
    }

    function initializeMap() {
	var html = controlPanel();
	html += '<div id="sidePanel">';
	html += '<div id="overview"></div>';
	html += '<div id="details"></div>';
	html += '</div>';
	html += '<div style="height:' + option.height + 'px" id="map"></div>';

        $("#" + id).empty().height('auto').append(html);
	graphControl = new yfiles.canvas.GraphControl.ForId("map");
        overviewControl = new yfiles.canvas.GraphOverviewControl.ForId("overview");
	overviewControl.graphControl =graphControl;

	// controller
	var inputMode = new yfiles.input.GraphViewerInputMode();
	inputMode.selectableItems = yfiles.graph.GraphItemTypes.NODE | yfiles.graph.GraphItemTypes.EDGE;
	inputMode.addItemClickedListener(itemClicked);
	graphControl.inputModes.add(inputMode);

	$("#layout").val(option.layout).change(function() { setLayout($(this).val()); });
	$("#panRight").click(function() { yfiles.system.ComponentCommands.SCROLL_PAGE_RIGHT.executeOnTarget(0.1,graphControl); });
	$("#panLeft").click(function() { yfiles.system.ComponentCommands.SCROLL_PAGE_LEFT.executeOnTarget(0.1,graphControl); });
	$("#panUp").click(function() { yfiles.system.ComponentCommands.SCROLL_PAGE_UP.executeOnTarget(0.1,graphControl); });
	$("#panDown").click(function() { yfiles.system.ComponentCommands.SCROLL_PAGE_DOWN.executeOnTarget(0.1,graphControl); });
	$("#zoomReset").click(function() { graphControl.fitContent(); });
	$("#zoomIn").click(function() { yfiles.system.NavigationCommands.INCREASE_ZOOM.executeOnTarget(2,graphControl); });
	$("#zoomOut").click(function() { yfiles.system.NavigationCommands.DECREASE_ZOOM.executeOnTarget(2,graphControl); });

	$(window).resize(function() { graphControl.fitContent(); });
    }

    var mapInitialized = false;
    function drawMap() {
	if(!mapInitialized) {
	    initializeMap();
	    mapInitialized = true;
	}
        displayGraph();
    }

    function getData() {
	$.ajax({
	    url: option.dataURL + (myData ? "&refresh=true" : ""),
	    success: function(data) {
		    if(myData) {
                      updateMap(data);
                    } else {
                      myData = data;
                      drawMap();
                    }
                    if(option.refresh) setTimeout(getData, 60000);	     
             },
	     error: function(result,status,errorThrown) {
		    $('#' + id).empty().append('<span class="warn">' + status + (errorThrown ? '&nbsp;/&nbsp;' + errorThrown : '') + '</span>');
	     },
	      timeout: option.dataTimeout
	 });
    }

    $.fn.graph = function(settings) {
	var defaults = {
	    height        : 400,
	    refresh       : false,
	    dataTimeout   : 600000,
            layout        : "organic",
	    panLabel    : "pan",
	    leftLabel     : "left",
	    rightLabel   : "right",
	    upLabel      : "up",
	    downLabel : "down",
	    zoomLabel : "zoom",
	    inLabel       : "in",
	    outLabel    : "out",
	    layoutLabel : "layout",
	    organicLabel : "organic",
	    orthogonalLabel : "orthogonal",
	    treeLabel    : "tree",
	    circularLabel : "circular",
	    hierarchicLabel : "hierarchic",
	    resetZoomLabel : "reset"
	};
	option = $.extend(defaults,settings);
	id = $(this).attr('id');

	require.baseUrl = 'inc/yfiles/';
	require(['inc/yfiles/license.js', 'inc/yfiles/yfiles/complete.js' ], function () { getData(); });
    }
})(jQuery);
