function DashKey(name, field, formatFn) {
    this.name = name;
    this.field = field;
    this.formatFn = formatFn;
}

function DashVal(units, field, scale, format, color) {
    this.units = units;
    this.field = field;
    this.scale = scale;
    this.format = format;
    this.color = color;
}

function DashCell(view, keys, values, filter) {
    this.view = view;
    this.keys=keys;
    this.values=values;
    this.filter=filter;
    // rest inherited from Dash.common
}

function Dash(title, columns, width, height) {
    this.title = title || "Dashboard";
    this.columns=columns || 2;
    this.cells = [];
    this.cellWidth = width || "800px";
    this.cellHeight = height || "150px";

    // additional properties not normally set
    this.legendopacity="0.8";
    this.refresh = 60;

    // placeholder for network object
    this.net = null;
    // debug log
    this.log = [];
    // name cache
    this.nameCache = {};

    // cell defaults
    this.common = new DashCell("rttraffic", [], [], null);
    this.common.interval = null;
    this.common.minutes=60;
    this.common.update_minutes=10;
    this.common.group=1; // time-grouping
    this.common.N=5;
    this.common.globaltruncate=false;
    this.common.includeother=false;
    this.common.nullsinkeys=false;
    this.common.table=false;
    this.common.stack=true;
    this.common.title=null;
    this.common.threshold=null;
}

DashCell.prototype.clone = function() {
    var cell = new DashCell(this.view, this.keys, this.values, this.filter);
    cell.interval = this.interval;
    cell.minutes = this.minutes;
    cell.update_minutes = this.update_minutes;
    cell.group = this.group;
    cell.N = this.N;
    cell.globaltruncate = this.globaltruncate;
    cell.includeother = this.includeother;
    cell.nullsinkeys = this.nullsinkeys;
    cell.table = this.table;
    cell.stack = this.stack;
    cell.title = this.title;
    cell.threshold = this.threshold;
    return cell;
}

DashCell.prototype.setView = function(view) { this.view = view; }
DashCell.prototype.getView = function() { return this.view; }

DashCell.prototype.setKeys = function(keys) { this.keys = keys; }
DashCell.prototype.getKeys = function() { return this.keys; }

DashCell.prototype.setValues = function(values) { this.values = values; }
DashCell.prototype.getValues = function() { return this.values; }

DashCell.prototype.setFilter = function(filter) { this.filter = filter; }
DashCell.prototype.getFilter = function() { return this.filter; }

DashCell.prototype.setMinutes = function(mm) { this.minutes = mm; }
DashCell.prototype.getMinutes = function() { return this.minutes; }

DashCell.prototype.setUpdateMinutes = function(mm) { this.update_minutes = mm; }
DashCell.prototype.getUpdateMinutes = function() { return this.update_minutes; }

DashCell.prototype.setInterval = function(interval) { this.interval = interval; }
DashCell.prototype.getInterval = function() { return this.interval; }

DashCell.prototype.setGroup = function(grp) { this.group = grp; }
DashCell.prototype.getGroup = function() { return this.group; }

DashCell.prototype.setN = function(nn) { this.N = nn; }
DashCell.prototype.getN = function() { return this.N; }

DashCell.prototype.setGlobalTruncate = function(bool) { this.globaltruncate = bool; }
DashCell.prototype.getGlobalTruncate = function() { return this.globaltruncate; }

DashCell.prototype.setIncludeOther = function(bool) { this.includeother = bool; }
DashCell.prototype.getIncludeOther = function() { return this.includeother; }

DashCell.prototype.setNullsInKeys = function(bool) { this.nullsinkeys = bool; }
DashCell.prototype.getNullsInKeys = function() { return this.nullsinkeys; }

DashCell.prototype.setStack = function(bool) { this.stack = bool; }
DashCell.prototype.getStack = function(bool) { return this.stack; };

DashCell.prototype.setThreshold = function(val) { this.threshold = val; }
DashCell.prototype.getThreshold = function() { return this.threshold; };

DashCell.prototype.setTable = function(bool) { this.table = bool; }
DashCell.prototype.getTable = function(bool) { return this.table; }
DashCell.prototype.divclass = function() { return this.table ? "topntable" : "trend"; }

DashCell.prototype.setTitle = function(str) { this.title = str; }
DashCell.prototype.getTitle = function() {
    if(this.title) return this.title;
    // make up a title from the spec
    var str = "";
    if(this.keys.length) {
	var keynames = [];
	for each (var dkey in this.keys) keynames.push(dkey.name);
	str = "Top " + keynames.join() + " By: ";
    }
    else {
	str = "Trend ";
    }
    var valuenames = [];
    for each (var dval in this.values) valuenames.push(dval.units);
    str += valuenames.join();
    var filter = this.getFilter();
    if(filter) str += " Where: " + filter;
    return str;
}

Dash.prototype.getTitle = function() { return this.title; }
Dash.prototype.getColumns = function() { return this.columns; }
Dash.prototype.getCellWidth = function() { return this.cellWidth; }
Dash.prototype.getCellHeight = function() { return this.cellHeight; }

Dash.prototype.setLegendOpacity = function(opacity) { this.legendopacity = opacity; }
Dash.prototype.getLegendOpacity = function() {  return this.legendopacity; }

Dash.prototype.setRefreshTimeout = function(secs) { this.refresh = secs; }
Dash.prototype.getRefreshTimeout = function() { return this.refresh; }

Dash.prototype.addCell = function(cell) {
    this.cells.push(cell);
    // fill in the defaults.
    // Object.assign() from ES6 would make this easier, but
    // that feature doesn't seem to be in our version of rhino.
    if(cell.interval == null) cell.interval = this.common.interval;
    if(cell.minutes == null) cell.minutes = this.common.minutes;
    if(cell.update_minutes == null) cell.update_minutes = this.common.update_minutes;
    if(cell.group == null) cell.group = this.common.group;
    if(cell.N == null) cell.N = this.common.N;
    if(cell.globaltruncate == null) cell.globaltruncate = this.common.globaltruncate;
    if(cell.includeother == null) cell.includeother = this.common.includeother;
    if(cell.nullsinkeys == null) cell.nullsinkeys = this.common.nullsinkeys;
    if(cell.table == null) cell.table = this.common.table;
    if(cell.stack == null) cell.stack = this.common.stack;
    if(cell.title == null) cell.title = this.common.title;
    if(cell.threshold == null) cell.threshold = this.common.threshold;
}

//// formatting functions ////

Dash.prototype.getNetwork = function() {
    if(this.net == null) this.net = Network.current();
    return this.net;
}

Dash.prototype.populateNameCache = function(debug) {
    if(debug) this.log.push("populateNameCache");
    var net = this.getNetwork();
    var cache = this.nameCache;
    var ipList = [];
    for(var ip in cache) ipList.push(ip);
    if(debug) this.log.push("ipList length=" + ipList.length);
    var nameList = [];
    try { nameList = net.dnsMap(ipList, false, 5000); }  catch(e) {
	if(debug) this.log.push("dnsMap exception: " + e);
    }
    if(debug) this.log.push("nameList length=" + nameList.length);
    for(var ii in nameList) {
	cache[ipList[ii]] = nameList[ii];
	if(debug) this.log.push("ipList["+ii+"]=" + ipList[ii] + " nameList["+ii+"]=" + nameList[ii]);
    }
}

Dash.prototype.linkHost = function(go, str) {
    return '<a href="/inmsf/Go?action=host&go=' + go + '">' + str + '</a>';
}
Dash.prototype.linkAgentOrIF = function(go, str) {
    return '<a href="/inmsf/Go?action=agent&go=' + go + '">' + str + '</a>';
}
Dash.prototype.linkProtocol = function(go, str) {
    return '<a href="/inmsf/Go?action=protocol&go=' + go + '">' + str + '</a>';
}

// an object used as a placeholder for a formatted key - used
// here to delay the printing of DNS names so we can look them all up at once.
function DashName(name, cache, linkFn) {
    this.name = name;
    this.cache = cache;
    this.linkFn = linkFn;
}

DashName.prototype.toString = function() {
    var lookup = this.cache[this.name] || this.name;
    return this.linkFn ? this.linkFn(this.name, lookup) : lookup;
}

Dash.prototype.format_dns_name = function(ip) {
    this.nameCache[ip] = ip;
    return new DashName(ip, this.nameCache, this.linkHost);
}

Dash.prototype.format_agent_name = function(ip) {
    var net = this.getNetwork();
    net.path = ip;
    var nm = net.displayName();
    return this.linkAgentOrIF(ip, nm ? nm : ip);
}

Dash.prototype.format_interface_str = function(intf) {
    var net = this.getNetwork();
    net.path = intf;
    var ans = net.agent() + ">" + net.displayName();
    var neigh = net.neighborMap([ intf ]);
    if(neigh && neigh[0]) {
	net.path = neigh[0];
	ans += " -- " + net.agent() + ">" + net.displayName();
    }
    return this.linkAgentOrIF(intf, ans);
}

Dash.prototype.format_host = function(nameOrAddr) {
    return this.linkHost(nameOrAddr, nameOrAddr);
}

Dash.prototype.format_protocol = function(port) {
    var net = this.getNetwork();
    var label = port;
    var map = net.protocolNameMap([port])[0];
    if(map && map != port) {
      var parts = map.split(':');
      if(parts.length == 2) label += " (" + parts[1] + ")";
    }
    return this.linkProtocol(port, label);
  }


Dash.prototype.dataAction = function(update, debug) {
    var separator = "_SEP_";
    var otherstr = "-other-";
    
    if(debug) this.log.push("starting dataAction");

    // utility to scale values in a callback row
    function scaleValues(row, scale) {
    	if(scale.length) {
    	    for(var ss = 0; ss < scale.length; ss++) {
    		var scl = scale[ss];
    		var idx = row.length - scale.length + ss;
    		var val = row[idx];
    		if(val && scl) row[idx] = val * scl;
    	    }
    	}
    }

    function formatValues(row, format, as_str) {
    	if(format.length) {
    	    for(var ss = 0; ss < format.length; ss++) {
    		var fmt = format[ss];
    		var idx = row.length - format.length + ss;
    		var val = row[idx];
    		if(val && fmt) {
    		    if(typeof(val) == "string") val = parseFloat(val);
    		    row[idx] = formatnumber(val, fmt);
		    if(!as_str) row[idx] = parseFloat(row[idx]);
    		}
    	    }
    	}
    }


    function formatKeys(dash, row, cell) {
    	// apply format function translation if present
    	var stop = Math.min(cell.keys.length, row.length);
    	for(var kk = 0; kk < stop; kk++) {
    	    var formatFn = cell.keys[kk].formatFn;
    	    if(formatFn) {
		row[kk] = formatFn.call(dash, row[kk]);
	    }
    	}
    }

    // function used to build query-callback for a query row
    function topNTrendCallback(dash, chart, cell, scale, format) {

    	// utility to represent a compound key as a string
    	function extractKey(row) {
    	    // assume row[0] is time, and keys are next
	    var keyList=[];
    	    var other = true;
    	    if(cell.keys) {
    		for(var kk = 1; kk < (1 + cell.keys.length); kk++) {
    		    if(row[kk]) other = false;
    		    keyList.push(row[kk] || "null");
    		}
    	    }
    	    return other ? [otherstr] : keyList;
    	}

    	return function(row) {
    	    var times = chart.trend.times;
    	    var trend = chart.trend.trends[chart.id];
    	    var tbin = chart.trend.maxpoints-1;
    	    var tim = row[0].getTime();
    	    var keyList = extractKey(row);
	    var key = keyList.join(separator);

    	    scaleValues(row, scale);
    	    formatValues(row, format, false);
    	    // take last value
    	    var val = row[row.length-1];
    	    if(val) chart.trailingNull = false;
    	    if(tbin == -1 || tim != times[tbin]) {
    		// start new time-bin
    		chart.trend.maxpoints++;
    		times.push(tim);
    		trend.push({});
    		tbin++;
    		chart.trailingNull = true;
    	    }
	    // apply key-formatting
    	    formatKeys(dash, keyList, cell);
    	    // add key-value pair to time-bin
    	    var topN = trend[tbin];
	    // keep raw keyList object for now - see unpackTopNKeys()
    	    topN[key] = [keyList, val];
    	};
    }

    // function used to build query-callback for a query row
    // where there are no keys - only one or more values
    function valueTrendCallback(dash, chart, cell, scale, format, units) {

    	return function(row) {
    	    var times = chart.trend.times;
    	    var trend = chart.trend.trends[chart.id];
    	    var tbin = chart.trend.maxpoints-1;
    	    var tim = row[0].getTime();
    	    if(tbin == -1 || tim != times[tbin]) {
    		// start new time-bin
    		chart.trend.maxpoints++;
    		times.push(tim);
    		trend.push({});
    		tbin++;
    		chart.trailingNull = true;
    	    }
    	    scaleValues(row, scale);
    	    formatValues(row, format, false);
    	    for(var vv = 0; vv < units.length; vv++) {
    		var key = units[vv];
    		var val = row[1 + vv];
    		// add key-value pair to time-bin - will this scramble the ordering?
    		var topN = trend[tbin];
    		topN[key] = val;
    		if(val) chart.trailingNull = false;
    	    }
    	};
    }


    function tableQueryCallback(dash, chart, cell, scale, format) {
    	return function(row) {
    	    formatKeys(dash, row, cell);
    	    scaleValues(row, scale);
    	    formatValues(row, format, true);
    	    chart.rows.push(row);
    	};
    }

    function tableTailQueryCallback(dash, chart, cell, scale, format) {
    	return function(row) {
    	    formatKeys(dash, row, cell);
    	    scaleValues(row, scale);
    	    formatValues(row, format, true);
    	    // just want the last N rows, newest first
    	    // (events query sometimes gives more than N)
    	    chart.rows.unshift(row);
    	    if(chart.rows.length > chart.n) {
    		chart.rows.pop();
    	    }
    	};
    }

    // build a multiquery job for each view
    var jobs = {};
    for (var cc in this.cells) {
    	var cell = this.cells[cc];
    	var cellID = "cell" + cc;
    	var view = cell.getView();
    	var job = jobs[view];
    	if(job == null) {
    	    job = jobs[view] = {};

    	    job.q = new Query();
    	    job.q.view = view;
    	    job.q.multiquery = true;
    	    job.q.select = [];
    	    job.q.where = [];
    	    job.q.interval = [];
    	    job.q.group = [];
    	    job.q.sort = [];
    	    job.q.truncate = [];
    	    job.q.includeother = [];
    	    job.q.nullsinkeys = [];
    	    job.q.globaltruncate = [];
    	    job.q.threads = 1;

    	    job.callbacks = [];
    	    job.charts = {};
    	}

     	// construct query for this cell
     	var stack = cell.getStack();
     	var table = cell.getTable();
     	var select = [];
	var keynames = [];
	// for now just assume no time-grouping for tables
     	var group = table ? null : cell.getGroup();

     	// time always comes first, if we are grouping
     	if(group) select.push("time");

     	// then keys
     	for each (var dkey in cell.keys) {
     	    select.push(dkey.field);
     	    keynames.push(dkey.name);
     	}

    	// now add values and their properties
    	var sort = null;
    	var nvalues = cell.values ? cell.values.length : 0;
    	var units = [];
    	var sortUnits = null;
    	var scale = [];
    	var colors = [];
    	var format = [];

    	if(nvalues) {
    	    for each (var dval in cell.values) {
    		select.push(dval.field);
    		scale.push(dval.scale   || 1);
    		units.push(dval.units   || "");
    		format.push(dval.format || null);
    		colors.push(dval.color  || "");
    	    }
    	    sort = select[select.length - 1]; // sort by last value
    	    sortUnits = units[units.length - 1];
    	}

    	// add the cell's query to the multiquery for this view
    	job.q.select.push(select.join(","));
    	job.q.sort.push(sort);
    	job.q.where.push(cell.getFilter());
	var ival = cell.getInterval();
	if(!ival) {
	    if(debug) this.log.push("setting interval from minutes because ival=" + ival);
    	    var minutes = (update == "no" || cell.getGlobalTruncate()) ? cell.getMinutes() : cell.getUpdateMinutes();
	    ival = "now - " + minutes + " min,last";
	}
    	job.q.interval.push(ival);
    	job.q.group.push(group);
    	var truncate = cell.getN();
    	job.q.truncate.push(truncate);
    	job.q.includeother.push(cell.getIncludeOther());
    	job.q.nullsinkeys.push(cell.getNullsInKeys());
    	job.q.globaltruncate.push(cell.getGlobalTruncate());

    	// add keynames, color, units etc. to the chart object so that
    	// they are visible to the poller when charts are constructed
    	var chart = job.charts[cellID] = {
    	    "id": cellID,
    	    "keynames": keynames,
    	    "units": sortUnits,
    	    "separator": separator,
    	    "otherstr": otherstr,
    	    "divclass": cell.divclass(),
    	    "nvalues": nvalues,
    	    "format": format,
    	    "colors": colors,
    	    "n": truncate,
    	    "stack": stack,
	    "threshold": cell.getThreshold()
    	};

    	// now prepare the chart object to receive data row by row as the query runs
    	if(cell.divclass() == "trend") {
    	    // build the chart data structure to match what chart is looking for
    	    chart.trend =  { 'maxpoints': 0, 'times': [], "trends": {}};
    	    chart.trend.trends[cellID] = [];
    	    // and add the query callback to populate it.  Note that we use a
    	    // function-to-build-a-function so that 'chart' is captured in closure
    	    if(cell.keys && cell.keys.length > 0) {
		chart.unpackKeys = true;
    		job.callbacks.push(topNTrendCallback(dash, chart, cell, scale, format));
    	    }
    	    else {
    		job.callbacks.push(valueTrendCallback(dash, chart, cell, scale, format, units));
    	    }
    	}
    	else if(cell.divclass() == "topntable") {
    	    // client side expects just an array of arrays in "chart.rows"
    	    chart.rows = [];
    	    if(nvalues) {
    		// round out the keynames list to be the table column headings
    		if(group) chart.keynames.unshift("time");
    		chart.keynames = chart.keynames.concat(units);
    	    }
    	    if(view == "events") {
    		// if "id" is in the keys, then this is a "tailing" query
    		var tail = false;
    		for each(var kk in cell.keys) if(kk == "id") tail = true;
    		chart.tail = tail;
    	    }
    	    job.callbacks.push(chart.tail ?
    			       tableTailQueryCallback(dash, chart, cell, scale, format) :
    			       tableQueryCallback(dash, chart, cell, scale, format));
    	}
    	else {
    	    throw("unknown divclass: " + cell.divclass());
    	}
    }


    function cleanupChart(chart) {
    	// remove trailing nulls - caused by "last" timebin not having data yet
    	if(chart
    	   && chart.trailingNull
    	   && chart.trend
    	   && chart.trend.times) {
    	    chart.trend.times.pop();
    	}
    }

    function unpackTopNKeys(chart) {
    	var trend = chart.trend.trends[chart.id];
	for(var tbin in trend) {
	    var topN = trend[tbin];
	    var newTopN = {};
	    for(var key in topN) {
		var valObj = topN[key];
		var newKey;
		if(key == "" || key == otherstr) newKey = key;
		else {
		    var keyList = valObj[0];
		    newKey = keyList.join(separator); // where DNS names are found in nameCache
		}
		newTopN[newKey] = valObj[1];
	    }
	    trend[tbin] = newTopN;
    	}
    }
    
    var result = {};
    if(debug) {
	result.jobs = JSON.stringify(jobs, null, 4);
    }
    // run each multiquery job, and collect results
    result.ok = true;
    result.refresh = this.getRefreshTimeout();
    result.charts = {};
    try {
	for each (var job in jobs) {
	    job.q.run(job.callbacks);
	    for (var ch in job.charts) {
	    	var chart = job.charts[ch];
	    	cleanupChart(chart);
	    	result.charts[ch] = chart;
	    }
	}
    } catch(err) {
	result.ok=false;
	result.error = err;
    }

    this.populateNameCache.call(dash, debug);
    for each(var cc in result.charts) {
	if(cc.unpackKeys) unpackTopNKeys(cc);
    }

    if(debug) {
	result.log = this.log;
    }
    println(JSON.stringify(result, null, 2));
}


Dash.prototype.htmlAction = function(update, debug) {
    // action = "dash" -- emit HTML describing the dashboard

    if(debug) {
	this.log.push("starting htmlAction");
	this.log.push("interval=" + interval);
    }

    println('<html>');
    println('<head>');
    println('<link rel="icon" type="image/vnd.microsoft.icon" href="/inmsf/images/favicon.ico" />');
    println('<link rel="stylesheet" href="/inmsf/inc/dash/jquery-ui-1.11.0/jquery-ui.min.css" type="text/css"/>');
    println('<link rel="stylesheet" href="/inmsf/inc/dash/jquery-ui-1.11.0/jquery-ui.structure.min.css" type="text/css"/>');
    println('<link rel="stylesheet" href="/inmsf/inc/dash/jquery-ui-1.11.0/jquery-ui.theme.min.css" type="text/css"/>');
    println('<link rel="stylesheet" href="/inmsf/inc/dash/css/dash.css" type="text/css"/>');
    println('<link rel="stylesheet" href="/inmsf/inc/dash/css/stripchart.css" type="text/css"/>');

    // Some style settings can be overridden in Dash
    println('<style type="text/css">');
    println('#dashboard .trend { width:' + this.getCellWidth() + '; height:' + this.getCellHeight() +'; }');
    println('#dashboard .topntable { width:' + this.getCellWidth() +'; }');
    println('.stripchart table { background: rgba(255,255,255,' + this.getLegendOpacity() + '); }');
    println('</style>');

    println('<script type="text/javascript" src="/inmsf/inc/dash/jquery-2.1.1.js"></script>');
    println('<script type="text/javascript" src="/inmsf/inc/dash/jquery-ui-1.11.0/jquery-ui.min.js"></script>');
    println('<script type="text/javascript" src="/inmsf/inc/dash/jquery.stripchart.js"></script>');
    println('<script type="text/javascript" src="/inmsf/inc/dash/chart.js"></script>');

    // poll for data periodically
    println('<script type="text/javascript" src="/inmsf/inc/dash/poller2.js"></script>');

    println('</head>');
    println('<body>');
    println('<div id="dashboard">');
    println('<table>');
    println('<thead>');
    println('<tr><th colspan="2">' + this.getTitle() + '</th></tr>');
    println('</thead>');
    println('<tbody>');

    for (var cc in this.cells) {
	var cell = this.cells[cc];
	var cellID = "cell" + cc;
	var cols = this.getColumns();
	if((cc % cols) == 0) {
	    // new row
	    if(cc > 0) 	println('</tr>');
	    println('<tr>');
	}
	println('<td>');
	println('<div>');
	println('<h3>' + cell.getTitle() + '</h3>');
	println('<div id="' + cellID + '" class="' + cell.divclass() + '" </div>');
	println('</div>');
	println('</td>');
    }
    println('</tr>');
    println('</tbody>');
    println('</table>');
    println('</div>');
    if(debug) {
	println('<div id="debug"><ul>');
	for(var ii in this.log) {
	    println("<li>");
	    println(this.log[ii]);
	    println("</li>");
	}
	println('</ul></div>');
    }
    println('</body>');
    println('</html>');
}

Dash.prototype.docAction = function(debug) {
    // action = "doc" -- emit HTML describing this dashboard API
    
    if(debug) {
	this.log.push("starting docAction");
    }
    
    println('<html>');
    println('<head>');
    println('<link rel="icon" type="image/vnd.microsoft.icon" href="/inmsf/images/favicon.ico" />');
    println('</head>');
    println('<body>');
    println('<div id="apidoc">');
    println('<table>');
    println('<thead>');
    println('<tr><th colspan="2">Dashboard 1.0 API Documentation</th></tr>');
    println('</thead>');
    println('<tbody>');
    println('</tr>');
    println('<td>');
    println('<p>');
    println('For documentation on how to use this API, please see <a href=http://www.inmon.com/tutorials7/restapi.php>this tutorial.</a>');
    println('</p>');
    println('</td>');
    println('</tr>');
    
    println('</tbody>');
    println('</table>');
    println('</div>');
    if(debug) {
	println('<div id="debug"><ul>');
	for(var ii in this.log) {
	    println("<li>");
	    println(this.log[ii]);
	    println("</li>");
	}
	println('</ul></div>');
    }
    println('</body>');
    println('</html>');
}

Dash.prototype.run = function(action, update, debug) {
    if(action == "spec") {
	println(JSON.stringify(dash, null, 4));
    }
    else if(action == "data") {
	this.dataAction(update, debug);
    }
    else if(action == "doc") {
	this.docAction(debug);
    }
    else {
	this.htmlAction(update, debug);
    }
}
