-----------------------------------------------------------------------
----- Copyright (C) InMon Corp. 2014 ALL RIGHTS RESERVED           ----
----- Module to send measurements into Traffic Sentinel            ----
-----------------------------------------------------------------------

local json = require("dkjson")
local measurement = {}

function measurement.accumulate(obj, field, value, aggmode)
   local val = tonumber(value) or 0
   if(not obj[field]) then obj[field] = 0.0 end
   if(aggmode == "sum") then
      obj[field] = obj[field] + val
   elseif (aggmode == "max") then
      if((obj[field] == nil) or (val > obj[field])) then obj[field] = val end
   elseif (aggmode == "min") then
      if((obj[field] == nil) or (val < obj[field])) then obj[field] = val end
   else
      throw("unknown aggmode")
   end
end

function measurement.countersample(ag, dsclass, dsindex, csamplename, csampleObj)
   local mm = {}
   mm.agent = ag
   mm.ds_class = tonumber(dsclass)
   mm.ds_index = tonumber(dsindex)
   local cs = {}
   mm.counter_sample = cs
   cs[csamplename]=csampleObj
   local mm_json = json.encode(mm, { indent=true })
   -- inx.log(0, "sending json countersample: " .. mm_json)
   inx.measurement(mm_json)
end

function measurement.if_counters(ag, ifindex, type, speed, direction, status, inoctets, inucasts, inmcasts, inbcasts, indiscards, inerrors, unknownprotos, outoctets, outucasts, outmcasts, outbcasts, outdiscards, outerrors, promisc)
   local gen = {}
   gen.if_index = tonumber(ifindex)
   gen.if_type = tonumber(type)
   gen.if_speed = tonumber(speed)
   gen.if_direction = tonumber(direction)
   gen.status = tonumber(status)
   gen.if_in_octets = tonumber(inoctets)
   gen.if_in_ucasts = tonumber(inucasts)
   gen.if_in_mcasts = tonumber(inmcasts)
   gen.if_in_bcasts = tonumber(inbcasts)
   gen.if_in_discards = tonumber(indiscards)
   gen.if_in_errors = tonumber(inerrors)
   gen.if_in_unknown_protos = tonumber(unknownprotos)
   gen.if_out_octets_ = tonumber(outoctets)
   gen.if_out_ucasts = tonumber(outucasts)
   gen.if_out_mcasts = tonumber(outmcasts)
   gen.if_out_bcasts = tonumber(outbcasts)
   gen.if_out_discards = tonumber(outdiscards)
   gen.if_out_errors = tonumber(outerrors)
   gen.if_promiscuous_mode = tonumber(promisc)
   measurement.countersample(ag, 0, tonumber(ifindex), "if_counters", gen)
end

function measurement.processor(ag, mtotal, mused, cpu60)
   local tot = tonumber(mtotal) or 0
   local usd = tonumber(mused) or 0
   local cpu = tonumber(cpu60) or 0
   local pr = {}
   pr.total_memory = tot
   pr.free_memory = (tot - usd)
   pr.one_min_cpu = cpu
   measurement.countersample(ag, 0, 0x3FFFFFFF, "processor", pr)
end

-- processorMax: variant that is easier to populate for multi-cpu systems
function measurement.processorMax(ag, cpu60, memutil)
   local cpu = tonumber(cpu60) or 0
   local mem = tonumber(memutil) or 0
   local pr = {}
   pr.cpu60_max = cpu
   pr.memutil_max = memutil
   measurement.countersample(ag, 0, 0x3FFFFFFF, "processorMax", pr)
end

function measurement.env(ag, power_w, temp_c, humidity_pc, fan_rpm, fancount, fanfailed, psucount, psufailed)
   local ev = {}
   ev.power = tonumber(power_w) or 0
   ev.temp = tonumber(temp_c) or 0
   ev.humidity = tonumber(humidity_pc) or 0
   ev.fanrpm = tonumber(fan_rpm) or 0
   ev.fans = tonumber(fancount) or 0
   ev.fans_failed = tonumber(fanfailed) or 0
   ev.psus = tonumber(psucount) or 0
   ev.psus_failed = tonumber(psufailed) or 0
   measurement.countersample(ag, 0, 0x3FFFFFFF, "env", ev)
end

measurement.rtmetric_types = {
   "string",
   "counter32",		    
   "counter64",		    
   "gauge32",		    
   "gauge64",		    
   "gaugefloat",		    
   "gaugedouble",
}

function measurement.rtmetric(ag, ds_class, ds_index, mname, mindex, mtype, mvalue)
   local rtm = {}
   rtm.mname = tostring(mname)
   rtm.mindex = tostring(mindex)
   rtm.mtype = tostring(mtype)
   if(mtype == "string") then
      rtm.mvalue = tostring(mvalue)
   else
      rtm.mvalue = tonumber(mvalue) or 0
   end
   -- inx.log(0, "sending rtmetric: " .. json.encode(rtm, { indent=true }) )
   measurement.countersample(ag, ds_class, ds_index, "rtmetric", rtm)
end

return measurement
