-----------------------------------------------------------------------
----- Copyright (C) InMon Corp. 2014 ALL RIGHTS RESERVED           ----
----- Module to poll CPU and memory stats from Brocade(TM) devices ----
-----------------------------------------------------------------------

local measurement = require("measurement")

local brocade_pattern = "^1%.3%.6%.1%.4%.1%.1991%."

local e_brocade = "1.3.6.1.4.1.1991"
local e_brocade_mem_total = e_brocade .. ".1.1.2.1.54"
local e_brocade_mem_free  = e_brocade .. ".1.1.2.1.55"
local e_brocade_cpu       = e_brocade .. ".1.1.2.11.1.1.4"
local tw_cpumem_brocade = inx.TWCreate({ e_brocade_mem_total, e_brocade_mem_free, e_brocade_cpu })

local agents = {}

local function getAgent(ip)
   if(not agents[ip]) then agents[ip] = { ip=ip } end
   return agents[ip]
end

local function walkCB(ip, status, rowid, mem_total, mem_free, cpu60)
   local agent = getAgent(ip)
   if status == nil then
      measurement.accumulate(agent, "mem_total", mem_total, "sum")
      measurement.accumulate(agent, "mem_free", mem_free, "sum")
      measurement.accumulate(agent, "cpu60", cpu60, "sum")
   elseif (status == "END") then
      if(agent.debug) then
	 inx.log(0,"Brocade walkCB normal termination, send processor sample for " .. ip)
      end
      measurement.processor(ip, agent.mem_total, agent.mem_used, agent.cpu60)
      agent.mem_total = 0
      agent.mem_free = 0
      agent.cpu60 = 0
   end
   return true -- keep going
end

local function sendWALK(args)
   if(args.agent.debug) then inx.log(0,"sendWALK: " .. args.agent.ip) end
   inx.timer(sendWALK, args, args.mS)
   if(args.agent.up) then
      inx.TWDoTableWalk(args.agent.twc, walkCB)
   end
end

local function doAgent(ip, objId, up, args)
   if(string.find(objId, brocade_pattern)) then
      local agent = getAgent(ip)
      agent.up = up
      if(ip == args.debugAgent) then
	 inx.log(0,"Brocade doAgent ip=" .. ip .. " objId=" .. tostring(objId) .. " up=" .. tostring(up))
	 agent.debug = true
      end
      if(up and not agent.twc) then
	 agent.twc = inx.TWContextCreate(tw_cpumem_brocade, ip)
	 local poll_mS = 1000 * args.interval
	 inx.timer(sendWALK, { agent=agent, mS=poll_mS }, math.random(poll_mS))
      end
   end
end

local pollCpuMem_Brocade = { agentCB = doAgent }
return pollCpuMem_Brocade
