-----------------------------------------------------------------------
----- Copyright (C) InMon Corp. 2014 ALL RIGHTS RESERVED         ------
----- Module to poll CPU and memory stats from Cisco(TM) devices ------
-----------------------------------------------------------------------

local measurement = require("measurement")

local cisco_pattern = "^1%.3%.6%.1%.4%.1%.9%."

local e_cisco = "1.3.6.1.4.1.9"
local e_cisco_mem_pool = e_cisco .. ".9.48.1.1.1"
local e_cisco_mem_used = e_cisco_mem_pool .. ".5"
local e_cisco_mem_free = e_cisco_mem_pool .. ".6"
local e_cisco_cpu = e_cisco .. ".9.109.1.1.1.1.7"
local tw_cpumem_cisco = inx.TWCreate({ e_cisco_mem_used, e_cisco_mem_free, e_cisco_cpu })

local agents = {}

local function getAgent(ip)
   if(not agents[ip]) then agents[ip] = { ip=ip } end
   return agents[ip]
end

local function walkCB(ip, status, rowid, mem_used, mem_free, cpu60)
   local agent = getAgent(ip)
   if status == nil then
      measurement.accumulate(agent, "mem_used", mem_used, "sum")
      measurement.accumulate(agent, "mem_free", mem_free, "sum")
      measurement.accumulate(agent, "cpu60", cpu60, "max")
   elseif (status == "END") then
      if(agent.debug) then
	 inx.log(0,"Cisco walkCB normal termination, send processor sample for " .. ip)
      end
      measurement.processor(ip, (agent.mem_used + agent.mem_free), agent.mem_used, agent.cpu60)
      agent.mem_used = 0
      agent.mem_free = 0
      agent.cpu60 = 0
   end
   return true -- keep going
end

local function sendWALK(args)
   if(args.agent.debug) then inx.log(0,"Cisco sendWALK: " .. args.agent.ip) end
   inx.timer(sendWALK, args, args.mS)
   if(args.agent.up) then
      inx.TWDoTableWalk(args.agent.twc, walkCB)
   end
end

local function doAgent(ip, objId, up, args)
   if(string.find(objId, cisco_pattern)) then
      local agent = getAgent(ip)
      agent.up = up
      if(ip == args.debugAgent) then
	 inx.log(0,"Cisco doAgent ip=" .. ip .. " objId=" .. tostring(objId) .. " up=" .. tostring(up))
	 agent.debug = true
      end
      if(up and not agent.twc) then
	 agent.twc = inx.TWContextCreate(tw_cpumem_cisco, ip)
	 local poll_mS = 1000 * args.interval
	 inx.timer(sendWALK, { agent=agent, mS=poll_mS }, math.random(poll_mS))
      end
   end
end

local pollCpuMem_Cisco = { agentCB = doAgent }
return pollCpuMem_Cisco
