-----------------------------------------------------------------------
----- Copyright (C) InMon Corp. 2014 ALL RIGHTS RESERVED       --------
----- Module to poll CPU and memory stats from Hewlett-Packard --------
----- ProCurve devices                                         --------
-----------------------------------------------------------------------

local measurement = require("measurement")

local hp_pattern = "^1%.3%.6%.1%.4%.1%.11%."

local e_hp = "1.3.6.1.4.1.11"
local hp_mem_total = e_hp .. ".2.14.11.5.1.1.2.1.1.1.5.1"
local hp_mem_used = e_hp .. ".2.14.11.5.1.1.2.1.1.1.6.1"
local hp_cpu_pc = e_hp .. ".2.14.11.5.1.9.6.1.0"
local tw_cpumem_hp = inx.TWCreate({ hp_mem_used, hp_mem_total, hp_cpu_pc })

local agents = {}

local function getAgent(ip)
   if(not agents[ip]) then agents[ip] = { ip=ip } end
   return agents[ip]
end

local function getCB(ip, err, mem_used, mem_total, cpu60)
   if(err) then
      inx.log(0, "HP walkCB got error: " .. err)
   else
      local agent = getAgent(ip)
      -- inx.log(0, "HP walkCB normal termination, send processor sample for " .. ip)
      measurement.processor(ip, mem_total, mem_used, cpu60)
   end
   return true -- keep going
end

local function sendGET(args)
   if(args.agent.debug) then inx.log(0, "HP sendGET: " .. args.agent.ip) end
   inx.timer(sendGET, args, args.mS)
   if(args.agent.up) then
      inx.TWDoGet2(args.agent.twc, getCB)
   end
end

local function doAgent(ip, objId, up, args)
   if(string.find(objId, hp_pattern)) then
      local agent = getAgent(ip)
      agent.up = up
      if(ip == args.debugAgent) then
	 inx.log(0, "HP doAgent ip=" .. ip .. " objId=" .. tostring(objId) .. " up=" .. tostring(up))
	 agent.debug = true
      end
      if(up and not agent.twc) then
	 agent.twc = inx.TWContextCreate(tw_cpumem_hp, ip)
	 local poll_mS = 1000 * args.interval
	 inx.timer(sendGET, { agent=agent, mS=poll_mS }, math.random(poll_mS))
      end
   end
end

local pollCpuMem_HP = { agentCB = doAgent }
return pollCpuMem_HP
