-----------------------------------------------------------------------
----- Copyright (C) InMon Corp. 2014 ALL RIGHTS RESERVED       --------
----- Module to poll CPU and memory stats from devices         --------
----- that support the HOST-RESOURCES-MIB                      --------
-----------------------------------------------------------------------

local measurement = require("measurement")

local arista_pattern = "^1%.3%.6%.1%.4%.1%.30065%."
local net_snmp_pattern = "^1%.3%.6%.1%.4%.1%.8072%."

local host_resources = "1.3.6.1.2.1.25"

local hr_storage_type = host_resources .. ".2.3.1.2"
local hr_storage_type_ram = host_resources .. ".2.1.2"
local hr_storage_size = host_resources .. ".2.3.1.5"
local hr_storage_used = host_resources .. ".2.3.1.6"
local hr_mem_total = host_resources .. ".2.2"
local hr_cpu_pc = host_resources .. ".3.3.1.2"
local tw_cpumem_hr = inx.TWCreate({ hr_mem_total, hr_cpu_pc, hr_storage_type, hr_storage_size, hr_storage_used })

local agents = {}

local function getAgent(ip)
   if(not agents[ip]) then agents[ip] = { ip=ip } end
   return agents[ip]
end

local function walkCB(ip, status, rowid, mem_total, cpu60, storage_type, storage_size, storage_used)
   local agent = getAgent(ip)
   if status == nil then
      measurement.accumulate(agent, "mem_total", mem_total, "sum")
      measurement.accumulate(agent, "cpu60", cpu60, "sum")
      if (storage_type and storage_type == hr_storage_type_ram) then
	 measurement.accumulate(agent, "mem_total2", storage_size, "sum")
	 measurement.accumulate(agent, "mem_used", storage_used, "sum")
      end
   elseif (status == "END") then
      measurement.processor(ip, agent.mem_total, agent.mem_used, agent.cpu60);
      agent.mem_total = 0;
      agent.mem_total2 = 0;
      agent.mem_used = 0;
      agent.cpu60 = 0;
   end
   return true -- keep going
end

local function sendWALK(args)
   if(args.agent.debug) then inx.log(0, "HR sendWALK: " .. args.agent.ip) end
   inx.timer(sendWALK, args, args.mS)
   if(args.agent.up) then
      inx.TWDoTableWalk(args.agent.twc, walkCB)
   end
end

local function doAgent(ip, objId, up, args)
   if(string.find(objId, arista_pattern)	
      or string.find(objId, net_snmp_pattern)) then
      local agent = getAgent(ip)
      agent.up = up
      if(ip == args.debugAgent) then
	 inx.log(0, "HR doAgent ip=" .. ip .. " objId=" .. tostring(objId) .. " up=" .. tostring(up))
	 agent.debug = true
      end
      if(up and not agent.twc) then
	 agent.twc = inx.TWContextCreate(tw_cpumem_hr, ip)
	 local poll_mS = 1000 * args.interval
	 inx.timer(sendWALK, { agent=agent, mS=poll_mS }, math.random(poll_mS))
      end
   end
end

local pollCpuMem_HR = { agentCB = doAgent }
return pollCpuMem_HR
