-----------------------------------------------------------------------
----- Copyright (C) InMon Corp. 2014 ALL RIGHTS RESERVED           ----
----- Module to poll temp/PSU/fan data from Brocade(TM) devices    ----
-----------------------------------------------------------------------

local measurement = require("measurement")

local brocade_pattern = "^1%.3%.6%.1%.4%.1%.1991%."

local e_brocade = "1.3.6.1.4.1.1991"
local snChasGen = e_brocade .. ".1.1.1.1"
local snChasPwr = e_brocade .. ".1.1.1.2"
local snChasFan = e_brocade .. ".1.1.1.3"
local snChasActualTemperature  = snChasGen .. ".18"
local snChasPwrSupplyOperStatus = snChasPwr .. ".1.1.3"
local snChasFanOperStatus = snChasFan .. ".1.1.3"
local tw_env_brocade = inx.TWCreate({ snChasActualTemperature, snChasPwrSupplyOperStatus, snChasFanOperStatus })

local agents = {}

local function resetAgent(agent)
   agent.temp = nil
   agent.fan_count = 0
   agent.fan_failed = 0
   agent.psu_count = 0
   agent.psu_failed = 0
end

local function getAgent(ip)
   if(not agents[ip]) then
      agents[ip] = { ip=ip }
      resetAgent(agents[ip])
   end
   return agents[ip]
end

local function walkCB(ip, status, rowid, temp, psuStatus, fanStatus)
   local agent = getAgent(ip)
   if status == nil then
      -- temp provided in half-degrees
      local degrees = (tonumber(temp) or 0) / 2
      -- should only be one here, but take the max because we'll get nulls when there are multiple psu/fan entries
      measurement.accumulate(agent, "temp", degrees, "max")
      local psu = tonumber(psuStatus) or 0
      if(psu == 1 or psu == 2 or psu == 3) then
        agent.psu_count = agent.psu_count + 1
        if(psu == 3) then agent.psu_failed = agent.psu_failed + 1 end
      end
      local fan = tonumber(fanStatus) or 0
      if(fan == 1 or fan == 2 or fan == 3) then
        agent.fan_count = agent.fan_count + 1
        if(fan == 3) then agent.fan_failed = agent.fan_failed + 1 end
      end
   elseif (status == "END") then
      if(agent.debug) then
	 inx.log(0,"Brocade ENV walkCB normal termination, send ENV sample for " .. ip)
      end
      measurement.env(ip, 0, agent.temp, 0, 0, agent.fan_count, agent.fan_failed, agent.psu_count, agent.psu_failed)
      resetAgent(agent)
   end
   return true -- keep going
end

local function sendWALK(args)
   if(args.agent.debug) then inx.log(0,"ENV sendWALK: " .. args.agent.ip) end
   inx.timer(sendWALK, args, args.mS)
   if(args.agent.up) then
      inx.TWDoTableWalk(args.agent.twc, walkCB)
   end
end

local function doAgent(ip, objId, up, args)
   if(string.find(objId, brocade_pattern)) then
      local agent = getAgent(ip)
      agent.up = up
      if(ip == args.debugAgent) then
	 inx.log(0,"Brocade ENV doAgent ip=" .. ip .. " objId=" .. tostring(objId) .. " up=" .. tostring(up))
	 agent.debug = true
      end
      if(up and not agent.twc) then
	 agent.twc = inx.TWContextCreate(tw_env_brocade, ip)
	 local poll_mS = 1000 * args.interval
	 inx.timer(sendWALK, { agent=agent, mS=poll_mS }, math.random(poll_mS))
      end
   end
end

local pollEnv_Brocade = { agentCB = doAgent }
return pollEnv_Brocade
