-----------------------------------------------------------------------
----- Copyright (C) InMon Corp. 2014 ALL RIGHTS RESERVED       --------
----- Module to poll CPU and memory stats from                 --------
----- Dell/Force10 E-Series devices                            --------
-----------------------------------------------------------------------

local measurement = require("measurement")

local f10e_pattern = "^1%.3%.6%.1%.4%.1%.6027%.1%.1%." -- E Series

local f10_chassis = "1.3.6.1.4.1.6027.3.1"

-- chRpmCpuUtil1Min[chRpmCpuIndex]
-- rpm = "route processing module"
local f10_cpu60_rpm = f10_chassis .. ".1.3.7.1.4"

-- chRpmMemUsageUtil[chRpmCpuIndex]
local f10_memutil_rpm = f10_chassis .. ".1.3.7.1.6"

-- chLineCardCpuUtil1Min[chSysCardNumber]
local f10_cpu60_lc = f10_chassis .. ".1.5.1.1.2"

-- chLineCardMemUsageUtil[chSysCardNumber]
local f10_memutil_lc = f10_chassis .. ".1.5.1.1.4"

-- chSysProcessorMemSize[chSysSwSlotIndex,chSysSwProcessorIndex]
local f10_mem_total = f10_chassis .. ".1.2.6.1.6"

-- to match memutil with memtotal requires index mappings from
-- the chSysCardTable.  Here slots 0,1 are rpms, and
-- slots 2..N are line-cards.  Knowing slot>card>processor
-- allows us to join the data from the other tables.  We get
-- temperature info from this table too.
local f10_cardtable = f10_chassis .. ".1.2.3"
local f10_slot = f10_cardtable .. ".1.1"
local f10_card = f10_cardtable .. ".1.3"
local f10_temp_max = f10_cardtable .. ".1.8"
local f10_temp_min = f10_cardtable .. ".1.9"
local f10_card_processors = f10_cardtable .. ".1.10"

-- but for now we just take the max cpuutil and
-- max memutil across any rpm processor or
-- line-card.

local tw_cpumem_f10 = inx.TWCreate({ f10_mem_total, f10_cpu60_rpm, f10_cpu60_lc, f10_memutil_rpm, f10_memutil_lc, f10_temp_max, f10_temp_min })

local agents = {}

local function getAgent(ip)
   if(not agents[ip]) then agents[ip] = { ip=ip } end
   return agents[ip]
end

local function walkCB(ip, status, rowid, mem_total, cpu60, cpu60_lc, memutil_rpm, memutil_lc, temp_max, temp_min)
   local agent = getAgent(ip)
   if status == nil then
      measurement.accumulate(agent, "mem_total", mem_total, "sum")
      measurement.accumulate(agent, "cpu60", cpu60, "max")
      measurement.accumulate(agent, "cpu60", cpu60_lc, "max")
      measurement.accumulate(agent, "memutil", memutil_rpm, "max")
      measurement.accumulate(agent, "memutil", memutil_lc, "max")
      measurement.accumulate(agent, "temp_max", temp_max, "max")
      measurement.accumulate(agent, "temp_min", temp_min, "min")
   elseif (status == "END") then
      measurement.processor(ip, agent.mem_total, 0, agent.cpu60)
      measurement.processorMax(ip, agent.cpu60, agent.memutil)
      measurement.env(ip, 0, agent.temp_max, 0, 0)
      measurement.rtmetric(ip, 3, 1, "entity.temp_min", 1, "gaugedouble", agent.temp_min)
      agent.mem_total = 0
      agent.cpu60 = 0
      agent.memutil = 0
      agent.temp_min = nil
      agent.temp_max = nil
   end
   return true -- keep going
end

local function sendWALK(args)
   if(args.agent.debug) then inx.log(0, "F10E sendWALK: " .. args.agent.ip) end
   inx.timer(sendWALK, args, args.mS)
   if(args.agent.up) then
      inx.TWDoTableWalk(args.agent.twc, walkCB)
   end
end

local function doAgent(ip, objId, up, args)
   if(string.find(objId, f10e_pattern)) then
      local agent = getAgent(ip)
      agent.up = up
      if(ip == args.debugAgent) then
	 inx.log(0, "F10E doAgent ip=" .. ip .. " objId=" .. tostring(objId) .. " up=" .. tostring(up))
	 agent.debug = true
      end
      if(up and not agent.twc) then
	 agent.twc = inx.TWContextCreate(tw_cpumem_f10, ip)
	 local poll_mS = 1000 * args.interval
	 inx.timer(sendWALK, { agent=agent, mS=poll_mS }, math.random(poll_mS))
      end
   end
end

local poll_Force10_E = { agentCB = doAgent }
return poll_Force10_E
