-----------------------------------------------------------------------
----- Copyright (C) InMon Corp. 2014 ALL RIGHTS RESERVED       --------
----- Module to poll CPU and memory stats from                 --------
----- Dell/Force10 S-Series devices                            --------
-----------------------------------------------------------------------

local measurement = require("measurement")

local f10s_pattern = "^1%.3%.6%.1%.4%.1%.6027%.1%.3%." -- S Series

local f10_chassis = "1.3.6.1.4.1.6027.3.10"

local f10_stackunit = f10_chassis .. ".1.2.2"
local f10_power = f10_chassis .. ".1.2.3"
local f10_fan = f10_chassis .. ".1.2.4"
local f10_processor = f10_chassis .. ".1.2.7"
local f10_stackunit_util = f10_chassis .. ".1.2.9"

-- chStackUnitCpuUtil1Min[chStackUnitNumber]
local f10_cpu60 = f10_stackunit_util .. ".1.3"

-- chStackUnitMemUsageUtil[chStackUnitNumber]
local f10_memutil = f10_stackunit_util .. ".1.5"

-- chSysProcessorMemSize[chStackUnitNumber]
local f10_mem_total = f10_processor .. ".1.4"

-- chStackUnitTemp[chStackUnitIndex]
local f10_temp = f10_stackunit .. ".1.14"
local f10_fan_count = f10_stackunit .. ".1.27"
local f10_psu_count = f10_stackunit .. ".1.28"

-- chSysPowerSupplyOperStatus[chStackUnitIndex,chSysPowerSupplyIndex]
local f10_psu_status = f10_power .. ".1.2"
-- chSysFanTrayOperStatus[chStackUnitIndex,chSysFanTrayIndex]
local f10_fan_status = f10_fan .. ".1.2"

-- memutil and mem_total are both indexed by chStackUnitNumber
-- even though they are in different tables,  so theoretically
-- we can extract the free_memory numbers that way and populate
-- the sFlow processor structure with it.  However for now we
-- just offer the max memutil.

local tw_cpumem_f10 = inx.TWCreate({ f10_mem_total, f10_cpu60, f10_memutil, f10_temp, f10_fan_count, f10_fan_status, f10_psu_count, f10_psu_status })

local agents = {}

local function resetAgent(agent)
   agent.mem_total = 0
   agent.cpu60 = 0
   agent.memutil = 0
   agent.temp_min = nil
   agent.temp_max = nil
   agent.fan_count = 0;
   agent.fan_failed = 0
   agent.psu_count = 0
   agent.psu_failed = 0
end

local function getAgent(ip)
   if(not agents[ip]) then agents[ip] = { ip=ip } end
   resetAgent(agents[ip])
   return agents[ip]
end

local function walkCB(ip, status, rowid, mem_total, cpu60, memutil, temp, fan_count, fan_status, psu_count, psu_status)
   local agent = getAgent(ip)
   if status == nil then
      measurement.accumulate(agent, "mem_total", mem_total, "sum")
      measurement.accumulate(agent, "cpu60", cpu60, "max")
      measurement.accumulate(agent, "memutil", memutil, "max")
      measurement.accumulate(agent, "temp_max", temp, "max")
      measurement.accumulate(agent, "temp_min", temp, "min")
      measurement.accumulate(agent, "fan_count", fan_count, "sum")
      if(fan_status == "2") then
	 measurement.accumulate(agent, "fan_failed", 1, "sum")
      end
      measurement.accumulate(agent, "psu_count", psu_count, "sum")
      if(psu_status == "2") then
	 measurement.accumulate(agent, "psu_failed", 1, "sum")
      end
   elseif (status == "END") then
      measurement.processor(ip, agent.mem_total, 0, agent.cpu60)
      measurement.processorMax(ip, agent.cpu60, agent.memutil)
      measurement.env(ip, 0, agent.temp_max, 0, 0, agent.fan_count, agent.fan_failed, agent.psu_count, agent.psu_failed)
      measurement.rtmetric(ip, 3, 1, "entity.temp_min", 1, "gaugedouble", agent.temp_min)
      resetAgent(agent)
   end
   return true -- keep going
end

local function sendWALK(args)
   if(args.agent.debug) then inx.log(0, "F10S sendWALK: " .. args.agent.ip) end
   inx.timer(sendWALK, args, args.mS)
   if(args.agent.up) then
      inx.TWDoTableWalk(args.agent.twc, walkCB)
   end
end

local function doAgent(ip, objId, up, args)
   if(string.find(objId, f10s_pattern)) then
      local agent = getAgent(ip)
      agent.up = up
      if(ip == args.debugAgent) then
	 inx.log(0, "F10S doAgent ip=" .. ip .. " objId=" .. tostring(objId) .. " up=" .. tostring(up))
	 agent.debug = true
      end
      if(up and not agent.twc) then
	 agent.twc = inx.TWContextCreate(tw_cpumem_f10, ip)
	 local poll_mS = 1000 * args.interval
	 inx.timer(sendWALK, { agent=agent, mS=poll_mS }, math.random(poll_mS))
      end
   end
end

local poll_Force10_S = { agentCB = doAgent }
return poll_Force10_S
