local util = {}

function util.inc(t,k,v)
   t[k]=t[k]+(v or 1)
end

function util.dec(t,k,v)
   t[k]=t[k]-(v or 1)
end

function util.prefBoolean(pref)
   if(pref == nil) then return false end
   local lower = string.lower(pref);
   return ("yes" == lower or "true" == lower or "1" == lower)
end

function util.splitPort(pid)
   return pid:match("([%d%.]+)>(%d+)")
end

-- based on http://lua-users.org/wiki/TableSerialization

local function indent(sb, depth)
   if(depth > 0) then
      table.insert(sb, string.rep (" ", depth))
   end
end

local function safe_tostring(obj)
   if type(obj) == "userdata" then
      return "{userdata}"
   else
      return tostring(obj)
   end
end

function util.table_print(tt, max_depth, depth, done, name)
   local indent_inc = 2
   max_depth = max_depth or 0
   done = done or {}
   depth = depth or indent_inc
   name = name or ""

   if type(name) == "number" then name = "" end
   if type(tt) == "table" then
      local sb = {}
      table.insert(sb, string.format("%s{\n", safe_tostring(name)))
      for key, value in pairs (tt) do
	 indent(sb, depth)
	 if type (value) == "table" then
	    if not done[value] then
	       -- remember we did this already to avoid infinite recursion
	       done [value] = true
	       if (max_depth > 0 and (depth > (max_depth * indent_inc))) then
		  table.insert(sb, string.format("%s = {maxdepth}\n", safe_tostring(key)))
	       else
		  table.insert(sb, util.table_print (value, max_depth, depth + indent_inc, done, key))
	       end
	    else
	       -- indicate the duplicate reference
	       table.insert(sb, string.format("%s = {done}\n", safe_tostring(key)))
	    end
	 elseif type(key) == "number" then
	    -- numeric index - just show the value
	    table.insert(sb, string.format("%s\n", tostring(value)))
	 elseif type(value) == "number" then
	    -- numeric value - no quotes
	    table.insert(sb, string.format("%s = %s\n", safe_tostring (key), safe_tostring(value)))
	 elseif type(value) == "boolean" then
	    -- boolean value - no quotes
	    table.insert(sb, string.format("%s = %s\n", safe_tostring (key), safe_tostring(value)))
	 else
	    table.insert(sb, string.format("%s = \"%s\"\n", safe_tostring (key), safe_tostring(value)))
	 end
      end
      indent(sb, depth - indent_inc)
      table.insert(sb, "}\n")
      return table.concat(sb)
   else
      return tostring(tt) .. "\n"
   end
end

return util
