// path-filter: nav.map:nav.map.aspath

// disply the tree formed by the top paths to or through
// the selected AS

var value = 'bytes';
var view = 'rttraffic';
var rootcolor = 'YELLOW';
var asncolor = 'LIGHT_BLUE';
var asnshape = "octogon";

var webapp = WebApp.current();
webapp.view = view;

function createmap(aspath, truncate, interval, where) {
  var n = Network.current();

  var graph = {};
  var nodes = {};
  var edges = {};
  graph.nodes = nodes;
  graph.edges = edges;
  graph.digraph = true;

  var select = 'first(bgpas),bgpdestinationaspath,' + value;
 
  var filter = where;
  if(aspath) {
      var filt = aspath;
      if(aspath.indexOf("-") == -1) {
	  // make it -AS- if it's just AS
	  filt = "-" + filt + "-";
      }
      var asnfilter = 'bgpdestinationaspath=' + filt;
     if(filter) filter = '(' + filter + ')&(' + asnfilter + ')';
     else filter = asnfilter;
  }

  var q = Query.topN(view,
                     select,
                     filter,
                     interval,
                     value,
                     truncate);

  function addAS(asn,rootFlag) {
     if(!nodes[asn]) {
        nodes[asn] = {
	    label: asn,
            asn: asn,
            color: rootFlag ? rootcolor : asncolor,
	    shape: asnshape,
	    evtId: 'asn'
        }
     }
  }

  function addEdge(id,n1,n2,weight) {
      if(!edges[id]) {
	  edges[id] = {
	      node1: n1,
	      node2: n2,
	      weight: weight,
	      arrow: 1,
              evtId: 'aslink'
	  }
      }
      else {
	  edges[id].weight += weight;
      }
  }
  
  q.run(function(row,table) {
            var myas = row[0];
	    var aspath = row[1];
	    var value  = row[2];
            var prev = null;
            if(myas) {
		addAS(myas,true); 
                prev = myas;
            }
	    if(aspath != null) {
		for each (var asn in aspath.split("-")) {
			addAS(asn,false);
			if(prev != null) {
			    addEdge(prev + "-" + asn, prev, asn, value);
			}
			prev = asn;
		}
	    }
      });
  
  // set the AS names
  var asns = [];
  var ndarray = [];
  for (var asn in nodes) {
      asns.push(asn);
  }


  var asnames = n.asMap(asns);
  for (var i in asns) {
      var asname = asnames[i][0]; // we get an array with the short and the long name
      if(asname != null && asname != "") {
          nodes[asns[i]].label = asname;
      }
  }

  
  // normalize the edge weights
  var maxval = 1;
  for each (var ed in edges) {
      maxval = Math.max(maxval, ed.weight);
  }
  for each (var ed in edges) {
      var normalizedWeight = Math.max(1, Math.round(Math.log(100 * ed.weight / maxval)));
      ed.weight = normalizedWeight;
  }


  return graph;
}

function header() {
  webapp.filterBar(
       [ 
         webapp.button(webapp.localize("map.reset"),webapp.createURL({aspath:"all"}))
       ], 
       [
         webapp.spinBox("interval","t.interval","last15minutes","interval"),
         webapp.spinBox("truncate","t.truncate","50",["20","50","100","200","500","1000"])
       ],
       "");
}

function data(evtName,evtId,evtValue) {
  var resp = {};

  // can signal an error message like this if necessary
  // top.action = "error";
  // top.message = "test error message";

  // evtId will be the node ASN or the edge ASN-ASN, but add
  // leading and trailing zeros so that the "connections" button
  // from the search page will lead to a trend chart.

  var aspath = webapp.getValue("aspath");
  switch(evtName) {
  case "nodeClick": 
      switch(evtId) {
      case "asn":
	  var names = Network.current().asMap([evtValue]);
	  resp.action = "dialog";
	  resp.title = names[0] && names[0][0] ? names[0][0] : evtValue;
	  resp.values = [];
	  if(names[0]) resp.values.push({label:webapp.localize("go.host.as.name"),value:names[0][0]});
	  resp.values.push({label:webapp.localize("map.filter"),value:evtValue,evtId:'filterASN'});
	  resp.values.push({label:webapp.localize("map.search"),value:evtValue,evtId:'searchASN'});
	  break;
      }
      break;
  case "edgeClick":
      switch(evtId) {
      case "aslink":
          resp.action = "dialog";
          resp.title = evtValue;
          resp.values = [];
          resp.values.push({label:webapp.localize("map.filter"),value:evtValue,evtId:'searchASN'});
          resp.values.push({label:webapp.localize("map.search"),value:evtValue,evtId:'searchASN'});
          break;
      }
      break;
  case "dialogClick": 
      switch(evtId) {
      case "searchASN": 
	  var filt = "-" + evtValue + "-";
	  resp.action = "loadpage";
	  resp.url = webapp.encodeRedirectURL("Go?action=aspath&aspath=" + filt);
	  break;
      case "filterASN":
	  var filt = "-" + evtValue + "-";
	  resp.action = "loadgraph";
          resp.refreshLayout = true;
	  webapp.setValue("aspath", filt);
	  break;
      }
      break;
  default:
      resp.action = "loadgraph";
  }

  if(resp.action == "loadgraph") {
    var interval = webapp.getValue("interval","last15minutes");
    var truncate = webapp.getValue("truncate","100");
    var where = webapp.getValue("where");
    var aspath = webapp.getValue("aspath");
    resp.graph = createmap(aspath,truncate,interval,where);
  }

  println(JSON.stringify(resp, null, 2)); 
}

var aspath = webapp.queryParameter("aspath");
if(aspath) webapp.setValue("aspath",aspath);
webapp.graph(
	     header, 
	     data, 
	     {
		     layout: "hierarchic",
                     height: webapp.getValue("com.inmon.servlets.map.height"),
		     panLabel: webapp.localize("map.pan"),
		     upLabel: webapp.localize("map.pan.up"),
		     downLabel: webapp.localize("map.pan.down"),
		     leftLabel: webapp.localize("map.pan.left"),
		     rightLabel: webapp.localize("map.pan.right"),
		     zoomLabel: webapp.localize("map.zoom"),
		     inLabel: webapp.localize("map.zoom.in"),
		     outLabel: webapp.localize("map.zoom.out"),
		     layoutLabel: webapp.localize("map.layout"),
		     organicLabel: webapp.localize("map.layout.organic"),
		     orthogonalLabel: webapp.localize("map.layout.orthogonal"),
		     treeLabel: webapp.localize("map.layout.tree"),
		     circularLabel: webapp.localize("map.layout.circular"),
		     hierarchicLabel: webapp.localize("map.layout.hierarchic"),
		     resetZoomLabel: webapp.localize("map.zoom.reset")
             });

