// path-filter: nav.map:nav.map.l4

// create a dependency tree
// who depends on this host (clients)
// who provides services to this host (servers)

var value = 'frames';
var view = 'rttraffic';
var hcolor = "LIGHT_CORAL";
var hshape = "rectangle";
var pcolor = "LIGHT_YELLOW";
var pshape = "ellipse";

var webapp = WebApp.current();
webapp.view = view;

var n = Network.current();
function pname(port) { var pname = n.protocolNameMap([port])[0]; return pname ? pname : port };

function createmap(host,truncate,interval,protocol,where) {

  var graph = {};
  var nodes = {};
  var edges = {};
  graph.nodes = nodes;
  graph.edges = edges;
  graph.digraph = true;

  var zonelabel = webapp.localize("config.zone");
  var grouplabel = webapp.localize("config.group");
  var hostlabel = webapp.localize("t.host");
  var protocollabel = webapp.localize("t.protocol");
  var externallabel = webapp.localize("t.EXTERNAL");

  var select = 'clientzone,clientgroup,first(clientname),clientaddress,serverzone,servergroup,first(servername),serveraddress,serverport,' + value;
 
  var filter = null;
  if(host) {
     var atype = n.addressType(host);

     var srccol = 'sourceaddress';
     var dstcol = 'destinationaddress';
     switch(atype) {
       case 'uuid': srccol = 'uuidsource'; dstcol = 'uuiddestination'; break;
       case 'ipv4': srccol = 'ipsource';  dstcol = 'ipdestination';  break;
       case 'ipv6': srccol = 'ip6source'; dstcol = 'ip6destination'; break;
       case 'mac' : srccol = 'macsource'; dstcol = 'macdestination'; break;
     }
     var host_filter = srccol + '=' + host + "|" + dstcol + '=' + host;
     filter = '(' + host_filter + ')';
  }
  if(protocol) {
    var parts = protocol.split(":");
    var ptype = parts[0];
    var pnum = parts[1];
    var protocol_filter;
    switch(ptype) {
      case 'ETHERNET': protocol_filter = "ethernetprotocol=" + pnum; break;
      case 'IEEE802' : protocol_filter = "ieee802sap=" + pnum; break;
      case 'IP'      : protocol_filter = "ipprotocol=" + pnum; break;
      default: protocol_filter = "serverport=" + protocol;
    }
    if(filter) filter += "&" + protocol_filter;
    else filter = protocol_filter;
  }
  if(where) {
    if(filter) filter += "&(" + where + ")";
    else filter = where;
  }

  var q = Query.topN(view,
                     select,
                     filter,
                     interval,
                     value,
                     truncate);

  function cLabel(label) { return label == 'EXTERNAL' ? externallabel : label };
  function addHost(name,address) {
     if(!nodes[address]) {
        nodes[address] = {
            label: name, 
            color: hcolor,
	    shape: hshape,
            evtId: 'host',
	    evtValue: address + "," + name
        }
     }
  }
  function addProtocol(address,protocol) {
     var id = address + "," + protocol;
     if(!nodes[id]) {
        nodes[id] =  {
            label: pname(protocol), 
            shape: pshape, 
            color: pcolor,
            evtId: 'protocol'
        }
     }
     return id;
  }
  function addEdge(id,n1,n2,weight) {
     if(!edges[id]) {
        edges[id] = {
           node1: n1,
           node2: n2,
           weight: weight,
           arrow: 1,
           evtId: 'edge'
        }
     }
  }

  var maxval = 1;
  q.run(function(row,table) {
	  var clientzone    = cLabel(row[0]);
	  var clientgroup   = cLabel(row[1]);
          var clientname    = row[2];
          var clientaddress = row[3];
          var serverzone    = cLabel(row[4]);
          var servergroup   = cLabel(row[5]);
          var servername    = row[6];
          var serveraddress = row[7];
          var serverport    = row[8];
          var value         = row[9];

          if(!(clientaddress && serveraddress && serverport)) return;

	  // DNS names may be null - especially for last5minutes
	  if(!clientname) clientname = clientaddress;
	  if(!servername) servername = serveraddress;

          maxval = Math.max(maxval,value);

          var weight = Math.max(1,Math.round(Math.log(100 * value / maxval)));
    
          addHost(clientname,clientaddress);
          addHost(servername,serveraddress);

          var protonode = addProtocol(serveraddress,serverport);
 
          addEdge(protonode,serveraddress,protonode,weight);
          addEdge(clientaddress + "," + protonode,protonode,clientaddress,weight);
       });

 
  return graph;
}

function header() {
  webapp.filterBar(
       [ 
         webapp.button(webapp.localize("map.reset"),webapp.createURL({host:"all"}))
       ], 
       [
         webapp.spinBox("interval","t.interval","last15minutes","interval"),
         webapp.spinBox("protocol","t.protocol",null,"protocol"),
         webapp.spinBox("truncate","t.truncate","50",["20","50","100","200","500","1000"])
       ],
       "");
}

function data(evtName,evtId,evtValue) {
  var resp = {};

  var host = webapp.getValue("host");
  switch(evtName) {
    case "nodeClick":
       switch(evtId) {
         case "host":
           resp.action = "dialog";
	   var parts = evtValue.split(",");
	   var addr = parts[0];
	   var name = parts[1];
           resp.title = name;
           resp.values = [];
	   resp.values.push({label:webapp.localize("map.filter"), value:addr, evtId:'hostFilter', evtValue: addr});
	   resp.values.push({label:webapp.localize("map.search"), value:addr, evtId:'hostSearch', evtValue: addr});
           break;
         case "protocol":
           var parts = evtValue.split(",");
           resp.action = "dialog";
	   var protocol = parts[1];
           resp.title = pname(protocol);
           resp.values = [];
	   resp.values.push({label:webapp.localize("map.search"),value:protocol, evtId:'protocolSearch', evtValue: protocol});
           break;
       } 
       break;
    case "edgeClick":
       break;
  case "dialogClick":
      switch(evtId) {
      case "hostFilter":
	  resp.action = 'loadgraph';
	  resp.refreshLayout = true;
	  webapp.setValue("host",evtValue);
	  break;
      case "hostSearch":
	  resp.action = "loadpage";
	  resp.url = webapp.encodeRedirectURL('Go?action=host&host=' + evtValue);
	  break;
      case 'protocolSearch':
	  resp.action = 'loadpage';
	  resp.url = webapp.encodeRedirectURL('Go?action=protocol&protocol=' + evtValue);
	  break;
      }
      break;
  default:
       resp.action = "loadgraph";
  }

  if(resp.action == "loadgraph") {
    var interval = webapp.getValue("interval","last15minutes");
    var truncate = webapp.getValue("truncate","100");
    var protocol = webapp.getValue("protocol");
    var where = webapp.getValue("where");
    resp.graph = createmap(webapp.getValue("host"),truncate,interval,protocol,where);
  }

  println(JSON.stringify(resp, null, 2)); 
}

var host = webapp.queryParameter("host");
if(host) webapp.setValue("host",host);
webapp.graph(
	     header, 
	     data, {
		     layout: "organic",
                     height: webapp.getValue("com.inmon.servlets.map.height"),
		     panLabel: webapp.localize("map.pan"),
		     upLabel: webapp.localize("map.pan.up"),
		     downLabel: webapp.localize("map.pan.down"),
		     leftLabel: webapp.localize("map.pan.left"),
		     rightLabel: webapp.localize("map.pan.right"),
		     zoomLabel: webapp.localize("map.zoom"),
		     inLabel: webapp.localize("map.zoom.in"),
		     outLabel: webapp.localize("map.zoom.out"),
		     layoutLabel: webapp.localize("map.layout"),
		     organicLabel: webapp.localize("map.layout.organic"),
		     orthogonalLabel: webapp.localize("map.layout.orthogonal"),
		     treeLabel: webapp.localize("map.layout.tree"),
		     circularLabel: webapp.localize("map.layout.circular"),
		     hierarchicLabel: webapp.localize("map.layout.hierarchic"),
		     resetZoomLabel: webapp.localize("map.zoom.reset")
            });

