// path-filter: nav.services

var app = WebApp.current();

var jsonSpec = app.getValue("com.inmon.servlets.services.trendchart");
var spec = JSON.parse(jsonSpec);

String.prototype.startsWith = function(str) {
  return (this.match("^"+str)==str);
}

function simplifyFilter(filter) {
  var query = new Query();
  query.view = spec.view;
  query.where = filter;
  return query.simplifiedFilter();
}

function columnTypes() {
  var query = new Query();
  query.view = spec.view;
  query.select = spec.key;
  return query.columnTypes();
}

function data() {
  if(!spec) return;

  var network = Network.current();

  function protocolName(val) {
    var label = null;
    var map = network.protocolNameMap([val])[0];
    if(map && map != val) {
      var parts = map.split(':');
      if(parts.length == 2) label = val + " (" + parts[1] + ")";
    }
    return label;
  }

  var refresh = "true" == app.queryParameter("refresh","false");
  var scale = spec.scale ? spec.scale : 1.0;

  var query = Query.topN(
   spec.view,
   "time," + spec.key + "," + spec.val,
   spec.filter, 
   (refresh && spec.refreshInterval) ? spec.refreshInterval : spec.interval, 
   spec.val, 
   spec.resultTruncate);

  if(spec.delim) query.$delim = spec.delim;

  query.group = spec.intervalMinutes ? spec.intervalMinutes : "minute";

  var table = query.run();
  if(!table || table.nrows == 0) return;

  var data = [];
  var result = {data:data};

  var prevTime;
  var timeStr;
  var row;
  var topn;
  for(var r = 0; r < table.nrows; r++) {
    var time = table.cell(r,0).getTime();
    if(time != prevTime) {
      timeStr = formatdate(table.cell(r,0),"HH:mm");
      topn = [];
      row = [time,timeStr,topn];
      data.push(row);
      prevTime = time;
    }

    // keys
    var entry = [];
    for(var i = 1; i < table.ncols - 1; i++) {
      var ctype = table.ctypes[i];
      var val = table.cell(r,i);
      if(typeof val != 'undefined' && val != null) {
         var name = null;
         if('port' == ctype) name = protocolName(val);

	 var key = [val];
         if(name && name != val) key.push(name);
         entry.push(key);
      } else entry.push(val);
    };
    // value
    entry.push(table.cell(r,table.ncols - 1) * scale);
    topn.push(entry);
  }
  println(JSON.stringify(result));
}

function cFilt(cname,cval,notFlag) {
  var col_name = cname;
  var col_val = cval;
  var col_op = "=";
  if(cname.startsWith("pref_")) {
    var col_name = cname.substring(5).split("[")[0];
    col_val = "^" + cval.replace(/[-[\]{}()*+?.,\\^$|#\s]/g, "\\\\$&");
    col_op = notFlag ? "!~" : "~";
  } else col_op = notFlag ? "!=" : "=";
	
  return col_name + " " + col_op + " '" + col_val + "'";
}

function action(evtName,evtId,evtValue) {
  switch(evtName) {
    case "nodeClick": 
       var response = {};
       if(evtId > 0) {
         var cname = spec.keynames.split(",")[evtId - 1];
         var types = columnTypes();
         var link = types ? app.linkForTypeAndValue(types[evtId - 1],evtValue) : null;
         response.action = "dialog";
         response.selectFlag = true;
         response.excludeFlag = true;
         response.searchFlag = link ? true : false;
         response.message = '<table class="overview"><tbody>';
         response.message += '<tr class="even"><th>' + cname + '</th><td>' + evtValue + '</td></tr>';
         // extra information here
         response.message += '</tbody></table>';
       } else {
         // path dialog
         var names = spec.keynames.split(",");
         var values = evtValue.split(",");
         response.action = "dialog";
         response.selectFlag = true;
         response.excludeFlag = true;
         response.message = '<table class="overview"><tbody>';
         for(var i = 0; i < values.length; i++) {
            response.message += '<tr class="' + (i%2 == 0 ? 'even' : 'odd') + '"><th>' + names[i] + '</th><td>' + values[i] + '</td></tr>';
         }
         response.message += '</tbody></table>';
       }
       println(JSON.stringify(response,null,2));
       break;
    case "select":
       var response = {};
       response.action = "loadpage";
       var appname = app.getValue("com.inmon.servlets.services.service.u","");
       var filter = app.getValue("com.inmon.servlets.services.filter." + appname, "");
       if(evtId > 0) {
         var cname = spec.key.split(",")[evtId - 1];
         if(filter) filter = "(" + filter + ") & " + cFilt(cname,evtValue,false);
         else filter = cFilt(cname,evtValue,false);
       } else {
         // select all keys
         var names = spec.key.split(",");
         var values = evtValue.split(",");
         var clause = null;
         for(var i = 0; i < values.length; i++) {
           if(clause) clause += " & " + cFilt(names[i],values[i],false);
           else clause = cFilt(names[i],values[i],false);
         }
         var filter = app.getValue("com.inmon.servlets.services.filter." + appname, "");
         if(filter) filter = "(" + filter + ") & " + clause;
         else filter = clause;
       }
       app.setValue("com.inmon.servlets.services.filter." + appname, simplifyFilter(filter));
       response.url = app.encodeRedirectURL(app.createURL({}));
       println(JSON.stringify(response,null,2));
       break;
    case "exclude":
       var response = {};
       var appname = app.getValue("com.inmon.servlets.services.service.u","");
       var filter = app.getValue("com.inmon.servlets.services.filter." + appname, "");
       response.action = "loadpage";
       if(evtId > 0) {
         var cname = spec.key.split(",")[evtId - 1];
         if(filter) filter = "(" + filter + ") & " + cFilt(cname,evtValue,true);
         else filter = cFilt(cname,evtValue,true);
       } else {
         // exclude all keys
         var names = spec.key.split(",");
         var values = evtValue.split(",");
         var clause = null;
         for(var i = 0; i < values.length; i++) {
           if(clause) clause += " & " + cFilt(names[i],values[i],false);
           else clause = cFilt(names[i],values[i],false);
         }
         var filter = app.getValue("com.inmon.servlets.services.filter." + appname, "");
         if(filter) filter = "(" + filter + ") & !(" + clause + ")";
         else filter = "!(" + clause + ")";
       }
       app.setValue("com.inmon.servlets.services.filter." + appname, simplifyFilter(filter));
       response.url = app.encodeRedirectURL(app.createURL({}));
       println(JSON.stringify(response,null,2));
       break;
    case "search":
       var response = {};
       response.action = "loadpage";
       var types = columnTypes();
       var link = types ? app.linkForTypeAndValue(types[evtId - 1],evtValue) : null;
       response.url = app.encodeRedirectURL(link);
       println(JSON.stringify(response,null,2));
       break;
    default: data();
  }
}

app.trend(null,action,{
  "cnames"        : spec.keynames + "," + spec.valname,
  "colors"        : spec.colors,
  "valueAxisLabel": spec.valueAxisLabel,
  "height"        : spec.chartHeight,
  "select"        : app.localize("appletmenu.filter"),
  "exclude"       : app.localize("appletmenu.exclude"),
  "search"        : app.localize("appletmenu.search"),
  "intervals"     : spec.intervals,
  "refresh"       : spec.refreshInterval ? true : false,
  "truncate"      : spec.resultTruncate,
  "dataTimeout"   : preference("ajax.timeout") ? preference("ajax.timeout") : 600000
});

