// path-filter: nav.services

var webapp = WebApp.current();
var jsonSpec = webapp.getValue("com.inmon.servlets.services.statsSpec");
var spec = JSON.parse(jsonSpec);

var service = spec.service;
var trends = spec.trends;
var show = spec.show;

var n = Network.current();
var path = webapp.getValue("com.inmon.servlets.services.path");
n.path = path;

var showAllFlag = !show || "all" == show;

var height = 200;
var width = 500;

var ncols = 0;

function bgColorForStatus(sval) {
  color = "white";
  switch(sval) {
    case 'unknown': color = "white"; break;
    case 'good': color = "200,255,200"; break;
    case 'warn': color = "255,255,200"; break;
    case 'critical': color = "255,200,200"; break;
    default: color = "white";
  }
  return color;
}

function localize(str) {
   var res = webapp.localize("t." + str);
   if(res.indexOf("t.") === 0) {
     res = webapp.localize("c." + str);
     if(res.indexOf("c.") === 0) res = str;
   }
   return res;
}

function columnLabel(cname,localizeFlag) {
  // strip aggregations function to get underlying column name
  var match = /\(([^\s]+)\)/.exec(cname);
  var label =  match ? match[1] : cname;

  // get part of name after dot
  var idx = label.lastIndexOf('.');
  if(idx >= 0) label = label.substring(idx+1);
  return localizeFlag ? localize(label) : label;
}

function createChart(t,trend,status,title,h,w) {
  // fix column names
  for(var c = 0; c < t.cnames.length; c++) t.cnames[c] = columnLabel(t.cnames[c],true);

  // scale data, if needed
  if(trend.scale > 0) {
     for(var c = 1; c < t.ncols; c++) t.scaleColumn(c,trend.scale);
  }

  var chart = Chart.multiSeries(
                      "trend",
                      title,
                      t,
                      null,
                      0,
                      webapp.localize("t." + trend.ylabel),
                      Chart.range(1,t.ncols - 1));
  chart.height = h;
  chart.width = w;
  chart.step = true;
  chart.stack = trend.stack;
  chart.backgroundColor = bgColorForStatus(status);
  chart.timeLabels = t.column(0).map(function(timestamp) { return formatdate(timestamp,"HH:mm"); });
  if(trend.colors) chart.colors = trend.colors;
  return chart;
}

function data() {
  var charts = [];

  if(!trends) return; 

  var filter = spec.filter;
  if(filter) filter = "(" + filter + ") & path='" + path + "'";
  else filter = "path='" + path + "'";
  if(n.isInterface()) filter += " & v_dsindex=" + n.dsIndex();

  if(showAllFlag) {
    // build all the charts at this level in the path

    var select = [];

    var q = Query.trend(spec.view, select, filter, spec.interval, spec.intervalMinutes);
    q.multiquery = true;

    for(var i = 0; i < trends.length; i++) {
      select.push(trends[i].select);
    }

    var tables = q.run();

    // override - these charts are layed out for 2 columns
    ncols = 2;
    for(var i = 0; i < trends.length; i++) {
      var t = tables[i];
      var status = "unknown";

      if(t) {
        var chart = createChart(t,trends[i],status,webapp.localize("services.stats.show." + trends[i].id),height,width);
        chart.nodeClick = { id: trends[i].id };
        charts.push(chart);
      }
    }
  }
  else {
    // build the same chart for every child in this path

    var trend = null;
    for(var i = 0; i < trends.length; i++) {
      if(trends[i].id == show) {
         trend = trends[i];
         break;
       }
    }
    if(trend) {
      var ppath = path.split(">");
      
      var childkey;
      switch(ppath.length) {
        case 2: childkey = "zone"; break;
        case 3: childkey = "group"; break;
        case 4: childkey = "agent"; break;
        case 5: childkey = "v_dsindex"; break;
        case 6: childkey = "v_dsindex"; break;
        default: childkey = "zone";
      }

      // make a top N query to find the set of paths
      var children = [];
      var filters = [];
      
      // need to put in the select keys, sort key may contain function
      var select = trend.select.split(',');
      // remove functions
      for(var i = 0; i < select.length; i++) select[i] = columnLabel(select[i]);
      // replace time with childkey
      select[0] = childkey;
      select.push(trend.sort);

      var q = Query.topN(spec.view,select, filter,spec.interval,trend.sort,spec.truncate);
      q.run(function(row) {
         if(row[0]) {
            children.push(row[0]);
            filters.push(filter + "&" + childkey + '="' + row[0] + '"');
         }
      });

      // build a multiquery for each of the trends
      var tables;
      if(filters.length > 0) {
        q = Query.trend(spec.view,trend.select,filters,spec.interval,spec.intervalMinutes);
        q.multiquery = true;
        tables = q.run(); 
      }

      if(tables) {

         for(var i = 0; i < tables.length; i++) {
           var child = children[i];
           var childpath = childkey == "v_dsindex" ? path + ">3:" + child : path + ">" + ("EXTERNAL" != child ? child : "");
           n.path = childpath;
           var ct = tables[i];
           var title = n.displayName();

           var status = "unknown";
           var chart = ppath.length < 6 ? createChart(ct,trend,status,title,height,width) : createChart(ct,trend,status,title,null,null);
           if(ppath.length < 6) chart.nodeClick = { id: trend.id, value: childpath };
           charts.push(chart);
        }
      }
    }
  }
  println(JSON.stringify({data:charts}));
}

function action(evtName,evtId,evtValue) {
  switch(evtName) {
    case "nodeClick":
      var response = {action:"loadpage"};
      var urlArgs = {};
      if(evtValue) urlArgs.path=evtValue;
      else urlArgs.show=evtId;
      response.url = webapp.encodeRedirectURL(webapp.createURL(urlArgs));
      println(JSON.stringify(response,null,2));
      break;
    default: data();
  }
  
}

webapp.charts(null,action, {
    refresh     : spec.refresh,
    columns     : showAllFlag ? 2 : 0,
    rebuild     : !showAllFlag,
    large       : n.isInterface() && !showAllFlag,
    dataTimeout : preference("ajax.timeout") ? preference("ajax.timeout") : 600000
});
