// path-filter: nav.monitor

var webapp = WebApp.current();
var chartSpec = webapp.getValue("com.inmon.servlets.monitor.counters.json");
var spec = chartSpec ? JSON.parse(chartSpec) : null;
var trends = spec ? spec.trends : null;

var n = Network.current();

var path = webapp.getValue("com.inmon.servlets.monitor.path");
n.path = path;

var includeStatus = false;

function getTrend(show) {
   for(var i = 0; i < trends.length; i++) {
     if(trends[i].id == spec.show) return trends[i];
   }
   return null;
}  

function localize(str) {
   var res = webapp.localize("t." + str);
   if(res.indexOf("t.") === 0) {
      res = webapp.localize("c." + str);
      if(res.indexOf("c.") === 0) res = str;
   }
   return res;
}

function columnLabel(cname,localizeFlag) {
  // strip aggregations function to get underlying column name
  var match = /\(([^\s]+)\)/.exec(cname);
  var label =  match ? match[1] : cname;

  // get part of name after dot
  var idx = label.lastIndexOf('.');
  if(idx >= 0) label = label.substring(idx+1);
  return localizeFlag ? localize(label) : label;
}

function createChart(t,trend,status,title) {
   // fix column names
   for(var c = 0; c < t.cnames.length; c++) t.cnames[c] = columnLabel(t.cnames[c],true);

   // scale data, if needed
   if(trend.scale > 0) {
      for(var c = 1; c < t.ncols; c++) t.scaleColumn(c,trend.scale);
   }

   var chart = Chart.multiSeries(
                      "trend",
                      title,
                      t,
                      null,
                      0,
                      webapp.localize("t." + trend.ylabel),
                      Chart.range(1,t.ncols - 1));
   chart.step = true;
   chart.mirror = true;
   if(trend.colors) chart.colors = trend.colors;
   chart.timeLabels = t.column(0).map(function(timestamp) { return formatdate(timestamp,"HH:mm"); });
   return chart;
}

function data() {
   if(!spec) return;

   var charts = [];
   var n = Network.current();

   var filter = "dsindex!=0&path='" + path + "'";

   if(n.isInterface()) {
      var trend = getTrend(spec.show);
      if(!trend) return;
      
      var q = Query.trend("ifcounters",trend.select,filter,spec.interval,spec.intervalMinutes);
      var t = q.run();
      if(t) {
           var status = n.status(trend.id == "bytes" ? "utilization" : trend.id);
           var title = n.displayName();
	   if(title == null) {
	      title = n.path;
	   }
	   else {
              n.path = n.agent();
              title = n.displayName() + ">" + title;
	   }
           var chart = createChart(t,trend,status, title, null, null);
           //chart.nodeClick = {id: trend.id};
           charts.push(chart);
      }
    
   } else {
     if(spec.ifType) filter += " & iftype=" + spec.ifType;
     if(spec.ifSpeed) filter += " & ifspeed=" + spec.ifSpeed;


    // build the same chart for every child in this path

    var trend = getTrend(spec.show);
    if(!trend) return;

    // make a top N query to find the set of interfaces
      
    // need to put in the select keys, sort key may contain function
    var select = trend.sort.split(',');
    var sort = select[select.length - 1];

    var interfaces = [];

    var q = Query.topN("ifcounters",select, filter,spec.interval,sort,spec.resultTruncate);
    q.run(function(row) {
       if(row[0]) interfaces.push(row[0]);
    });

    var table;
    if(interfaces.length > 0) {
      q = Query.trend("ifcounters",trend.select + ",interface","interface=" + interfaces,spec.interval,spec.intervalMinutes);
      table = q.run();
    }

    if(!table) return;

    var tables = {};
    var cols = trend.select.split(",");
    for(var i = 1; i < cols.length; i++) {
      var pvt = table.pivotTime(0,cols.length,i,true);
      for(var j = 1; j < pvt.ncols; j++) {
        var t = tables[pvt.cnames[j]];
        if(!t) {
          t = Table.create([],[]);
          t.start = table.start;
          t.end = table.end;
          t.addColumn(table.cnames[0],table.ctypes[0],pvt.column(0));
          tables[pvt.cnames[j]] = t;
        }
        t.addColumn(table.cnames[i],table.ctypes[i],pvt.column(j));
      }
    }

    var names;
         
    for(var i = 0; i < interfaces.length; i++) {
      var ifx = interfaces[i];
      n.path = ifx;
      var ct = tables[ifx];
      var status = n.status(trend.id == "bytes" ? "utilization" : trend.id);
      var title = n.displayName();
      if(title == null) title = ifx; // fallback on ifx in case path lookup fails
      else {
        n.path = n.agent();
        title = n.displayName() + ">" + title;
      }
      var chart = createChart(ct,trend,status,title);
      chart.nodeClick = {id: trend.id, value: ifx };
      charts.push(chart);
    }
  }
  println(JSON.stringify({data:charts}));
}

function action(evtName,evtId,evtValue) {
  switch(evtName) {
    case "nodeClick":
      var response = {action:"loadpage"};
      var urlArgs = {};
      if(evtValue) urlArgs.path=evtValue;
      else urlArgs.show=evtId;
      response.url = webapp.encodeRedirectURL(webapp.createURL(urlArgs));
      println(JSON.stringify(response,null,2));
      break;
    default: data();
  }
}

var isInterface = n.isInterface();

webapp.charts(null,action, {
    refresh: spec.refresh,
    columns: 0,
    large: isInterface,
    rebuild: !isInterface,
   "dataTimeout": preference("ajax.timeout") ? preference("ajax.timeout") : 600000
});
