// path-filter: nav.monitor

var app = WebApp.current();
var chartSpec = app.getValue("com.inmon.servlets.monitor.jsonSpec",null);
var spec = chartSpec ? JSON.parse(chartSpec) : null;

var nvals = spec.bidirectional ? 3 : 1;
var nkeys = spec.resultField.split(",").length - nvals;

function simplifyFilter(filter) {
  var query = new Query();
  query.view = spec.view;
  query.where = filter;
  return query.simplifiedFilter();
}

function columnTypes() {
  var query = new Query();
  query.view = spec.view;
  query.select = spec.resultField;
  return query.columnTypes();
}

String.prototype.endsWith = function(suffix) {
    return this.indexOf(suffix, this.length - suffix.length) !== -1;
}
 
function data() {
  if(!spec) return; 

  var network = Network.current();

  function protocolName(val) {
    return network.protocolName(val,true);
  }

  function cipherName(val) {
    return network.cipherName(val);
  }

  function scsiOpName(val) {
    return network.scsiOpName(val);
  }

  function interfaceName(val) {
      network.path = val;
      var ifstr = network.displayName();
      if(ifstr == null) return val;
      network.path = network.parent();
      return network.displayName() + ">" + ifstr;
  }

  var vendorMap;

  var addressMap;
  var lookupCols = [];
  var nameCols = [];
  var nameToIdx = {};
  function nameColumns(select) {
    var cols = select.split(",");
    // time is always 1st column
    for(var i = 1; i < cols.length; i++) {
      var cname = cols[i];
      var ncol = null;
      switch(cname) {
        case 'clientaddress'     : ncol = 'clientname'; break;
        case 'ipclient'          : ncol = 'clientname'; break;
        case 'ip6client'         : ncol = 'clientname'; break;
        case 'serveraddress'     : ncol = 'servername'; break;
        case 'ipserver'          : ncol = 'servername'; break;
        case 'ip6server'         : ncol = 'servername'; break;
	case 'sourceaddress'     : ncol = 'sourcename'; break;
        case 'ipsource'          : ncol = 'sourcename'; break;
        case 'ip6source'         : ncol = 'sourcename'; break;
        case 'destinationaddress': ncol = 'destinationname'; break;
        case 'ipdestination'     : ncol = 'destinationname'; break;
        case 'ip6destination'    : ncol = 'destinationname'; break;
      }
      if(ncol) {
	// This should perhaps use last(), but first() is more efficient.
        var namecol = 'first(' + ncol + ')';
        var idx = nameToIdx[namecol];
        if(!idx) {
          idx = cols.length + nameCols.length;
          nameCols.push(namecol);
          nameToIdx[namecol] = idx;
        }
        lookupCols[i] = idx;
      }
    }
  }

  function appendNameColumns(select) {
     if(nameCols.length > 0) return select + "," + nameCols;
     return select;
  }

  function initializeNameMap(t) {
    var addrs = {};
    for(var c = 1; c < t.ncols; c++) {
      switch(t.ctypes[c]) {
      case 'address':
         var namecol = lookupCols[c];
         for(var r = 0; r < t.nrows - nameCols.length; r++) {
           var addr = t.cell(r,c);
           var name;
           if(addr) {
             if(namecol) name = t.cell(r,namecol);
             if(name) {
                if(!addressMap) addressMap = {};
                addressMap[addr] = network.truncateDNS(name);
             }
             else {
               var atype = network.addressType(addr);
               if('ipv4' == atype || 'ipv6' == atype) addrs[addr] = addr;
             }
           }
         }
         break;
      case 'string':
         var cname = t.cnames[c];
         if(cname.endsWith('.macsourcevendor') || cname.endsWith('.macdestinationvendor')) {
           for(var r = 0; r < t.nrows - nameCols.length; r++) {
             var vendorcode = t.cell(r,c);
             if(vendorcode) {
               if(!vendorMap) vendorMap = {};
               vendorMap[vendorcode] = vendorcode;
             }
           }
         }
         break;     
      }     
    }

    if(vendorMap) {
      var vendorlist = [];
      for(var vendor in vendorMap) vendorlist.push(vendor);
      var vendors = network.vendorMap(vendorlist);
      for(var i = 0; i < vendorlist.length; i++) vendorMap[vendorlist[i]] = vendors[i];
    }

    var addrlist = [];
    for (var addr in addrs) addrlist.push(addr);
    if(addrlist.length > 0) {
      var names = network.dnsMap(addrlist);
      for(var i = 0; i < addrlist.length; i++) {
         var namelist = names[i];
         if(namelist && namelist.length == 1) {
           if(!addressMap) addressMap = {};
           addressMap[addrlist[i]] = namelist[0];
         }
      }
    }
    
    // delete the extra name cols
    for(var i = 0; i < nameCols.length; i++) {
      table.deleteColumn(table.ncols - 1);
    }
  }

  function addressName(addr) {
    var label = null;
    if(addressMap && addr) {
       var name = addressMap[addr];
       if(name && name != addr) label = name + " (" + addr + ")";
    }
    return label;
  }

  function vendorName(vendorcode) {
    var label = null;
    if(vendorMap && vendorcode) {
       var name = vendorMap[vendorcode];
       if(name && name != vendorcode) label = name + " (" + vendorcode + ")";
    }
    return label;
  }

  function dscpName(dscpcode) {
    var label = null;
    var name = network.dscpName(dscpcode);
    if(name) label = name + " (" + dscpcode + ")";
    return label;
  }

  // add name columns to the query (if needed)

  var refresh = "true" == app.queryParameter("refresh","false");

  var scale = spec.scale ? spec.scale : 1.0;

  nameColumns(spec.resultField);
  var query = Query.topN(
    spec.view,
    appendNameColumns(spec.resultField),
    spec.resultFilter,
    (refresh && spec.refreshInterval) ? spec.refreshInterval : spec.interval,
    spec.resultSort,
    spec.resultTruncate);

  query.group = spec.intervalMinutes ? spec.intervalMinutes : "minute";

  var table = query.run();
  if(!table || table.nrows == 0) return;

  initializeNameMap(table);

  var data = [];
  var result = {data:data};

  var prevTime;
  var timeStr;
  var row;
  var topn;
  for(var r = 0; r < table.nrows; r++) {
      var time = table.cell(r,0).getTime();
      if(time != prevTime) {
	  timeStr = formatdate(table.cell(r,0),"HH:mm");
	  topn = [];
	  row = [time,timeStr,topn];
	  data.push(row);
	  prevTime = time;
      }

      // keys
      var entry = [];
      for(var c = 1; c < nkeys; c++) {
	  var ctype = table.ctypes[c];
	  var val = table.cell(r,c);
	  if(typeof val != undefined && val != null) {
	      var name = null; 
	      switch(ctype) {
	      case 'port': name = protocolName(val); break;
	      case 'address': name = addressName(val); break;
	      case 'interface': name = interfaceName(val); break;
	      case 'string': 
		  var cname = table.cnames[c];
		  if(cname.endsWith('.macsourcevendor') || cname.endsWith('.macdestinationvendor')) name = vendorName(val);
		  else if(cname.endsWith('.ipdscp') || cname.endsWith('.ip6dscp')) name = dscpName(val);
		  else if(cname.endsWith('.wificipher')) name = cipherName(val);
		  else if(cname.endsWith('.scsiop')) name = scsiOpName(val);
		  break;
	      }
	      var key = [val];
	      if(name && name != val) key.push(name);
	      entry.push(key);
	  } else entry.push(val);
      };
      // values
      for(var c = nkeys; c < table.ncols; c++) {
	  entry.push(table.cell(r,c) * scale);
      }
      topn.push(entry);
  }

  println(JSON.stringify(result));
}

function updateHistory(ctype,cvalue) {
   if('address' == ctype) app.addressToHistory(cvalue);
   else if('port' == ctype) app.protocolToHistory(cvalue);
   else if('agent' == ctype) app.protocolToHistory(cvalue);
}

function action(evtName,evtId,evtValue) {
  switch(evtName) {
    case "nodeClick":
       var response = {};
       if(evtId > 0) { 
         var names = spec.resultFieldNames.split(",");
         var types = columnTypes();
         var link = types ? app.linkForTypeAndValue(types[evtId],evtValue) : null;
         response.action = "dialog";
         response.selectFlag = true;
         response.excludeFlag = true;
         response.searchFlag = link ? true : false;
         response.message = '<table class="overview"><tbody>';
         response.message += '<tr class="even"><th>' + names[evtId - 1] + '</th><td>' + evtValue + '</td></tr>';
         // add extra information here
         response.message += '</tbody></table>';
       } else {
         // path dialog
         var names = spec.resultFieldNames.split(",");
         var values = evtValue.split(",");
         response.action = "dialog";
         response.selectFlag = true;
         response.excludeFlag = true;
         response.searchFlag = spec.searchPath ? true : false;
         response.message = '<table class="overview"><tbody>';
         for(var i = 0; i < values.length; i++) {
            response.message += '<tr class="' + (i%2 == 0 ? 'even' : 'odd') + '"><th>' + names[i] + '</th><td>' + values[i] + '</td></tr>';
         }
         response.message += '</tbody></table>';
       }
       println(JSON.stringify(response,null,2));
       break;
    case "select":
       var response = {};
       response.action = "loadpage";
       var filter = app.getValue("com.inmon.servlets.monitor.filter.rttraffic","");
       if(evtId > 0) {
          var types = columnTypes();
          if(types) updateHistory(types[evtId],evtValue);
          var cname = spec.resultField.split(",")[evtId];
          if(filter) filter = "(" + filter + ") & " + cname + " = '" + evtValue + "'"; 
          else filter = cname + " = '" + evtValue + "'";
       } else {
          // select all keys         
          var names = spec.resultField.split(",");
          var values = evtValue.split(","); 
          var clause = null;
          for(var i = 0; i < values.length; i++) {
             if(clause) clause += " & " + names[i + 1] + " = '" + values[i] + "'";
             else clause = names[i + 1] + " = '" + values[i] + "'";
          }
          if(filter) filter = "(" + filter + ") & " + clause; 
          else filter = clause;
       }
       app.setValue("com.inmon.servlets.monitor.filter.rttraffic",simplifyFilter(filter));
       response.url = app.encodeRedirectURL(app.createURL({}));
       println(JSON.stringify(response,null,2));
       break;
    case "exclude":
       var response = {};
       response.action = "loadpage";
       var filter = app.getValue("com.inmon.servlets.monitor.filter.rttraffic","");
       if(evtId > 0)  {
          var cname = spec.resultField.split(",")[evtId];
          if(filter) filter = "(" + filter + ") & " + cname + " != '" + evtValue + "'";
          else filter = cname + " != '" + evtValue + "'";
       } else {
          // exclude all keys      
          var names = spec.resultField.split(",");
          var values = evtValue.split(","); 
          var clause = null;
          for(var i = 0; i < values.length; i++) {
             if(clause) clause += " & " + names[i + 1] + " = '" + values[i] + "'";
             else clause = names[i + 1] + " = '" + values[i] + "'";
          }
          if(filter) filter = "(" + filter + ") & !(" + clause + ")"; 
          else filter = "!(" + clause + ")";
       }
       app.setValue("com.inmon.servlets.monitor.filter.rttraffic",simplifyFilter(filter));
       response.url = app.encodeRedirectURL(app.createURL({}));
       println(JSON.stringify(response,null,2));
       break;
    case "search":
       var response = {};
       response.action = "loadpage";
       if(evtId > 0) { 
          var types = columnTypes();
          var link = types ? app.linkForTypeAndValue(types[evtId],evtValue) : null;
          response.url = app.encodeRedirectURL(link);
       } else {
          var keys = evtValue.split(",");
	  var q = new Query();
	  q.view = spec.view;
          q.interval = spec.interval;
          var [start,end] = q.timeRange();
	  var startTimeStr = formatdate(start,"yyyyMMddHHmm");
	  var endTimeStr = formatdate(end,"yyyyMMddHHmm");
          response.url = app.encodeRedirectURL("Go?action=path&source=" + keys[spec.searchPath[0]] + "&destination=" + keys[spec.searchPath[1]] + "&startTime=" + startTimeStr + "&endTime="+ endTimeStr);
       }
       println(JSON.stringify(response,null,2));
       break;
    default: data();
  }
}

app.trend(null, action, {
   "cnames"       : spec.resultFieldNames,
   "bidirectional": spec.bidirectional,
   "colors"       : spec.colors,
   "darkColors"   : spec.darkColors,
   "height"       : spec.chartHeight,
   "select"       : app.localize("appletmenu.filter"),
   "exclude"      : app.localize("appletmenu.exclude"),
   "search"       : app.localize("appletmenu.search"),
   "valueAxisLabel": spec.valueAxisLabel,
   "integer"      : spec.integerValues,
   "intervals"    : spec.intervals,
   "refresh"      : spec.refreshInterval ? true : false,
   "truncate"     : spec.resultTruncate,
   "dataTimeout"  : preference("ajax.timeout") ? preference("ajax.timeout") : 600000
});

