// thresholdtype filter
function thresholdTypeFilter(thresholdtype) {
  var filter;
  switch(quota.thresholdtype) {
    case "offsite"     : filter = "destinationzone=EXTERNAL"; break;
    case "outsidezone" : filter = "interzone=1";              break;
    case "outsidegroup": filter = "intergroup=1";             break;
    default            : filter = "";
  }
  return filter;
}

// callback function for accumulating totals
function getCallback(quota) { 
  return function(row,table) { 
    if(row[0]) {
      if(!quota.totals[row[0]]) quota.totals[row[0]] = row[1] * 8;
      else quota.totals[row[0]] += row[1] * 8;
    }
  }; 
}

var vcol = 'rate(bytes)';
var truncate = 1000;
var selects   = [];
var filters   = [];
var intervals = [];
var callbacks = [];
var query = Query.topN('rttraffic',selects,filters,intervals,vcol,truncate);
query.multiquery = true;
for each (var quota in quotas) {
   // attach an accumulator to the quota
   quota.totals = {};

   // sent traffic
   selects.push("ipsource," + vcol);
   intervals.push(quota.interval);
   var filter = thresholdTypeFilter(quota.thresholdtype);
   filter += (filter ? "&" : "") + "sourcepath='" + quota.path + "'";
   if(quota.where) filter += "&(" + quota.where + ")";
   filters.push(filter);
   callbacks.push(getCallback(quota));

   // received traffic        
   selects.push("ipdestination," + vcol);
   filter = thresholdTypeFilter(quota.thresholdtype);
   filter += (filter ? "&" : "") + "destinationpath='" + quota.path + "'";
   if(quota.where) filter += "&(" + quota.where + ")";
   filters.push(filter);
   callbacks.push(getCallback(quota));
}

if(quotas.length > 0) {
    query.run(callbacks);
}

// we have data, now update controls

var network = Network.current();

// get existing controls
var existing = {};
var active = network.controlList(true);
for(var r = 0; r < active.nrows; r++) existing[active.cell(r,3)] = active.cell(r,0);

var add_host = [];
var add_controltype = [];
for each (quota in quotas) {
  for (var host in quota.totals) {
    if(quota.totals[host] >= quota.threshold) {
      // over quota, add control if we haven't already
      if(!existing[host]) {
        add_host.push(host);
        add_controltype.push(quota.controltype);
      }
      else delete existing[host];
    }
    else {
      // underquota, should I still keep based on hysteresis?
      if(quota.totals[host] >= (quota.threshold * (quota.hysteresis / 100))) {
         // keep control
         delete existing[host];
      }
    }
  }
}

// locate addresses and apply new controls
// Note: we only apply controls if we can locate address
var changed = false;
var locations = network.locationMap(add_host);
for (var i = 0; i < locations.length; i++) {
  network.path = locations[i];
  if(network.isInterface()) {
    changed |= network.controlAdd(add_controltype[i],add_host[i]);
  }
}

// remove controls that no longer apply
for (var addr in existing) {
  changed = true;
  network.controlRemove(existing[addr]);
}

// commit any changes
if(changed) network.controlApply(true);


